//***************************************************************************
// CMG_LLIO_AVR_8D4CTL_6800.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.02.2007
//***************************************************************************
// History:
//   05.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_LLIO_CONFIGURED
    #error CMG_LLIO not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_LLIO_AVR_8D4CTL_6800 )

    #if !defined( LLIO_DATA_PORT ) || !defined( LLIO_DATA_PIN ) || !defined( LLIO_DATA_PORT_DDR ) || \
      !defined( LLIO_CTRL_iCS_PORT ) || !defined( LLIO_CTRL_iCS_PORT_DDR ) || !defined( LLIO_CTRL_iCS_BIT ) || \
      !defined( LLIO_CTRL_A0_PORT ) || !defined( LLIO_CTRL_A0_PORT_DDR ) || !defined( LLIO_CTRL_A0_BIT ) || \
      !defined( LLIO_CTRL_RiW_PORT ) || !defined( LLIO_CTRL_RiW_PORT_DDR ) || !defined( LLIO_CTRL_RiW_BIT ) || \
      !defined( LLIO_CTRL_E_PORT ) || !defined( LLIO_CTRL_E_PORT_DDR ) || !defined( LLIO_CTRL_E_BIT )
        #error LLIO_DATA_* and/or LLIO_CTRL_* are not defined. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_LLIO.h"
#include    "CMG_LLIO_WaitCycles.h"
#include    "../MISC/CMG_PlatformDependent.h"
// other:
#include    <avr/io.h>
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------

// macros for ctrl interaction
#define     SET_CTRL_CS_SELECT          CLR_BIT( LLIO_CTRL_iCS_PORT, LLIO_CTRL_iCS_BIT )
#define     SET_CTRL_CS_DESELECT        SET_BIT( LLIO_CTRL_iCS_PORT, LLIO_CTRL_iCS_BIT )

#define     SET_CTRL_A0_0               CLR_BIT( LLIO_CTRL_A0_PORT, LLIO_CTRL_A0_BIT )
#define     SET_CTRL_A0_1               SET_BIT( LLIO_CTRL_A0_PORT, LLIO_CTRL_A0_BIT )

#define     SET_CTRL_RiW_WRITE          CLR_BIT( LLIO_CTRL_RiW_PORT, LLIO_CTRL_RiW_BIT )
#define     SET_CTRL_RiW_READ           SET_BIT( LLIO_CTRL_RiW_PORT, LLIO_CTRL_RiW_BIT )

#define     SET_CTRL_E_DISABLE          CLR_BIT( LLIO_CTRL_E_PORT, LLIO_CTRL_E_BIT )
#define     SET_CTRL_E_ENABLE           SET_BIT( LLIO_CTRL_E_PORT, LLIO_CTRL_E_BIT )


// macros for data interaction
#define     SET_DATA_PORT_DIRECTION_IN  LLIO_DATA_PORT_DDR = 0x00
#define     SET_DATA_PORT_DIRECTION_OUT LLIO_DATA_PORT_DDR = 0xff

#define     SET_DATA_PORT(data)         LLIO_DATA_PORT = ( data )
#define     GET_DATA_PORT()             LLIO_DATA_PIN


// macros for power interaction
// display power
#ifdef LLIO_DISPLAYPOWER_ENABLE
    #ifndef LLIO_DISPLAYPOWER_INVERTED
        // standard power logic
        #define     SET_DISPLAYPOWER_ON         SET_BIT( LLIO_DISPLAYPOWER_PORT, LLIO_DISPLAYPOWER_BIT )
        #define     SET_DISPLAYPOWER_OFF        CLR_BIT( LLIO_DISPLAYPOWER_PORT, LLIO_DISPLAYPOWER_BIT )
    #else
        // inverted power logic
        #define     SET_DISPLAYPOWER_ON         CLR_BIT( LLIO_DISPLAYPOWER_PORT, LLIO_DISPLAYPOWER_BIT )
        #define     SET_DISPLAYPOWER_OFF        SET_BIT( LLIO_DISPLAYPOWER_PORT, LLIO_DISPLAYPOWER_BIT )
    #endif
#endif

// backlight
#ifdef LLIO_BACKLIGHT_ENABLE
    #ifndef LLIO_BACKLIGHT_INVERTED
        // standard backlight logic
        #define     SET_BACKLIGHT_ON            SET_BIT( LLIO_BACKLIGHT_PORT, LLIO_BACKLIGHT_BIT )
        #define     SET_BACKLIGHT_OFF           CLR_BIT( LLIO_BACKLIGHT_PORT, LLIO_BACKLIGHT_BIT )
    #else
        // inverted backlight logic
        #define     SET_BACKLIGHT_ON            CLR_BIT( LLIO_BACKLIGHT_PORT, LLIO_BACKLIGHT_BIT )
        #define     SET_BACKLIGHT_OFF           SET_BIT( LLIO_BACKLIGHT_PORT, LLIO_BACKLIGHT_BIT )
    #endif
#endif

//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------



/****************************************************************************
| CMG_LLIO_Init
|-----------------------------------
| Init  (init downwards: nothing)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_LLIO_Init( void )
{
    // switch on display power if available
    #ifdef LLIO_DISPLAYPOWER_ENABLE
        // switch on
        SET_DISPLAYPOWER_ON;

        // set ctrl pins direction out
        SET_BIT( LLIO_DISPLAYPOWER_PORT_DDR, LLIO_DISPLAYPOWER_BIT );
    #endif
    
    // set data port std direction (all in) and std value
    SET_DATA_PORT( 0x00 );
    SET_DATA_PORT_DIRECTION_IN;
    
    // set ctrl pins std value
    SET_CTRL_CS_DESELECT;
    SET_CTRL_A0_0;
    SET_CTRL_RiW_WRITE;
    SET_CTRL_E_DISABLE;

    // set ctrl pins direction (all out)
    SET_BIT( LLIO_CTRL_iCS_PORT_DDR,    LLIO_CTRL_iCS_BIT );
    SET_BIT( LLIO_CTRL_A0_PORT_DDR,     LLIO_CTRL_A0_BIT );
    SET_BIT( LLIO_CTRL_RiW_PORT_DDR,    LLIO_CTRL_RiW_BIT );
    SET_BIT( LLIO_CTRL_E_PORT_DDR,      LLIO_CTRL_E_BIT );

    // wait until display is ready
    CMG_Sleep_ms( LLIO_DISPLAY_INIT_WAITTIME_MS );
    
    // select chip
    SET_CTRL_CS_SELECT;

    // enable backlight if available
    #ifdef LLIO_BACKLIGHT_ENABLE
        // set initial power state
        #ifdef LLIO_BACKLIGHT_INITIALLY_ON
            SET_BACKLIGHT_ON;
        #else
            SET_BACKLIGHT_OFF;
        #endif
        
        // set ctrl pins direction out
        SET_BIT( LLIO_BACKLIGHT_PORT_DDR, LLIO_BACKLIGHT_BIT );
    #endif

    return  CMG_OK;
}



/****************************************************************************
| CMG_LLIO_Exit
|-----------------------------------
| Exit (exit downwards: nothing)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_Exit( void )
{
    // switch off backlight if available
    #ifdef LLIO_BACKLIGHT_ENABLE
        SET_BACKLIGHT_OFF;
    #endif

    // deselect chip
    SET_CTRL_CS_DESELECT;

    // set data port std direction (all in) and std value
    SET_DATA_PORT_DIRECTION_IN;

    // reset ctrl pins direction (all in)
    CLR_BIT( LLIO_CTRL_iCS_PORT_DDR,    LLIO_CTRL_iCS_BIT );
    CLR_BIT( LLIO_CTRL_A0_PORT_DDR,     LLIO_CTRL_A0_BIT );
    CLR_BIT( LLIO_CTRL_RiW_PORT_DDR,    LLIO_CTRL_RiW_BIT );
    CLR_BIT( LLIO_CTRL_E_PORT_DDR,      LLIO_CTRL_E_BIT );

    // switch off display power if available
    #ifdef LLIO_DISPLAYPOWER_ENABLE
        SET_DISPLAYPOWER_OFF;
    #endif
    
    // wait until display is off
    CMG_Sleep_ms( LLIO_DISPLAY_INIT_WAITTIME_MS );
}



/****************************************************************************
| CMG_LLIO_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
        // display on
        case CMG_PWR_DISPLAY_ON:
            // ... nothing to do ...
            break;
        
        
    // if backlight is available
    #ifdef LLIO_BACKLIGHT_ENABLE

        // backlight on
        case CMG_PWR_BACKLIGHT_ON:
            SET_BACKLIGHT_ON;
            break;
        
        // backlight off
        case CMG_PWR_BACKLIGHT_OFF:
            SET_BACKLIGHT_OFF;
            break;

    #endif
    }
}




/****************************************************************************
| CMG_LLIO_WriteA0
|-----------------------------------
| 
+----------------------------------------------------------------------------
| @Params:          command/data byte
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_WriteA0( cmg_u8 byData )
{
    // set data port direction (all out)
    SET_DATA_PORT_DIRECTION_OUT;

    // set data port
    SET_DATA_PORT( byData );
    
    // ctrl pin sequence
    SET_CTRL_A0_0;
    SET_CTRL_E_ENABLE;
    
    // insert wait cycles
    LLIO_Write_WaitCycles();
    
    SET_CTRL_E_DISABLE;
    
    // set data port std direction (all in)
    SET_DATA_PORT_DIRECTION_IN;
}



/****************************************************************************
| CMG_LLIO_WriteA1
|-----------------------------------
| 
+----------------------------------------------------------------------------
| @Params:          data byte
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_WriteA1( cmg_u8 byData )
{
    // set data port direction (all out)
    SET_DATA_PORT_DIRECTION_OUT;

    // set data port
    SET_DATA_PORT( byData );
    
    // ctrl pin sequence
    SET_CTRL_A0_1;
    SET_CTRL_E_ENABLE;
    
    // insert wait cycles
    LLIO_Write_WaitCycles();
    
    SET_CTRL_E_DISABLE;
    
    // set data port std direction (all in)
    SET_DATA_PORT_DIRECTION_IN;
}



/****************************************************************************
| CMG_LLIO_ReadA0
|-----------------------------------
| 
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+***************************************************************************/
_PUBLIC
cmg_u8 CMG_LLIO_ReadA0( void )
{
    cmg_u8  byData;
    
    // ctrl pin sequence
    SET_CTRL_RiW_READ;
    SET_CTRL_A0_0;
    SET_CTRL_E_ENABLE;
    
    // insert wait cycles
    LLIO_Read_WaitCycles();
    // one is _always_ necessary
    asm volatile ("nop");

    // get data port (std data port direction is in)
    byData = GET_DATA_PORT();
    
    // ctrl pin sequence
    SET_CTRL_E_DISABLE;
    SET_CTRL_RiW_WRITE;

    return byData;
}



/****************************************************************************
| CMG_LLIO_ReadA1
|-----------------------------------
| 
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+***************************************************************************/
_PUBLIC
cmg_u8 CMG_LLIO_ReadA1( void )
{
    cmg_u8  byData;
    
    // ctrl pin sequence
    SET_CTRL_RiW_READ;
    SET_CTRL_A0_1;
    SET_CTRL_E_ENABLE;
    
    // insert wait cycles
    LLIO_Read_WaitCycles();
    // one is _always_ necessary
    asm volatile ("nop");

    // get data port (std data port direction is in)
    byData = GET_DATA_PORT();

    // ctrl pin sequence
    SET_CTRL_E_DISABLE;
    SET_CTRL_RiW_WRITE;

    return byData;
}
//---------------------------------------------------------------------------




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
