//***************************************************************************
// CMG_LLIO_EMU_DISPLAY_COMMON.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.11.2006
//***************************************************************************
// History:
//   07.11.2006     *stable*
//   10.11.2006     added _PixelColor support
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_LLIO_CONFIGURED
    #error CMG_LLIO not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_LLIO_EMU_DISPLAY_GDI ) ||           \
      defined( CMG_LLIO_EMU_DISPLAY_GTK )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_LLIO_EMU_DISPLAY_COMMON.h"
// other:
#include    <malloc.h>
#include    <stdlib.h>
#include    <time.h>
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
// subtile sizes
#define     SUBTILE1_WIDTH          48
#define     SUBTILE1_HEIGHT         26
#define     SUBTILE2_WIDTH          25
#define     SUBTILE2_HEIGHT         22

// tile sizes
#define     TILE_WIDTH              48
#define     TILE_HEIGHT             48

// tile background color (pal)
#define     TILE_BACKGROUNDCOLOR    223

// return image sizes
#define     IMAGE_MIN_WIDTH         ( TILE_WIDTH * 2 )
#define     IMAGE_MIN_HEIGHT        ( TILE_HEIGHT * 2 )
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// forward declarations (at the end of this file)
//---------------------------------------------------------------------------
cmg_u8      g_encPal[], g_encSubTile1[], g_encSubTile2[];
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// decoding macros for converted data (4byte ascii --> 3byte data)
//---------------------------------------------------------------------------
#define     D6(e)           (((e)<97)?((e)-48):((e)-54))
#define     CM_DEC24(d,e)   {(d)[0]=(D6((e)[0])<<2)|(D6((e)[1])>>4);(d)[1]=(D6((e)[1])<<4)|(D6((e)[2])>>2);(d)[2]=(D6((e)[2])<<6)|(D6((e)[3]));}
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
cmg_mu8     g_DISPLAY_COMMON_uByteOrder = CMG_LLIO_DISPLAY_COMMON_BYTEORDER_GDI;

// predefined pixel mix maps: 4 (2x2) subpixels starting from top left consisting of 3 colors (r, g, b) with each one an min and max value (color==0, color==255)
cmg_Color   g_DISPLAY_COMMON_aaPixMixMap_PhotoBlOn [4][6] = { { 210,  91, 205,  90, 200,  89 }, { 205, 129, 200, 127, 195, 122 }, { 195,  62, 190,  59, 185,  56 }, { 200, 108, 195, 106, 190, 102 } };
cmg_Color   g_DISPLAY_COMMON_aaPixMixMap_PhotoBlOff[4][6] = { { 180,  95, 185,  84, 150,  34 }, { 206,  68, 188,  51, 154,  20 }, { 204, 120, 194, 111, 151,  55 }, { 203,  84, 183,  70, 151,  34 } };
cmg_Color   g_DISPLAY_COMMON_aaPixMixMap_Exact     [4][6] = { { 255,   0, 255,   0, 255,   0 }, { 255,   0, 255,   0, 255,   0 }, { 255,   0, 255,   0, 255,   0 }, { 255,   0, 255,   0, 255,   0 } };
//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
// private functions
//---------------------------------------------------------------------------


/*---------------------------------------------------------------------------
| _DISPLAY_COMMON_CreateRGB
|-----------------------------------
| creates a 32 bit color from the three components according the selected
| byte order
+----------------------------------------------------------------------------
| @Params:          
|   * byRed:        red
|   * byGreen:      green
|   * byBlue:       blue
| @Return Value:    32 bit color value
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u32 _DISPLAY_COMMON_CreateRGB( cmg_u8 byRed, cmg_u8 byGreen, cmg_u8 byBlue )
{
    if ( g_DISPLAY_COMMON_uByteOrder == CMG_LLIO_DISPLAY_COMMON_BYTEORDER_GDI )
        return ( byRed << 16 ) | ( byGreen << 8 ) | byBlue;
    else
        return ( byBlue << 16 ) | ( byGreen << 8 ) | byRed;
}

//---------------------------------------------------------------------------






//---------------------------------------------------------------------------
// exported functions
//---------------------------------------------------------------------------



/****************************************************************************
| CMG_LLIO_DISPLAY_COMMON_SetByteOrder
|-----------------------------------
| set byte order of 32 bit colored pixels since GDI and GTK use a 
| different model.
+----------------------------------------------------------------------------
| @Params:          
|   * uByteOrder:   byte order
| @Return Value:    result
+***************************************************************************/
_PUBLIC
void CMG_LLIO_DISPLAY_COMMON_SetByteOrder( cmg_mu8 uByteOrder )
{
    g_DISPLAY_COMMON_uByteOrder = uByteOrder;
}





/****************************************************************************
| CMG_LLIO_DISPLAY_COMMON_CreateLCDImage8
|-----------------------------------
| Create an 8 bpp lcd background photo image
| NOTE: pImage and pPal have to be freed by caller
+----------------------------------------------------------------------------
| @Params:          
|   * ppImage:      ptr to ptr to the image (function will alloc buffer)
|   * ppPal:        ptr to ptr to the palette (256 entries) (func. will alloc buffer)
|   * iWidth:       width of image to create
|   * iHeight:      height of image to create
| @Return Value:    result
+***************************************************************************/
    #define     TILE1_VCOPY(X,Y,tileX)                                                              \
        {                                                                                           \
            cmg_mu32    uC;                                                                         \
            for ( uC = 0; uC < SUBTILE1_HEIGHT; uC++ )                                              \
                pImage8[ ( (Y) + uC ) * iWidth + (X) ] = pSubTile1[ uC * SUBTILE1_WIDTH + (tileX) ];\
        }

    #define     TILE1_VRCOPY(X,Y,tileX)                                                             \
        {                                                                                           \
            cmg_mu32    uC;                                                                         \
            for ( uC = 0; uC < SUBTILE1_HEIGHT; uC++ )                                              \
                pImage8[ ( (Y) + SUBTILE1_HEIGHT - 1 - uC ) * iWidth + (X) ] = pSubTile1[ uC * SUBTILE1_WIDTH + (tileX) ];\
        }

    #define     TILE2_HCOPY(X,Y,tileY)                                                              \
        {                                                                                           \
            cmg_mu32    uC;                                                                         \
            for ( uC = 0; uC < SUBTILE2_WIDTH; uC++ )                                               \
                pImage8[ (Y) * iWidth + (X) + uC ] = pSubTile2[ (tileY) * SUBTILE2_WIDTH + uC ];    \
        }

    #define     TILE2_HRCOPY(X,Y,tileY)                                                             \
        {                                                                                           \
            cmg_mu32    uC;                                                                         \
            for ( uC = 0; uC < SUBTILE2_WIDTH; uC++ )                                               \
                pImage8[ (Y) * iWidth + (X) + SUBTILE2_WIDTH - 1 - uC ] = pSubTile2[ (tileY) * SUBTILE2_WIDTH + uC ];\
        }

_PUBLIC
cmg_Result CMG_LLIO_DISPLAY_COMMON_CreateLCDImage8( cmg_u8 **ppbyImage, cmg_PaletteColor **ppPal, cmg_Coord iWidth, cmg_Coord iHeight )
{
    cmg_Result          iResult;

    cmg_u8              *pImage8 = NULL;
    cmg_mu32            uImage8_Size;
    cmg_PaletteColor    *pPal = NULL;

    cmg_u8              *pSubTile1 = NULL;
    cmg_u8              *pSubTile2 = NULL;

    cmg_ms32            iIdx;

    
    
    // . . . . . . . . . . . . . . . . . . . . . . .
    // init stuff
    // parameter check
    if ( ( ppbyImage == NULL ) | ( ppPal == NULL ) | ( iWidth < IMAGE_MIN_WIDTH ) | ( iHeight < IMAGE_MIN_HEIGHT ) )
        return CMG_ERROR_PARAM;

    // reset image and pal ptr
    *ppbyImage = NULL;
    *ppPal = NULL;



    // . . . . . . . . . . . . . . . . . . . . . . .
    // allocations
    // allocate image memory (per pixel 8 bit)
    uImage8_Size = iWidth * iHeight;
    pImage8 = malloc( uImage8_Size );
    CHECK( pImage8 == NULL, CMG_ERROR_NOMEM );

    // allocate pal memory (256 * 32 bit)
    pPal = malloc( 256 * sizeof(cmg_PaletteColor) );
    CHECK( pPal == NULL, CMG_ERROR_NOMEM );

    // allocate subtile1 memory (8bpp) (+decoding security)
    pSubTile1 = malloc( ( SUBTILE1_WIDTH * SUBTILE1_HEIGHT ) + 2 );
    CHECK( pSubTile1 == NULL, CMG_ERROR_NOMEM );

    // allocate subtile1 memory (8bpp) (+decoding security)
    pSubTile2 = malloc( ( SUBTILE2_WIDTH * SUBTILE2_HEIGHT ) + 2 );
    CHECK( pSubTile2 == NULL, CMG_ERROR_NOMEM );
    
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // loads 
    // load palette
    for ( iIdx = 0; iIdx < 256; iIdx++ )
    {
        CM_DEC24( (cmg_u8*)( &pPal[ iIdx ] ), &g_encPal[ iIdx<<2 ] );
        pPal[ iIdx ].byAlpha = 0;
    }

    // load subtile1
    for ( iIdx = 0; iIdx < ( ( ( SUBTILE1_WIDTH * SUBTILE1_HEIGHT ) - 1 ) / 3 ) + 1; iIdx++ )
        CM_DEC24( &pSubTile1[ iIdx*3 ], &g_encSubTile1[ iIdx<<2 ] );

    // load subtile2
    for ( iIdx = 0; iIdx < ( ( ( SUBTILE2_WIDTH * SUBTILE2_HEIGHT ) - 1 ) / 3 ) + 1; iIdx++ )
        CM_DEC24( &pSubTile2[ iIdx*3 ], &g_encSubTile2[ iIdx<<2 ] );
    


    // . . . . . . . . . . . . . . . . . . . . . . .
    // create image
    // fill with lcd background
    for ( iIdx = 0; iIdx < ( iWidth * iHeight ); iIdx++ )
        pImage8[ iIdx ] = TILE_BACKGROUNDCOLOR;

    // edges
    for ( iIdx = 0; iIdx < SUBTILE1_WIDTH; iIdx++ )
    {
        // top left
        TILE1_VCOPY ( iIdx, 0, iIdx );
        // top right
        TILE1_VCOPY ( iWidth - iIdx - 1, 0, iIdx );
        // bottom left
        TILE1_VRCOPY( iIdx, iHeight - SUBTILE1_HEIGHT, iIdx );
        // bottom right
        TILE1_VRCOPY( iWidth - iIdx - 1, iHeight - SUBTILE1_HEIGHT, iIdx );
    }

    // inner edges
    for ( iIdx = 0; iIdx < SUBTILE2_HEIGHT; iIdx++ )
    {
        // top left
        TILE2_HCOPY ( 0, iIdx + SUBTILE1_HEIGHT, iIdx );
        // top right
        TILE2_HCOPY ( 0, iHeight - iIdx - SUBTILE1_HEIGHT, iIdx );
        // bottom left
        TILE2_HRCOPY ( iWidth - SUBTILE2_WIDTH, iIdx + SUBTILE1_HEIGHT, iIdx );
        // bottom right
        TILE2_HRCOPY ( iWidth - SUBTILE2_WIDTH, iHeight - iIdx - SUBTILE1_HEIGHT, iIdx );
    }

    // top/bottom borders
    srand( (unsigned int) time( NULL ) );
    for ( iIdx = SUBTILE1_WIDTH; iIdx < ( iWidth - SUBTILE1_WIDTH ); iIdx++ )
    {
        // top border
        TILE1_VCOPY ( iIdx, 0, ( rand() % 18 ) + 30 );
        // bottom border
        TILE1_VRCOPY( iIdx, iHeight - SUBTILE1_HEIGHT, ( rand() % 18 ) + 30 );
    }

    // left/right borders
    for ( iIdx = SUBTILE1_HEIGHT + SUBTILE2_HEIGHT; iIdx < ( iHeight - ( SUBTILE1_HEIGHT + SUBTILE2_HEIGHT - 1) ); iIdx++ )
    {
        // left border
        TILE2_HCOPY ( 0, iIdx, ( rand() % 21 ) + 1 );
        // right border
        TILE2_HRCOPY( iWidth - SUBTILE2_WIDTH, iIdx, ( rand() % 21 ) + 1 );
    }


    // . . . . . . . . . . . . . . . . . . . . . . .
    // finish
    // free no more used mem
    SFREE( pSubTile1 );
    SFREE( pSubTile2 );

    
    // return image and pal ptr
    *ppbyImage = pImage8;
    *ppPal = pPal;

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    SFREE( pImage8 );
    SFREE( pPal );
    SFREE( pSubTile1 );
    SFREE( pSubTile2 );
    return iResult;
}




/****************************************************************************
| CMG_LLIO_DISPLAY_COMMON_CreateLCDImage32
|-----------------------------------
| Create an 32 bpp lcd background photo image
| NOTE: pImage has to be freed by caller
+----------------------------------------------------------------------------
| @Params:          
|   * ppImage:      ptr to ptr to the image (function will alloc buffer)
|   * iWidth:       width of image to create
|   * iHeight:      height of image to create
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_LLIO_DISPLAY_COMMON_CreateLCDImage32( cmg_u32 **ppdwImage, cmg_Coord iWidth, cmg_Coord iHeight )
{
    cmg_Result          iResult;

    cmg_u8              *pImage8 = NULL;
    cmg_PaletteColor    *pPal = NULL;

    cmg_mu32            uImage32_Size;
    cmg_u32             *pImage32 = NULL;

    cmg_ms32            iIdx;


    // . . . . . . . . . . . . . . . . . . . . . . .
    // init stuff
    // parameter check
    if ( ( ppdwImage == NULL ) | ( iWidth < IMAGE_MIN_WIDTH ) | ( iHeight < IMAGE_MIN_HEIGHT ) )
        return CMG_ERROR_PARAM;

    // reset image ptr
    *ppdwImage = NULL;
    


    // . . . . . . . . . . . . . . . . . . . . . . .
    // create image
    // create an 8bpp image first
    iResult = CMG_LLIO_DISPLAY_COMMON_CreateLCDImage8( &pImage8, &pPal, iWidth, iHeight );
    CHECKRESULT();



    // . . . . . . . . . . . . . . . . . . . . . . .
    // conversion
    // allocate image memory (per pixel 32 bit)
    uImage32_Size = iWidth * iHeight * sizeof( cmg_u32 ) ;
    pImage32 = malloc( uImage32_Size );
    CHECK( pImage32 == NULL, CMG_ERROR_NOMEM )


    // convert 8bbp image into an 32bpp image
    for ( iIdx = 0; iIdx < ( iWidth * iHeight ); iIdx++ )
    {
        // get palette entry
        cmg_PaletteColor    *pPalEntry = &pPal[ pImage8[ iIdx ] ];
        // store 32 bit color
        pImage32[ iIdx ] = _DISPLAY_COMMON_CreateRGB( pPalEntry->byRed, pPalEntry->byGreen, pPalEntry->byBlue );
    }



    // . . . . . . . . . . . . . . . . . . . . . . .
    // finish
    // free no more used mem
    SFREE( pImage8 );
    SFREE( pPal );


    // return image ptr
    *ppdwImage = pImage32;

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    SFREE( pImage8 );
    SFREE( pPal );
    SFREE( pImage32 );
    return iResult;
}




/****************************************************************************
| CMG_LLIO_DISPLAY_COMMON_CreatePixelColor_2x2_32bpp
|------------------------------------------
| Create an 2x2 PixelColor table
+----------------------------------------------------------------------------
| @Params:          
|   * aadwPixelColor:   array of 256 colors with a 2x2 matrix of 32bpp 
|                       color entries
|   * iType:            pixel color table type (photo or exact)
| @Return Value:        none
+***************************************************************************/

#define     MIX(cmin,cmax)          ( (cmg_Color) ( ((cmg_mu16)(uCol)) * ( (cmax) - (cmin) ) / 255 + (cmin) ) )

_PUBLIC
void CMG_LLIO_DISPLAY_COMMON_CreatePixelColor_2x2_32bpp( cmg_u32 aadwPixelColor[256][4], cmg_mu8 uType )
{
    cmg_mu16    uIdx, uCol;
    cmg_Color   (*paaPixMixMap)[4][6];

    // param check
    if ( aadwPixelColor == NULL )
        return;

    
    // select type
    switch ( uType )
    {
        // photo type backlight on
        case CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_PHOTO_BLON:
            paaPixMixMap = &g_DISPLAY_COMMON_aaPixMixMap_PhotoBlOn;
            break;

        // photo type backlight off
        case CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_PHOTO_BLOFF:
            paaPixMixMap = &g_DISPLAY_COMMON_aaPixMixMap_PhotoBlOff;
            break;

        // exact type
        case CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_EXACT:
            paaPixMixMap = &g_DISPLAY_COMMON_aaPixMixMap_Exact;
            break;
    }


    // create pixelmap

    // for each color
    for ( uCol = 0; uCol < 256; uCol++ )
        // for each subpixel (2x2)
        for ( uIdx = 0; uIdx < 4; uIdx++ )
            aadwPixelColor[ uCol ][ uIdx ] = _DISPLAY_COMMON_CreateRGB( MIX( (*paaPixMixMap)[ uIdx ][0], (*paaPixMixMap)[ uIdx ][1] ), MIX( (*paaPixMixMap)[ uIdx ][2], (*paaPixMixMap)[ uIdx ][3] ), MIX( (*paaPixMixMap)[ uIdx ][4], (*paaPixMixMap)[ uIdx ][5] ) );

}

//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// encoded image data
//---------------------------------------------------------------------------
// Pal:
cmg_u8 g_encPal[] = 
"00L11@L71@L900P000T200T51@T600T91PP81fP;1PT:1PT=0PX=20T>2@T:1fL71fTA1fTC1fXA"
"1fXC1fXB1@bA1fTD1fXD1fbE1fbF1PbG1@bF1ffG1PfE0fbB1fbH1fbJ1@bI1PbM1ffH1ffJ1@jI"
"1ffL1fnN1PjN0PXL20b@20XC20bB20TB20XE20bD20bF2PbE2PbG20TE2@fD20fF2PfG2PjE20bH"
"2@bJ2@fI20fJ2@nK2ffI2PjJ2fnI20jM20jN2@nM20nN2PnM30fI3@nJ3@nM3@bK1@LE2g0N1fnL"
"4@nJ4PbK4Q4M71DL1@XR1fjQ1PnV10XU0Pbb2@nQ2@nU3@nR2PnZ0Pfi0Prq00Hm110a2A0R2Q4V"
"3A4Q3Q0V314a3gjb1A8p2g8l31@p11Pp4g0R6gDS7APa61HZ4Abu8QTT8Qfb9gbV920m<hXk=2jp"
"<2Hd0PcD00YK11E31QE:11U=31g;2QM42gcE4goH1@o<6AoT3B9J32MG4B9;4RAN5C1D3RUS5R=R"
"6RET6hYT7BkZ4RoY7Bcj6RAb6S9c7C5n3h1t8C9i93Ml8BchCD5?O71m4Ql:4RJ=72d>6BdF6RZM"
"5BVI6C6J6hpS7CNU7CRg8CF=8iNW8Cdn<E>A:TFi<jdk=UNd>UVn;4ZV?6Ro=kW7;jm3?Va=A5lJ"
"?5pdBGFt@VNmAVC8A6GJAWG8F7mLDW;CCFiUE7GWEFKQJXJqFX7;FXKFCX7TFHeZJnOGHYGYJ9ek"
"HnGbNZerOaOpKZ?iQWgpSnR9THZ6TXZ>U8d;T8N1UY6>UI29VY>DdKVqQKesQpupeKu3equ1eau3"
"er33er;5fL72fb73fb?3fL75fL?6fb?5gLC4f<?8fLG9frK9hLm?i<u?h<mBhLmBhLqAhLqBhbm@"
"hbqAhbqBhbuBhbuChruBhruCi<q@i<uAiLq@i<uBi<uChc3>hM3Chs3Chc3Ei=7CiM3Ci=;Cic7B"
"ic?BiM7Di=7Ei=;EiM;Eic7Dic;Dic;Gis?H";

// Bitmap1: 48x26x8
cmg_u8 g_encSubTile1[] = 
"BF>DWYFFV9BHVoNGV9BDWYRKUHlCTo>GUoVGSIFDXZROWpRHV9RIVodQVoZJWYtL06:Vf<hueKZr"
"fKa0eqVoeqW0i:>Vc:tieblfaa:de;m0fL2fb;RVYa2rdJVeda2o1BUmR8d3Omp3OWN3PmMpOWIu"
"PUUkN7IlMX1JFV=kMGN3NWIIFVIVG5gkFUYkHkAJ1AbQ:BTY:59@:BU9BB4R8Ab@BCPO<g0@50b9"
"0@490PT20P4?0@890P810@X83fn91@fJ:58W9k1A:24J5SPT8gLB=3Di:Pn80Pj20@410@410P81"
"0@480@X;0@4120r10fDKDE<Q92@X8C<JBATH5g<M4gLB0P410Pj;0P410@420@410@4?0SsO3089"
"2P880fLSDB8K6R0J61bJ6B4K6gbU5g0@3@b;4g<D4g0C4A0@41Hi5Q4A4DTS5g<i<i<i0fLQGRTP"
"6gbG;QXG6R4Q6ATH:Pb251Pn924J8ArO6CPn<gro>3<n7i8i6AHF<S990fLTDU4U7Q<I828Q6QbI"
";S8g3@822fT;2fj;3@T:3A4@40j93@j=3Bj=3A0@4BjZ00Lp9R@U7QDC6B4F6gLd<Brj3@T10PT9"
"3A0a:g8H>20P>A4g<i<h<i8n6QXP>CXH00foD28X:29N9h4K5gLe:gLa:g893A0Q?3TR8R9CDk=@"
"D2U@:E=CD5=CDk=CDk=@0fg@I6ADGE5N9gc96gDe62<n;S<RDU9HIGUoNmd4RXV7QGl5Nml9S8Z9"
"RXV9OGl@00gFHlABGBPQ8B4F6A@D5gXd<U1IQGYDHFcaN7d4QnN?QWl:Q8p?S8N7QHN5OGl>0goB"
"F50Y81XK6gXJ5Qb@6B0f>55JQpYnFEYKFkcrO7=rLmgrO7=iLmAjM7AiLm=j0fgAI6AQEB8R@As9"
"4gLG9B8P@5ILHn68RnhMYJ:hcJpWaZpQXKRTYq>ebqFqXpBW0fMAIFcTEU5@:30A51<G6AfQ:2LW"
"DUgUP86MXp>mcq2Vd:haYKNba:JVXpdlX:2V0ffVDBTX9R0Q8ALC6gPI4gjK8gjH7B@XGE9LFEAI"
"FEAIFEULFUYIG5ADE5cDE5AD00fL8B4V9AjI<1@H6g4g<ALh8gfH63XT9RE:BhDrA5EAGDbWEUjW"
"9hLWGE9BEUIF0ffP:5HP6iDi<gLF4AHn>1rq>1PG>R4V93Xs8h<q9RQEDB<T@B1;GBM;94=A9kE3"
"00fVDE4W6gfO6gbK5QLd<1TS71bJ92A;83TS@E4X9j0W9hPq@3c2?2<r9BMEDB@W00fqDB4O61bG"
"6gHC5gbG5i<P@2HT91rT83fW9hLV>ifo9BDqA2@T83U49hLW@3TT0fgM8QbJ91bK6AHG4g4F6ATW"
"GBLU8B@V93c4@413?3I6A3XpAC@l?TLVBifT>io410g0D55BDAbG5QLG4gLI6QrP924G?4<Q>R<q"
"?4=M@3nrBTLt?ift?3fS?jXrEkM40fgAGE5BDB@G5QI97QTJ6Arq>1rn>j0r?T12GUIBGEMWEkkO"
"Gl5NBTo6Gl1QF5oQ0fg291jr92<G5QnF8gTH6C0e53Do>4Q<CVYeHFYXClYYJ6QXKFk?Cjs?KlsY"
"KFUe00fk>QPl<QLC510F>QTF<30g4C4t>TIWTI:Bfb;2fb;5fr?6gbK6grO4hba8hLO5";

// Bitmap2: 25x22x8
cmg_u8 g_encSubTile2[] = 
"0fbl=QPD:i0H41LI5QHi@B8P<hnoA3U=g@<<?1XM;g@@:i<n7gHF6B0P>34g<TDgCrb07UDO;hrb"
"52bd<gHA5S<d<C@k<S4gCFS?01DS830F52rF;S<F4AnG6ALa5iU5<Dk?k0@<@28J<i0f;QTF4AHA"
"61Td<SU7A4E5K<r56kIA8ALe5hni5QnA4A<G<3Po84A8CFi?01oFER8i:i<i5Q4N6i0G61LM71To"
"ADEbiP<<A5HR838F4A0@41LG;APd5iDp?DE8K=005D118S<G4gM941LM410Z5iHL=SU<CFqA10g0"
"DC<g;A0A44TG6AHB:Rfl>j18C4kdk0@E>hHd;Q<@5ADC414A51PI?SU8ADg=Lc847R@Q8A4D410C"
"4gHK41LH73Hl?iHuC7;J11<O8AXA4ALC4A0A410e<34g7Crk=hYhjPDK82I:4A0A41H@410A4Bnj"
";iE5?jI<L=455APH<Rn@4A59410A5S<g=34l=TI7CG3J10b@42bd5g@@4An@5QLd;SI5ACHuATkh"
"lfDE5gD@5QrPBA0@4A<a5gLg<Sju=jE5LMf57QPK4QHo8QbN410a:g<a=3@lACs6CG7D11np8C<a"
";QQ94A0C5QHF:i@e<TDsAjkfjfDE8gXH53@g5g4A:gLg<i0lAC54?UMGL=H4:SDH62rD;2r@510G"
"<i<OBinh?UEGEm3C12Xt6hbb;2XB51<A:hno824r>CjrEkMglC54";
//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
