//***************************************************************************
// CMG_LLIO_EMU_DISPLAY_GTK.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         26.02.2007
//***************************************************************************
// History:
//   ??.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************


// main CMG include (will include user configuration)
#include    "../../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_LLIO_CONFIGURED
    #error CMG_LLIO not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_LLIO_EMU_DISPLAY_GTK )

    #if !defined( CMG_LLIO_EMU_WIDTH ) || !defined( CMG_LLIO_EMU_HEIGHT ) || !defined( CMG_LLIO_EMU_BITDEPTH )
        #error CMG_LLIO_EMU_* are not defined. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "../CMG_LLIO.h"
#include    "CMG_LLIO_EMU_DISPLAY.h"
#include    "CMG_LLIO_EMU_DISPLAY_COMMON.h"
// other:
#include    <gtk/gtk.h>
#include    <glib.h>
#include    <memory.h>
#include    <string.h>
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
#define     DISPLAY_WINDOW_TITLE            "CMG - LCD Emulator"
#define     DISPLAY_WINDOW_WAITTIME_MS      5000    // wait 5 secs
#define     DISPLAY_WINDOW_TIMER_MS         25      // every 25 msec (max 40fps)

#define     MENU_MODE_PHOTO                 1
#define     MENU_MODE_EXACT                 2
#define     MENU_MODE_PHOTO_TEXT            "Photo Mode"
#define     MENU_MODE_EXACT_TEXT            "Exact Mode"
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// typedefs
//---------------------------------------------------------------------------
// structure to check correct gtk window init
typedef struct _cmg_GTK_WindowInit
{
    GCond       *gtkCondition;
    GMutex      *gtkMutex;
    cmg_bool    bInited;
} cmg_GTK_WindowInit;
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------

cmg_bool        g_DISPLAY_bInitedGTK = false;   // GTK+ inited (initally false)

GThread        *g_DISPLAY_gtkThread = NULL;     // window thread
GtkWidget      *g_DISPLAY_gtkWindow = NULL;     // lcd window handle
GtkWidget      *g_DISPLAY_gtkDrawingArea = NULL;// drawing area
GtkWidget      *g_DISPLAY_gtkMenu = NULL;       // popup menu

cmg_u8         *g_DISPLAY_pbyLCDBuffer = NULL;  // local buffer
cmg_Coord       g_DISPLAY_iWidth;               // emulated lcd width in pixels
cmg_Coord       g_DISPLAY_iHeight;              // emulated lcd height in pixels

cmg_u32        *g_DISPLAY_pdwLCDBackground = NULL; // lcd background bitmap
cmg_Coord       g_DISPLAY_iLCDBackground_Width; // lcd bitmap size in pixels
cmg_Coord       g_DISPLAY_iLCDBackground_Height;// lcd bitmap size in pixels

cmg_u32        *g_DISPLAY_pdwLCD = NULL;        // lcd bitmap
cmg_Coord       g_DISPLAY_iLCD_Width;           // lcd bitmap size in pixels
cmg_Coord       g_DISPLAY_iLCD_Height;          // lcd bitmap size in pixels

cmg_u32         g_DISPLAY_aadwPixelColor[256][4];// graycolor 2x2 pixel values

volatile cmg_bool   g_DISPLAY_bLCD_On;          // lcd mode status
volatile cmg_bool   g_DISPLAY_bModeChanged;     // content changed since last timer
volatile cmg_bool   g_DISPLAY_bContentChanged;  // content changed since last timer
volatile cmg_bool   g_DISPLAY_bBacklightOn;     // backlight status
volatile cmg_bool   g_DISPLAY_bExactMode;       // exact mode status

//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// needed private predefs
//---------------------------------------------------------------------------
_PRIVATE    void    _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer( void );
//---------------------------------------------------------------------------





/*---------------------------------------------------------------------------
| _DISPLAY_ReDraw
|-----------------------------------
| callback for redrawing
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _DISPLAY_ReDraw( void )
{
    // mode changed?
    if ( g_DISPLAY_bModeChanged )
    {
        // redraw lcd background
        gdk_draw_rgb_32_image( g_DISPLAY_gtkDrawingArea->window, g_DISPLAY_gtkDrawingArea->style->fg_gc[ GTK_STATE_NORMAL ], 0, 0, g_DISPLAY_iLCDBackground_Width, g_DISPLAY_iLCDBackground_Height, GDK_RGB_DITHER_MAX, (guchar*) g_DISPLAY_pdwLCDBackground, g_DISPLAY_iLCDBackground_Width * sizeof( cmg_u32 ) );
        
        // reset bModeChanged
        g_DISPLAY_bModeChanged = false;

        // but redraw content too
        g_DISPLAY_bContentChanged = true;
    }

    // content changed?
    if ( g_DISPLAY_bContentChanged )
    {
        // redraw lcd content
        if ( g_DISPLAY_bLCD_On )
            gdk_draw_rgb_32_image( g_DISPLAY_gtkDrawingArea->window, g_DISPLAY_gtkDrawingArea->style->fg_gc[ GTK_STATE_NORMAL ], CMG_LLIO_DISPLAY_COMMON_LCDBG_START_X, CMG_LLIO_DISPLAY_COMMON_LCDBG_START_Y, g_DISPLAY_iLCD_Width, g_DISPLAY_iLCD_Height, GDK_RGB_DITHER_MAX, (guchar*) g_DISPLAY_pdwLCD, g_DISPLAY_iLCD_Width * sizeof( cmg_u32 ) );

        // reset bContentChanged
        g_DISPLAY_bContentChanged = false;
    }
}




/*---------------------------------------------------------------------------
| _DISPLAY_OnExpose
|-----------------------------------
| callback for redrawing
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
gboolean _DISPLAY_OnExpose( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
    // complete redraw
    g_DISPLAY_bModeChanged = true;
    g_DISPLAY_bContentChanged = true;

    _DISPLAY_ReDraw();
    return TRUE;
}




/*---------------------------------------------------------------------------
| _DISPLAY_OnTimer
|-----------------------------------
| callback for timout (timer)
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
gboolean _DISPLAY_OnTimer( gpointer data )
{
    // window already closed? -> exit
    if ( !g_DISPLAY_gtkWindow )
        return FALSE;

    // partial redraw
    _DISPLAY_ReDraw();

    return TRUE;
}




/*---------------------------------------------------------------------------
| _DISPLAY_OnButtonPress_Or_OnMotion
|-----------------------------------
| callback for button press and motion
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
gboolean _DISPLAY_OnButtonPress_Or_OnMotion( GtkWidget *widget, GdkEvent *event, gpointer user_data )
{
    static  gint    s_iClickedX, s_iClickedY;                 // _STATIC_ clicked mouse position
    static  gint    s_iWindowStartX = 0, s_iWindowStartY = 0; // _STATIC_ window position on click


    switch ( event->any.type )
    {

        // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
        // button (mouse button pressed)
        case GDK_BUTTON_PRESS:
        {
            // left mouse?
            if ( event->button.button == 1 )
            {
                // save mouse start position
                s_iClickedX = (gint) event->button.x_root;
                s_iClickedY = (gint) event->button.y_root;
                // save window start position
                gtk_window_get_position( GTK_WINDOW( g_DISPLAY_gtkWindow ), &s_iWindowStartX, &s_iWindowStartY );
            }
            // right mouse?
            else if ( event->button.button == 3 )
            {
                // popup menu
                gtk_menu_popup( GTK_MENU( g_DISPLAY_gtkMenu ), NULL, NULL, NULL, NULL, event->button.button, event->button.time );
            }
            break;
        }



        // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
        // motion (mouse moves)
        case GDK_MOTION_NOTIFY:
        {
            // left mouse pressed?
            if ( event->motion.state & GDK_BUTTON1_MASK )
            {
                gint    iNewX, iNewY;

                // calc new pos
                iNewX = (gint)(event->motion.x_root) - s_iClickedX + s_iWindowStartX;
                iNewY = (gint)(event->motion.y_root) - s_iClickedY + s_iWindowStartY;

                // set window
                gtk_window_move( GTK_WINDOW( g_DISPLAY_gtkWindow ), iNewX, iNewY );
            }
        }
    }
    return TRUE;
}




/*---------------------------------------------------------------------------
| _DISPLAY_OnMenuItem
|-----------------------------------
| popup menu selection
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
|   * user_data:    selected item
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
void _DISPLAY_OnMenuItem ( GtkMenuItem *menuitem, gpointer user_data )
{
    guint   uSelected = GPOINTER_TO_UINT( user_data );
           
    // select result
    if ( uSelected == MENU_MODE_PHOTO )
        g_DISPLAY_bExactMode = false;
    else if ( uSelected == MENU_MODE_EXACT )
        g_DISPLAY_bExactMode = true;

    // load and apply pixel color mode and redraw from buffer
    _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer();
}




/*---------------------------------------------------------------------------
| _DISPLAY_OnDelete
|-----------------------------------
| callback for deletion request
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
gboolean _DISPLAY_OnDelete( GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    // disable user window closing
    return TRUE;
}




/*---------------------------------------------------------------------------
| _DISPLAY_ThreadProc
|-----------------------------------
| LCD window thread
+----------------------------------------------------------------------------
| @Params:          default GTK (see GTK+)
| @Return Value:    default GTK (see GTK+)
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_Result _DISPLAY_ThreadProc( gpointer pData )
{
    cmg_Result                      iResult = CMG_OK;
    volatile cmg_GTK_WindowInit     *pGTK_WindowInit = (cmg_GTK_WindowInit*) pData;
    guint                           iTimeOut;
    GtkWidget                       *gtkMenuItemPhoto, *gtkMenuItemExact;

    
    // param check
    CHECK( !pGTK_WindowInit && !pGTK_WindowInit->gtkMutex && !pGTK_WindowInit->gtkCondition, CMG_ERROR_GTK );

    // lock mutex
    g_mutex_lock( pGTK_WindowInit->gtkMutex );


    // . . . . . . . . . . . . . . . . . . . . . . .
    // create window
    // create main window
    g_DISPLAY_gtkWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_window_set_resizable( GTK_WINDOW( g_DISPLAY_gtkWindow ), FALSE );
    gtk_window_set_position( GTK_WINDOW( g_DISPLAY_gtkWindow ), GTK_WIN_POS_CENTER );
    gtk_window_set_title ( GTK_WINDOW( g_DISPLAY_gtkWindow ), DISPLAY_WINDOW_TITLE );
    gtk_window_set_decorated( GTK_WINDOW( g_DISPLAY_gtkWindow ), FALSE );
    g_signal_connect( g_DISPLAY_gtkWindow, "delete-event", G_CALLBACK( _DISPLAY_OnDelete ), NULL );

    // drawing area    
    g_DISPLAY_gtkDrawingArea = gtk_drawing_area_new();
    gtk_widget_add_events( g_DISPLAY_gtkDrawingArea, GDK_BUTTON_PRESS_MASK | GDK_BUTTON1_MOTION_MASK );
    gtk_widget_set_size_request( g_DISPLAY_gtkDrawingArea, g_DISPLAY_iLCDBackground_Width, g_DISPLAY_iLCDBackground_Height );
    g_signal_connect( g_DISPLAY_gtkDrawingArea, "expose-event", G_CALLBACK( _DISPLAY_OnExpose ), NULL );
    g_signal_connect( g_DISPLAY_gtkDrawingArea, "button-press-event", G_CALLBACK( _DISPLAY_OnButtonPress_Or_OnMotion ), NULL );
    g_signal_connect( g_DISPLAY_gtkDrawingArea, "motion-notify-event", G_CALLBACK( _DISPLAY_OnButtonPress_Or_OnMotion ), NULL );
    gtk_container_add( GTK_CONTAINER( g_DISPLAY_gtkWindow ), g_DISPLAY_gtkDrawingArea );

    // show
    gtk_widget_show_all( g_DISPLAY_gtkWindow );
    gtk_window_present( GTK_WINDOW( g_DISPLAY_gtkWindow ) );



    // . . . . . . . . . . . . . . . . . . . . . . .
    // create timer
    // create a timer to update the content
    iTimeOut = g_timeout_add( DISPLAY_WINDOW_TIMER_MS, &_DISPLAY_OnTimer, NULL );
    CHECK( !iTimeOut, CMG_ERROR_GTK );



    // . . . . . . . . . . . . . . . . . . . . . . .
    // create popup menu
    g_DISPLAY_gtkMenu = gtk_menu_new();
    gtkMenuItemPhoto = gtk_menu_item_new_with_label( MENU_MODE_PHOTO_TEXT );
    gtkMenuItemExact = gtk_menu_item_new_with_label( MENU_MODE_EXACT_TEXT );
    CHECK( !g_DISPLAY_gtkMenu || !gtkMenuItemPhoto || !gtkMenuItemExact, CMG_ERROR_GTK );
    gtk_menu_shell_append( GTK_MENU_SHELL( g_DISPLAY_gtkMenu ), gtkMenuItemPhoto );
    gtk_menu_shell_append( GTK_MENU_SHELL( g_DISPLAY_gtkMenu ), gtkMenuItemExact );
    gtk_widget_show( gtkMenuItemPhoto );
    gtk_widget_show( gtkMenuItemExact );
    g_signal_connect( gtkMenuItemPhoto, "activate", G_CALLBACK( _DISPLAY_OnMenuItem ), GUINT_TO_POINTER( MENU_MODE_PHOTO ) );
    g_signal_connect( gtkMenuItemExact, "activate", G_CALLBACK( _DISPLAY_OnMenuItem ), GUINT_TO_POINTER( MENU_MODE_EXACT ) );



    // signalize caller a successfull init
    pGTK_WindowInit->bInited = true;
    g_cond_signal ( pGTK_WindowInit->gtkCondition );
    g_mutex_unlock( pGTK_WindowInit->gtkMutex );



    // . . . . . . . . . . . . . . . . . . . . . . .
    // message loop
    gtk_main();

    
    // . . . . . . . . . . . . . . . . . . . . . . .
    // cleanup
    // destroy vars
    g_source_remove( iTimeOut );
    gtk_widget_destroy( g_DISPLAY_gtkWindow );


    // reset
    g_DISPLAY_gtkThread = NULL;
    g_DISPLAY_gtkWindow = NULL;

    return CMG_OK;


    //.............................
    // on error
_Error:
    // reset
    g_DISPLAY_gtkThread = NULL;
    g_DISPLAY_gtkWindow = NULL;
    return iResult;
}





/*---------------------------------------------------------------------------
| _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer
|-----------------------------------
| redraws all pixels from buffer
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer( void )
{
    cmg_Coord   iX, iY;
    cmg_u8      *pbyLCDBuffer;      // local copy

    // load pixel color color
    if ( g_DISPLAY_bExactMode )
    {
        CMG_LLIO_DISPLAY_COMMON_CreatePixelColor_2x2_32bpp( g_DISPLAY_aadwPixelColor, CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_EXACT );
    }
    else 
    {
        if ( g_DISPLAY_bBacklightOn )
            CMG_LLIO_DISPLAY_COMMON_CreatePixelColor_2x2_32bpp( g_DISPLAY_aadwPixelColor, CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_PHOTO_BLON );
        else
            CMG_LLIO_DISPLAY_COMMON_CreatePixelColor_2x2_32bpp( g_DISPLAY_aadwPixelColor, CMG_LLIO_DISPLAY_COMMON_PIXELCOLORTYPE_PHOTO_BLOFF );
    }

    // prepare lcd bitmap (clear all pixels)
    pbyLCDBuffer = g_DISPLAY_pbyLCDBuffer;     // local copy
    for ( iY = 0; iY < g_DISPLAY_iHeight; iY++ )
        for ( iX = 0; iX < g_DISPLAY_iWidth; iX++ )
            CMG_LLIO_DISPLAY_SetPixel( iX, iY, *pbyLCDBuffer++ );
}

//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_LLIO_DISPLAY_Init
|-----------------------------------
| Init (init downwards: nothing)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_LLIO_DISPLAY_Init( void )
{
    cmg_Result                      iResult;
    volatile cmg_GTK_WindowInit     GTK_WindowInit = { NULL, NULL, false };



    // . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // init GTK+
    // GTK+ not already inited?
    if ( !g_DISPLAY_bInitedGTK )
    {
        // init GTK with fake parameters
        int         argc = 0;
        char        *firstargv = "", **argv = &firstargv;
        gboolean    bInited;

        bInited = gtk_init_check( &argc, &argv );
        CHECK( bInited == FALSE, CMG_ERROR_GTK );

        
        // GLib must support threads
        #if !defined( G_THREADS_ENABLED ) || defined ( G_THREADS_IMPL_NONE )
            #error GTK+: GLib: No thread system is available. GLib must support threads to work.
        #endif

        // init GLib thread system
        if ( !g_thread_supported() )
            g_thread_init( NULL );

        // GTK+ inited
        g_DISPLAY_bInitedGTK = true;
    }



    // . . . . . . . . . . . . . . . . . . . . . . .
    // init stuff
    // set the display resolution
    g_DISPLAY_iWidth = CMG_LLIO_EMU_WIDTH;
    g_DISPLAY_iHeight = CMG_LLIO_EMU_HEIGHT;
    g_DISPLAY_bLCD_On = true;
    g_DISPLAY_bContentChanged = false;
    g_DISPLAY_bModeChanged = false;


    // set byte order
    CMG_LLIO_DISPLAY_COMMON_SetByteOrder( CMG_LLIO_DISPLAY_COMMON_BYTEORDER_GTK );


    
    // . . . . . . . . . . . . . . . . . . . . . . .
    // lcd buffer
    // allocate lcd buffer (for redraw in another color mode)
    g_DISPLAY_pbyLCDBuffer = (cmg_u8*) malloc( g_DISPLAY_iWidth * g_DISPLAY_iHeight );
    CHECK( g_DISPLAY_pbyLCDBuffer == NULL, CMG_ERROR_NOMEM );

    // prepare buffer (clear all pixels)
    memset( g_DISPLAY_pbyLCDBuffer, 0, g_DISPLAY_iWidth * g_DISPLAY_iHeight );



    // . . . . . . . . . . . . . . . . . . . . . . .
    // lcd image
    // each emulated lcd pixel is 2x2 pixels large
    g_DISPLAY_iLCD_Width = g_DISPLAY_iWidth * 2;
    g_DISPLAY_iLCD_Height = g_DISPLAY_iHeight * 2;
    
    // allocate lcd bitmap (each pixels needs 4 bytes)
    g_DISPLAY_pdwLCD = malloc( g_DISPLAY_iLCD_Width * g_DISPLAY_iLCD_Height * sizeof( cmg_u32 ) );
    CHECK( g_DISPLAY_pdwLCD == NULL, CMG_ERROR_NOMEM );



    // . . . . . . . . . . . . . . . . . . . . . . .
    // create pixel color table and prepare lcd bitmap
    // redraw from buffer (=> prepare lcd bitmap (clear all pixels))
    g_DISPLAY_bBacklightOn = true;
    g_DISPLAY_bExactMode = false;
    _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer();

    
    
    // . . . . . . . . . . . . . . . . . . . . . . .
    // lcd background image
    // create image
    g_DISPLAY_iLCDBackground_Width = g_DISPLAY_iLCD_Width + ( 2 * CMG_LLIO_DISPLAY_COMMON_LCDBG_START_X );
    g_DISPLAY_iLCDBackground_Height = g_DISPLAY_iLCD_Height + ( 2 * CMG_LLIO_DISPLAY_COMMON_LCDBG_START_Y );
    iResult = CMG_LLIO_DISPLAY_COMMON_CreateLCDImage32( &g_DISPLAY_pdwLCDBackground, g_DISPLAY_iLCDBackground_Width, g_DISPLAY_iLCDBackground_Height );
    CHECKRESULT();



    // . . . . . . . . . . . . . . . . . . . . . . .
    // create and start thread
    // create condition and mutex to wait until thread is ready (window is correctly created)
    {
        GTimeVal    gtkAbsTimeout;
        gboolean    bConditionSignaled = true;

        GTK_WindowInit.gtkCondition = g_cond_new();
        GTK_WindowInit.gtkMutex     = g_mutex_new();
        GTK_WindowInit.bInited      = false;
        CHECK( GTK_WindowInit.gtkCondition == NULL, CMG_ERROR_GTK );
        CHECK( GTK_WindowInit.gtkMutex == NULL, CMG_ERROR_GTK );

        // lock mutex
        g_mutex_lock( GTK_WindowInit.gtkMutex );

        // create and start thread and submit hWaitEvent as parameter
        g_DISPLAY_gtkThread = g_thread_create( (GThreadFunc) _DISPLAY_ThreadProc, (gpointer) &GTK_WindowInit, TRUE, NULL );
        CHECK( g_DISPLAY_gtkThread == NULL, CMG_ERROR_GTK );

        // set timeout
        g_get_current_time( &gtkAbsTimeout );
        g_time_val_add( &gtkAbsTimeout, DISPLAY_WINDOW_WAITTIME_MS * 1000 );

        // wait for event (window is correctly created)
        while ( !GTK_WindowInit.bInited && bConditionSignaled )
            bConditionSignaled = g_cond_timed_wait( GTK_WindowInit.gtkCondition, GTK_WindowInit.gtkMutex, &gtkAbsTimeout );

        CHECK( ( bConditionSignaled == FALSE ) || ( GTK_WindowInit.bInited == false ), CMG_ERROR_GTK );
        CHECK( ( g_DISPLAY_gtkThread == NULL ) || ( g_DISPLAY_gtkWindow == NULL ), CMG_ERROR_GTK );
    
        // destroy and free GTK+ data structures
        g_mutex_unlock( GTK_WindowInit.gtkMutex );
        g_mutex_free  ( GTK_WindowInit.gtkMutex );
        g_cond_free   ( GTK_WindowInit.gtkCondition );
        GTK_WindowInit.gtkCondition = NULL;
        GTK_WindowInit.gtkMutex = NULL;
    }    

    // return success
    return CMG_OK;


    //.............................
    // on error free everything
_Error:
    // destroy and free GTK+ data structures
    if ( GTK_WindowInit.gtkCondition )
    {
        g_cond_free ( GTK_WindowInit.gtkCondition );
        GTK_WindowInit.gtkCondition = NULL;
    }
    if ( GTK_WindowInit.gtkMutex )
    {
        g_mutex_free( GTK_WindowInit.gtkMutex );
        GTK_WindowInit.gtkMutex = NULL;
    }
    CMG_LLIO_DISPLAY_Exit();
    return iResult;
}



/****************************************************************************
| CMG_LLIO_DISPLAY_Exit
|-----------------------------------
| Exit (exit downwards: nothing)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_DISPLAY_Exit( void )
{
    // . . . . . . . . . . . . . . . . . . . . . . .
    // shutdown lcd window

    // first destroy window
    if ( g_DISPLAY_gtkWindow )
        gtk_main_quit();
        
    // if thread is running wait for it
    if ( g_DISPLAY_gtkThread )
        g_thread_join( g_DISPLAY_gtkThread );

    // . . . . . . . . . . . . . . . . . . . . . . .
    // free memory
    SFREE( g_DISPLAY_pbyLCDBuffer );
    SFREE( g_DISPLAY_pdwLCD );
    SFREE( g_DISPLAY_pdwLCDBackground );    
}



/****************************************************************************
| CMG_LLIO_DISPLAY_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_DISPLAY_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // ... nothing to do ...
            // the driver layer will turn off the display
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // ... nothing to do ...
            // the driver layer will turn on the display
            break;


        // backlight on
        case CMG_PWR_BACKLIGHT_ON:
            g_DISPLAY_bBacklightOn = true;
            _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer();
            break;


        // backlight off
        case CMG_PWR_BACKLIGHT_OFF:
            g_DISPLAY_bBacklightOn = false;
            _DISPLAY_LoadAndApplyPixelColorTableAndRedrawFromBuffer();
            break;
    }
}



/****************************************************************************
| CMG_LLIO_DISPLAY_SetPixel
|-----------------------------------
| Set a new pixel color
+----------------------------------------------------------------------------
| @Params:         
|   * iX:           x coordinate
|   * iY:           y coordinate
|   * byColor:      grayscale color
| @Return Value:    none
+***************************************************************************/
void CMG_LLIO_DISPLAY_SetPixel( cmg_Coord iX, cmg_Coord iY, cmg_Color Color )
{
    // check range
    if ( ( iX >= 0 ) && ( iX < g_DISPLAY_iWidth ) && ( iY >= 0 ) && ( iY < g_DISPLAY_iHeight ) )
    {
        // first set in internal buffer
        g_DISPLAY_pbyLCDBuffer[ ( iY * g_DISPLAY_iWidth  ) + iX ] = Color;

        // set the 2x2 coresponding pixels
        g_DISPLAY_pdwLCD[ ( ( iY<<1 ) + 0 ) * g_DISPLAY_iLCD_Width + ( iX<<1 ) + 0 ] = g_DISPLAY_aadwPixelColor[ Color ][ 0 ];
        g_DISPLAY_pdwLCD[ ( ( iY<<1 ) + 0 ) * g_DISPLAY_iLCD_Width + ( iX<<1 ) + 1 ] = g_DISPLAY_aadwPixelColor[ Color ][ 1 ];
        g_DISPLAY_pdwLCD[ ( ( iY<<1 ) + 1 ) * g_DISPLAY_iLCD_Width + ( iX<<1 ) + 0 ] = g_DISPLAY_aadwPixelColor[ Color ][ 2 ];
        g_DISPLAY_pdwLCD[ ( ( iY<<1 ) + 1 ) * g_DISPLAY_iLCD_Width + ( iX<<1 ) + 1 ] = g_DISPLAY_aadwPixelColor[ Color ][ 3 ];

        // signalize changes
        g_DISPLAY_bContentChanged = true;
    }
}



/****************************************************************************
| CMG_LLIO_DISPLAY_SetMode
|-----------------------------------
| Set the mode
+----------------------------------------------------------------------------
| @Params:          byMode: defined in .h file
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void    CMG_LLIO_DISPLAY_SetMode( cmg_mu8 uMode )
{
    // select mode
    switch ( uMode )
    {
        // turn display off
        case CMG_LLIO_DISPLAY_MODE_OFF:
            g_DISPLAY_bLCD_On = false;
            break;

        // turn display on
        case CMG_LLIO_DISPLAY_MODE_ON:
            g_DISPLAY_bLCD_On = true;
            break;
    }
    
    // set mode changed flag   
    g_DISPLAY_bModeChanged = true;
}



/****************************************************************************
| CMG_LLIO_DISPLAY_ErrorMessage
|-----------------------------------
| Display an error message
+----------------------------------------------------------------------------
| @Params:         
|   * strMessage:   message to display
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LLIO_DISPLAY_ErrorMessage( cmg_char *strMessage )
{
/*    GtkWidget   *gtkDialog;
    
    // create message box
    gtkDialog = gtk_message_dialog_new( g_DISPLAY_gtkWindow, GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "!!!" );
    gtk_dialog_run( GTK_DIALOG( gtkDialog ) );
    gtk_widget_destroy( gtkDialog );
*/
    printf( strMessage );
}

//---------------------------------------------------------------------------


#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
