//***************************************************************************
// CMG_CTRL_LH155.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         02.02.2007
//***************************************************************************
// History:
//   03.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************


// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_CTRL_CONFIGURED
    #error CMG_CTRL not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_CTRL_LH155 )

    #if !defined( CMG_CTRL_WIDTH ) || !defined( CMG_CTRL_HEIGHT ) || !defined( CMG_CTRL_BITDEPTH ) 
        #error CMG_CTRL_* are not defined. Please customize the config file correctly using the *.config template.
    #endif 
    #if CMG_CTRL_BITDEPTH != 1
        #error CMG_CTRL_LH155 only supports CMG_CTRL_BITDEPTH==1. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_CTRL.h"
#include    "../1_LLIO/CMG_LLIO.h"
#include    "../MISC/CMG_PlatformDependent.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------

// address line defines
#define     CMG_LLIO_WriteCommand           CMG_LLIO_WriteA1
#define     CMG_LLIO_WriteData              CMG_LLIO_WriteA0
#define     CMG_LLIO_ReadData               CMG_LLIO_ReadA0


// LH155 commands   (* marks used)
#define     CMD_CURSOR_X_LO                 0x00    // x address set (lo)
#define     CMD_CURSOR_X_HI                 0x10    // x address set (hi)
#define     CMD_CURSOR_Y_LO                 0x20    // y address set (lo)
#define     CMD_CURSOR_Y_HI                 0x30    // y address set (hi)

#define     CMD_STARTLINE_LO                0x40    // start line (lo)
#define     CMD_STARTLINE_HI                0x50    // start line (hi)

#define     CMD_ALTERNATING_NLINE_LO        0x60    // alternating n-line (lo)
#define     CMD_ALTERNATING_NLINE_HI        0x70    // alternating n-line (hi)

#define     CMD_DISPLAY_CTRL_1              0x80    // display control 1
#define     CMD_DISPLAY_CTRL_2              0x90    // display control 2

#define     CMD_INCREMENT                   0xa0    // increment control set

#define     CMD_POWER_CTRL_1                0xb0    // power control 1
#define     CMD_POWER_CTRL_2                0xd0    // power control 2
#define     CMD_POWER_CTRL_3                0xe0    // power control 3

#define     CMD_RE_FLAG                     0xf0    // RE flag set


// flags
#define     CMD_DC1_SHIFT                   8       // dc1 flag: SHIFT
#define     CMD_DC1_SEGON                   4       // dc1 flag: SEG ON
#define     CMD_DC1_ALLON                   2       // dc1 flag: ALL ON
#define     CMD_DC1_ON                      1       // dc1 flag: GFX ON

#define     CMD_DC2_REV                     8       // dc2 flag: REV
#define     CMD_DC2_NLIN                    4       // dc2 flag: NLIN
#define     CMD_DC2_SWAP                    2       // dc2 flag: SWAP
#define     CMD_DC2_REF                     1       // dc2 flag: REF

#define     CMD_INC_AIM                     4       // increment flag: mode
#define     CMD_INC_AYI                     2       // increment flag: y inc
#define     CMD_INC_AXI                     1       // increment flag: x inc

#define     CMD_PC1_BIAS                    8       // pc1 flag: BIAS
#define     CMD_PC1_HALT                    4       // pc1 flag: HALT
#define     CMD_PC1_PON                     2       // pc1 flag: Power ON
#define     CMD_PC1_ACL                     1       // pc1 flag: ACL reset

#define     CMD_PC2_DVOL_OFF                0x0f    // pc2 value: DVOL off

#define     CMD_PC3_EXA                     2       // pc3 flag: EXA
#define     CMD_PC3_ICON                    1       // pc3 flag: ICON

#define     CMD_RE_RE                       1       // RE flag: RE


// combo commands (for LOHI function)
#define     CMD_CURSOR_X_LOHI               CMD_CURSOR_X_LO
#define     CMD_CURSOR_Y_LOHI               CMD_CURSOR_Y_LO
#define     CMD_STARTLINE_LOHI              CMD_STARTLINE_LO
#define     CMD_ALTERNATING_NLINE_LOHI      CMD_ALTERNATING_NLINE_LO

//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
cmg_Coord   g_CTRL_iDisplayWidth;           // display x resolution
cmg_Coord   g_CTRL_iDisplayHeight;          // display y resolution
cmg_mu16    g_CTRL_uBytesPerLine;           // bytes per line (==256: faked)

// controler specific
cmg_mu8     g_CTRL_uNeedDummyRead;          // last operation was a write
cmg_u8      g_CTRL_byCurrentCursorX;        // current cursor x position
cmg_u8      g_CTRL_byCurrentCursorY;        // current cursor y position
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// functions
//---------------------------------------------------------------------------



/*---------------------------------------------------------------------------
| _CTRL_SendCommand
|-----------------------------------
| send a command + 4 bit parameter to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
|   * byParam:      parameter
| @Return Value:    none
+--------------------------------------------------------------------------*/
#define _CTRL_SendCommand(byCommand,byParam)    CMG_LLIO_WriteCommand( (cmg_u8)( (byCommand) + ( (byParam) & 0x0f ) ) )




/*---------------------------------------------------------------------------
| _CTRL_SendCommand_LOHI
|-----------------------------------
| send two commands + 4 bit parameter each to the driver
| the command must be the _LO_ command
+----------------------------------------------------------------------------
| @Params:          
|   * byCommandLO:  LO command
|   * byParam:      parameter
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_SendCommand_LOHI( cmg_u8 byCommandLO, cmg_u8 byParam )
{
    // write lo command
    _CTRL_SendCommand( byCommandLO       , byParam );
    // write hi command
    _CTRL_SendCommand( byCommandLO + 0x10, byParam >> 4 );
}



//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_CTRL_Init
|-----------------------------------
| Init (init downwards: CMG_LLIO_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_Init( void )
{
    cmg_Result  iResult;

    // init CMG_LLIO_*
    iResult = CMG_LLIO_Init();
    CHECKRESULT();
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // set global variables
    g_CTRL_iDisplayWidth  = CMG_CTRL_WIDTH;
    g_CTRL_iDisplayHeight = CMG_CTRL_HEIGHT;
    g_CTRL_uBytesPerLine  = 0x0100; // set fake linesize for faster decoding
    g_CTRL_uNeedDummyRead = 1;
    g_CTRL_byCurrentCursorX = 0;
    g_CTRL_byCurrentCursorY = 0;
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // setup display driver
    // reset
    _CTRL_SendCommand( CMD_POWER_CTRL_1, CMD_PC1_PON | CMD_PC1_ACL );
    CMG_Sleep_ms( 10 );
    
    // power control
    _CTRL_SendCommand( CMD_RE_FLAG, 0 );
    _CTRL_SendCommand( CMD_POWER_CTRL_1, CMD_PC1_PON );
    _CTRL_SendCommand( CMD_POWER_CTRL_2, CMD_PC2_DVOL_OFF );
    _CTRL_SendCommand( CMD_POWER_CTRL_3, 0 );
    
    // display control
    _CTRL_SendCommand( CMD_DISPLAY_CTRL_1, 0 );
    _CTRL_SendCommand( CMD_DISPLAY_CTRL_2, CMD_DC2_SWAP );
    
    // other defaults
    _CTRL_SendCommand_LOHI( CMD_STARTLINE_LOHI, 0 );
    _CTRL_SendCommand_LOHI( CMD_ALTERNATING_NLINE_LOHI, 0 );

    // clear screen
    CMG_CTRL_FastClearScreen( 0x00 );

    // turn display on in graphics mode
    _CTRL_SendCommand( CMD_DISPLAY_CTRL_1, CMD_DC1_ON );


    // . . . . . . . . . . . . . . . . . . . . . . .
    // init
    CMG_CTRL_SetDirections( CTRL_DIR_RD_NONE_WR_FASTEST );
    CMG_CTRL_SetAddressPtr( 0x0000 );


    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_CTRL_Exit();
    return iResult;
}



/****************************************************************************
| CMG_CTRL_Exit
|-----------------------------------
| Exit (exit downwards: CMG_LLIO_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_Exit( void )
{
    // turn off display
    _CTRL_SendCommand( CMD_DISPLAY_CTRL_1, 0 );

    // turn off power
    _CTRL_SendCommand( CMD_POWER_CTRL_1, 0 );

    CMG_Sleep_ms( 10 );
    
    // exit CMG_LLIO_*
    CMG_LLIO_Exit();
}



/****************************************************************************
| CMG_CTRL_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // turn display off
            _CTRL_SendCommand( CMD_DISPLAY_CTRL_1, 0 );
            
            // on off call parent at last
            CMG_LLIO_PowerManagement( uState );
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // on on call parent first
            CMG_LLIO_PowerManagement( uState );
            
            // turn display on in graphics mode
            _CTRL_SendCommand( CMD_DISPLAY_CTRL_1, CMD_DC1_ON );
            break;


        // just pass trough unknown
        default:
            // case CMG_PWR_BACKLIGHT_ON:
            // case CMG_PWR_BACKLIGHT_OFF:
            CMG_LLIO_PowerManagement( uState );
            break;
    }
}



/****************************************************************************
| CMG_CTRL_ControllerSpecific
|-----------------------------------
| Controller specific functions (contrast, etc.)
+----------------------------------------------------------------------------
| @Params:         
|   * uFunction:        selected feature
|   * pParam:           corresponding parameter
| @Return Value:    result, if controller supports this
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam )
{
    // this driver supports no specific function
    return CMG_ERROR_NOTSUPPORTED;
}




/****************************************************************************
| CMG_CTRL_SetAddressPtr
|-----------------------------------
| set the address pointer
+----------------------------------------------------------------------------
| @Params:          
|   * wAddress:     address pointer to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetAddressPtr( cmg_ScrAddr wScreenAddress )
{
    // due the fact we faked a linesize of 256 bytes, we can easily split
    // the address to x and y
    g_CTRL_byCurrentCursorX = wScreenAddress & 0x00ff;
    g_CTRL_byCurrentCursorY = wScreenAddress >> 8;
    
    _CTRL_SendCommand_LOHI( CMD_CURSOR_X_LOHI, g_CTRL_byCurrentCursorX );
    _CTRL_SendCommand_LOHI( CMD_CURSOR_Y_LOHI, g_CTRL_byCurrentCursorY );
    
    g_CTRL_uNeedDummyRead = 1;
}



/****************************************************************************
| CMG_CTRL_SetDirections
|-----------------------------------
| set cursor read/write directions
+----------------------------------------------------------------------------
| @Params:          
|   * iDirections:    directions
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetDirections( cmg_ms8 iDirectionSet )
{
    switch ( iDirectionSet )
    {
        case CTRL_DIR_RD_NONE_WR_FASTEST:
            _CTRL_SendCommand( CMD_INCREMENT, 0 );
            break;
            
        case CTRL_DIR_RD_NONE_WR_RIGHT:
            _CTRL_SendCommand( CMD_INCREMENT, CMD_INC_AIM | CMD_INC_AXI );
            break;
            
        case CTRL_DIR_RD_NONE_WR_DOWN:
            _CTRL_SendCommand( CMD_INCREMENT, CMD_INC_AIM | CMD_INC_AYI );
            break;
            
        case CTRL_DIR_RD_RIGHT_WR_INVALID:
            _CTRL_SendCommand( CMD_INCREMENT, CMD_INC_AXI );
            break;
    }
    
    // (re) set the last setted cursor address (can be destroyed while
    // changing direction - according to datasheet)
    _CTRL_SendCommand_LOHI( CMD_CURSOR_X_LOHI, g_CTRL_byCurrentCursorX );
    _CTRL_SendCommand_LOHI( CMD_CURSOR_Y_LOHI, g_CTRL_byCurrentCursorY );
}



/****************************************************************************
| CMG_CTRL_SetByte
|-----------------------------------
| set a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       data byte
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetByte( cmg_u8 byData )
{
    // set data byte
    CMG_LLIO_WriteData( byData );
    g_CTRL_uNeedDummyRead = 1;
    
    // g_CTRL_byCurrentCursorX/Y is NOT updated at the moment, becaue it
    // is NOT needed yet. But don't forget it.
}



/****************************************************************************
| CMG_CTRL_GetByte
|-----------------------------------
| get a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+***************************************************************************/
_PUBLIC
cmg_u8 CMG_CTRL_GetByte( void )
{
    cmg_u8  byData, byDataSwapped, byBit;
    
    // need a dummy read? (after address set and write cycle)
    if ( g_CTRL_uNeedDummyRead )
    {
        // dummy read
        CMG_LLIO_ReadData();
        g_CTRL_uNeedDummyRead = 0;
    }

    // read byte
    byData = CMG_LLIO_ReadData();

    // swap byte :-(
    byDataSwapped = 0;
    for ( byBit = 0; byBit < 8; byBit++ )
    {
        byDataSwapped <<= 1;
        if ( byData & 0x01 )
            byDataSwapped++;
        byData >>= 1;
    }
    
    // g_CTRL_byCurrentCursorX/Y is NOT updated at the moment, becaue it
    // is NOT needed yet. But don't forget it.

    // return data byte
    return byDataSwapped;
}



/****************************************************************************
| CMG_CTRL_FastClearScreen
|-----------------------------------
| Fast fills complete screen with byCol color.
+----------------------------------------------------------------------------
| @Params:          
|   * byCol:        color
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_FastClearScreen( cmg_u8 byCol )
{
    cmg_u8      byX, byY, byLenBytes;

    // only 0x00 or 0xff is allowed
    if ( byCol )    byCol = 0xff;
    
    // prepare increment
    _CTRL_SendCommand( CMD_INCREMENT, CMD_INC_AXI );

    // calc byte length
    byLenBytes = GET_BYTES_FROM_BITS( g_CTRL_iDisplayWidth );

    // for each line
    for ( byY = 0; byY < g_CTRL_iDisplayHeight; byY++ )
    {
        // set cursor
        _CTRL_SendCommand_LOHI( CMD_CURSOR_X_LOHI, 0 );
        _CTRL_SendCommand_LOHI( CMD_CURSOR_Y_LOHI, byY );
        
        // fill line
        for ( byX = 0; byX < byLenBytes; byX++ )
            CMG_LLIO_WriteData( byCol );
    }

    CMG_CTRL_SetAddressPtr( 0 );
}


//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
