//***************************************************************************
// CMG_CTRL_S1D13700.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.02.2007
//***************************************************************************
// History:
//   06.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************


// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_CTRL_CONFIGURED
    #error CMG_CTRL not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_CTRL_S1D13700 )

    #if !defined( CMG_CTRL_WIDTH ) || !defined( CMG_CTRL_HEIGHT ) || !defined( CMG_CTRL_BITDEPTH ) 
        #error CMG_CTRL_* are not defined. Please customize the config file correctly using the *.config template.
    #endif 
    #if CMG_CTRL_BITDEPTH != 1
        #error CMG_CTRL_S1D13700 only supports CMG_CTRL_BITDEPTH==1. Please customize the config file correctly using the *.config template.
    #endif 
    #if !defined( CMG_CTRL_CONTROLLER_CLOCK_HZ ) || !defined( CMG_CTRL_FRAMERATE )
        #error CMG_CTRL_CONTROLLER_CLOCK_HZ or CMG_CTRL_FRAMERATE are not defined. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_CTRL.h"
#include    "../1_LLIO/CMG_LLIO.h"
#include    "../MISC/CMG_PlatformDependent.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------

// address line defines
#define     CMG_LLIO_WriteCommand           CMG_LLIO_WriteA1
#define     CMG_LLIO_WriteData              CMG_LLIO_WriteA0
#define     CMG_LLIO_ReadData               CMG_LLIO_ReadA1


// S1D13700 commands   (* marks used)
#define     CMD_SYSTEM_SET_8                0x40    //* set initial operation
#define     CMD_SLEEP_IN_0                  0x53    //  sleep op
#define     CMD_DISP_OFF_1                  0x58    //* turn display off
#define     CMD_DISP_ON_1                   0x59    //* turn display on
#define     CMD_SCROLL_10                   0x44    //* display start and area
#define     CMD_CSRFORM_2                   0x5d    //* cursor shape
#define     CMD_CSRDIR_RIGHT_0              0x4c    //* cursor direction
#define     CMD_CSRDIR_LEFT_0               0x4d    // cursor direction
#define     CMD_CSRDIR_UP_0                 0x4e    // cursor direction
#define     CMD_CSRDIR_DOWN_0               0x4f    //* cursor direction
#define     CMD_OVLAY_1                     0x5b    //* overlay mode
#define     CMD_CGRAM_ADR_2                 0x5c    // cgram start
#define     CMD_HDOT_SCR_1                  0x5a    //* hor dir dot unit + scroll
#define     CMD_GRAY_SCALE_1                0x60    //* grayscale mode
#define     CMD_CSRW_2                      0x46    //* set cursor address
#define     CMD_CSRR_2                      0x47    // get cursor address
#define     CMD_MWRITE                      0x42    //* write data
#define     CMD_MREAD                       0x43    //* read data


// flags
#define     CMG_SS_P1                       0x10    // ss param1 default value
#define     CMG_SS_P1_M0                    0x01    // ss p1 cg rom/ram
#define     CMG_SS_P1_M1                    0x02    // ss p1 cg rom/ram
#define     CMG_SS_P1_M2                    0x04    // ss p1 cg rom/ram
#define     CMG_SS_P1_WS                    0x08    // ss p1 single/dual screen
#define     CMG_SS_P1_IV                    0x20    // ss p1 correct scr. orign
#define     CMG_SS_P2_WF                    0x80    // ss p2 ac drive method

#define     CMG_CF_P2_CM                    0x80    // cf p2 cursor shape

#define     CMG_OL_P1_MX01_OR               0x00    // overlay screen 1+2 mode
#define     CMG_OL_P1_MX01_XOR              0x01    // overlay screen 1+2 mode
#define     CMG_OL_P1_MX01_AND              0x02    // overlay screen 1+2 mode
#define     CMG_OL_P1_MX01_PRIO             0x03    // overlay screen 1+2 mode
#define     CMG_OL_P1_DM1                   0x04    // screen 1 mode
#define     CMG_OL_P1_DM2                   0x08    // screen 3 mode
#define     CMG_OL_P1_OV                    0x10    // 2/3 screen overlay

#define     CMG_DO_FC_CURSOR_OFF            0x00    // cursor state
#define     CMG_DO_FC_CURSOR_ON             0x01    // cursor state
#define     CMG_DO_FC_SCREEN1_OFF           0x00    // screen state
#define     CMG_DO_FC_SCREEN1_ON            0x04    // screen state
#define     CMG_DO_FC_SCREEN2_OFF           0x00    // screen state
#define     CMG_DO_FC_SCREEN2_ON            0x10    // screen state
#define     CMG_DO_FC_SCREEN3_OFF           0x00    // screen state
#define     CMG_DO_FC_SCREEN3_ON            0x40    // screen state

#define     CMG_GS_P1_1BPP                  0x00    // grayscale 1bpp
#define     CMG_GS_P1_2BPP                  0x01    // grayscale 2bpp
#define     CMG_GS_P1_4BPP                  0x02    // grayscale 4bpp
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// display system parameters - defines
//---------------------------------------------------------------------------
#define     CMG_CTRL_FX         8                   // char x size (must be 8)
#define     CMG_CTRL_FY         1                   // char y size (must be 1)
#define     CMG_CTRL_LF         CMG_CTRL_HEIGHT     // number of lines
// display address range
#define     CMG_CTRL_CR         ( CMG_CTRL_WIDTH / CMG_CTRL_FX )   
#define     CMG_CTRL_AP         CMG_CTRL_CR         // virtual screen x size
// total display address range (depending on controller speed, desired framerate)
#define     CMG_CTRL_TCR_VAL    ( ( ( CMG_CTRL_CONTROLLER_CLOCK_HZ - ( ( CMG_CTRL_FRAMERATE + 0ul ) * CMG_CTRL_LF ) ) / ( 9 * ( ( CMG_CTRL_FRAMERATE + 0ul ) * CMG_CTRL_LF ) ) ) + 1 )
#define     CMG_CTRL_TCR        ( (cmg_u8)(CMG_CTRL_TCR_VAL) )

#define     CMG_SCREEN_SIZE_BYTES     ( CMG_CTRL_CR * CMG_CTRL_LF ) // byte of one screen

// values security checks according to controllers datasheet
#if ( ( CMG_CTRL_CR * CMG_CTRL_FX ) != CMG_CTRL_WIDTH )
    #error CMG_CTRL_WIDTH must be dividable by 8
#endif
#if ( CMG_CTRL_TCR_VAL < ( CMG_CTRL_CR + 4 ) )
    #error CMG_CTRL_TCR must be greater than ( CMG_CTRL_CR + 4 ). Check CMG_CTRL_CONTROLLER_CLOCK_HZ and CMG_CTRL_FRAMERATE or adjust CMG_CTRL_FRAMERATE (decrease).
#endif
#if ( CMG_CTRL_TCR_VAL > 255 )
    #error CMG_CTRL_TCR must be less than 255. Check CMG_CTRL_CONTROLLER_CLOCK_HZ and CMG_CTRL_FRAMERATE or adjust CMG_CTRL_FRAMERATE (increase).
#endif
#if ( CMG_SCREEN_SIZE_BYTES > (32l*KB) )
    #error CMG_SCREEN_SIZE_BYTES is larger than 32kb. Check screen resolution and reduce it.
#endif
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
cmg_Coord   g_CTRL_iDisplayWidth;            // display x resolution
cmg_Coord   g_CTRL_iDisplayHeight;           // display y resolution
cmg_mu16    g_CTRL_uBytesPerLine;            // bytes per line

// controler specific
cmg_ms8     g_CTRL_iDirections;             // next selected cursor dir
    #define     DIR_RIGHT                       0
    #define     DIR_DOWN                        1
cmg_ms8     g_CTRL_iCurrentDirection;       // currently selected cursor dir
cmg_ScrAddr g_CTRL_wCurrentScreenAddress;   // current address ptr
    #define     CMD_OTHER                       0
    #define     CMD_WRITE                       1
    #define     CMD_READ                        2
cmg_mu8     g_CTRL_uLastCommand;            // last command
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


/*---------------------------------------------------------------------------
| _CTRL_SendCommand
|-----------------------------------
| Sends the system set command.
| In an own function because I send it twice.
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void CMG_CTRL_Init_SystemSet( void )
{
    // system set command
    CMG_LLIO_WriteCommand( CMD_SYSTEM_SET_8 );
    CMG_LLIO_WriteData( CMG_SS_P1 | CMG_SS_P1_IV ); // default cgrom, single screen, no screen orign correction
    CMG_LLIO_WriteData( CMG_SS_P2_WF | ( CMG_CTRL_FX - 1 ) ); 
    CMG_LLIO_WriteData( CMG_CTRL_FY - 1 );
    CMG_LLIO_WriteData( CMG_CTRL_CR - 1 );
    CMG_LLIO_WriteData( CMG_CTRL_TCR - 1 );
    CMG_LLIO_WriteData( CMG_CTRL_LF - 1 );
    CMG_LLIO_WriteData( CMG_CTRL_AP );
    CMG_LLIO_WriteData( 0x00 );
}



/****************************************************************************
| CMG_CTRL_Init
|-----------------------------------
| Init (init downwards: CMG_LLIO_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_Init( void )
{
    cmg_Result  iResult;

    // init CMG_LLIO_*
    iResult = CMG_LLIO_Init();
    CHECKRESULT();
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // set global variables
    g_CTRL_iDisplayWidth  = CMG_CTRL_WIDTH;
    g_CTRL_iDisplayHeight = CMG_CTRL_HEIGHT;
    g_CTRL_uBytesPerLine  = GET_BYTES_FROM_BITS( g_CTRL_iDisplayWidth );
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // init display driver
    // reset
    CMG_CTRL_Init_SystemSet();
    CMG_Sleep_ms(10);

    // system set
    CMG_CTRL_Init_SystemSet();

    // turn display off 
    CMG_LLIO_WriteCommand( CMD_DISP_OFF_1 );
    CMG_LLIO_WriteData( CMG_DO_FC_CURSOR_OFF | CMG_DO_FC_SCREEN1_OFF | CMG_DO_FC_SCREEN2_OFF );

    // set grayscale mode
    CMG_LLIO_WriteCommand( CMD_GRAY_SCALE_1 );
    CMG_LLIO_WriteData( CMG_GS_P1_1BPP );

    // overlay set (two screens: 1: graphic, 2: graphic)
    CMG_LLIO_WriteCommand( CMD_OVLAY_1 );
    CMG_LLIO_WriteData( CMG_OL_P1_MX01_OR | CMG_OL_P1_DM1 | CMG_OL_P1_DM2 );

    // scroll - set display start and area (screen1 starts on 0x0000)
    CMG_LLIO_WriteCommand( CMD_SCROLL_10 );
    CMG_LLIO_WriteData( 0x00 );             // first screen address: 0x0000
    CMG_LLIO_WriteData( 0x00 );
    CMG_LLIO_WriteData( CMG_CTRL_LF - 1 );  // srceen line count
    CMG_LLIO_WriteData( (cmg_u8)( CMG_SCREEN_SIZE_BYTES ) );
    CMG_LLIO_WriteData( (cmg_u8)( CMG_SCREEN_SIZE_BYTES >> 8 ) );
    CMG_LLIO_WriteData( 0x00 );             // second screen has 1 dummy line
    CMG_LLIO_WriteData( 0x00 );             // unused
    CMG_LLIO_WriteData( 0x00 );             // unused
    CMG_LLIO_WriteData( 0x00 );             // unused
    CMG_LLIO_WriteData( 0x00 );             // unused

    // cursor form (8x8)
    CMG_LLIO_WriteCommand( CMD_CSRFORM_2 );
    CMG_LLIO_WriteData( 0x00 );             // cusor x size == 1
    CMG_LLIO_WriteData( 0x00 );             // cusor y size == 1

    // horizontal dot scrolling (disable)
    CMG_LLIO_WriteCommand( CMD_HDOT_SCR_1 );
    CMG_LLIO_WriteData( 0x00 );             // no scrolling

    // clear screen
    CMG_CTRL_FastClearScreen( 0x00 );

    // reset cursor
    CMG_LLIO_WriteCommand( CMD_CSRDIR_RIGHT_0 );
    CMG_CTRL_SetAddressPtr( 0x0000 );
    
    // turn display on (screen 1 only)
    CMG_LLIO_WriteCommand( CMD_DISP_ON_1 );
    CMG_LLIO_WriteData( CMG_DO_FC_CURSOR_OFF | CMG_DO_FC_SCREEN1_ON | CMG_DO_FC_SCREEN2_OFF );


    // . . . . . . . . . . . . . . . . . . . . . . .
    // init globals
    g_CTRL_iDirections = CTRL_DIR_RD_NONE_WR_FASTEST;
    g_CTRL_wCurrentScreenAddress = 0;
    g_CTRL_iCurrentDirection = DIR_RIGHT;
    g_CTRL_uLastCommand = CMD_OTHER;
    

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_CTRL_Exit();
    return iResult;
}


/****************************************************************************
| CMG_CTRL_Exit
|-----------------------------------
| Exit (exit downwards: CMG_LLIO_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_Exit( void )
{
    // turn display off 
    CMG_LLIO_WriteCommand( CMD_DISP_OFF_1 );
    CMG_LLIO_WriteData( CMG_DO_FC_CURSOR_OFF | CMG_DO_FC_SCREEN1_OFF | CMG_DO_FC_SCREEN2_OFF );

    CMG_Sleep_ms( 10 );

    // exit CMG_LLIO_*
    CMG_LLIO_Exit();
}



/****************************************************************************
| CMG_CTRL_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // turn display off
            CMG_LLIO_WriteCommand( CMD_DISP_OFF_1 );
            CMG_LLIO_WriteData( CMG_DO_FC_CURSOR_OFF | CMG_DO_FC_SCREEN1_OFF | CMG_DO_FC_SCREEN2_OFF );
            
            // on off call parent at last
            CMG_LLIO_PowerManagement( uState );
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // on on call parent first
            CMG_LLIO_PowerManagement( uState );
            
            // turn display on (screen 1 only)
            CMG_LLIO_WriteCommand( CMD_DISP_ON_1 );
            CMG_LLIO_WriteData( CMG_DO_FC_CURSOR_OFF | CMG_DO_FC_SCREEN1_ON | CMG_DO_FC_SCREEN2_OFF );
            break;


        // just pass trough unknown
        default:
            // case CMG_PWR_BACKLIGHT_ON:
            // case CMG_PWR_BACKLIGHT_OFF:
            CMG_LLIO_PowerManagement( uState );
            break;
    }
    g_CTRL_uLastCommand = CMD_OTHER;
}



/****************************************************************************
| CMG_CTRL_ControllerSpecific
|-----------------------------------
| Controller specific functions (contrast, etc.)
+----------------------------------------------------------------------------
| @Params:         
|   * uFunction:        selected feature
|   * pParam:           corresponding parameter
| @Return Value:    result, if controller supports this
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam )
{
    // this driver supports no specific function
    return CMG_ERROR_NOTSUPPORTED;
}




/****************************************************************************
| CMG_CTRL_SetAddressPtr
|-----------------------------------
| set the address pointer
+----------------------------------------------------------------------------
| @Params:          
|   * wAddress:     address pointer to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetAddressPtr( cmg_ScrAddr wScreenAddress )
{
    // send set cursor command
    CMG_LLIO_WriteCommand( CMD_CSRW_2 );
    CMG_LLIO_WriteData( (cmg_u8)( wScreenAddress & 0x00ff ) );
    CMG_LLIO_WriteData( (cmg_u8)( wScreenAddress >> 8 ) );
    
    // store current cursor
    g_CTRL_wCurrentScreenAddress = wScreenAddress;
    g_CTRL_uLastCommand = CMD_OTHER;
}



/****************************************************************************
| CMG_CTRL_SetDirections
|-----------------------------------
| set cursor pointer directions
+----------------------------------------------------------------------------
| @Params:          
|   * iDirections:      directions
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetDirections( cmg_ms8 iDirections )
{
    // just store next direction
    g_CTRL_iDirections = iDirections;
}



/****************************************************************************
| CMG_CTRL_SetByte
|-----------------------------------
| set a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       data byte
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetByte( cmg_u8 byData )
{
    // check for a direction change
    switch ( g_CTRL_iDirections )
    {
        // -> (right)
        case CTRL_DIR_RD_NONE_WR_RIGHT:
            // if current direction is NOT right
            if ( g_CTRL_iCurrentDirection != DIR_RIGHT )
            {
                // set right direction
                CMG_LLIO_WriteCommand( CMD_CSRDIR_RIGHT_0 );
                g_CTRL_iCurrentDirection = DIR_RIGHT;
                g_CTRL_uLastCommand = CMD_OTHER;
            }
            // adjust cursor
            g_CTRL_wCurrentScreenAddress++;
            break;
        
        // v (down)
        case CTRL_DIR_RD_NONE_WR_DOWN:
            // if current direction is NOT down
            if ( g_CTRL_iCurrentDirection != DIR_DOWN )
            {
                // set down direction
                CMG_LLIO_WriteCommand( CMD_CSRDIR_DOWN_0 );
                g_CTRL_iCurrentDirection = DIR_DOWN;
                g_CTRL_uLastCommand = CMD_OTHER;
            }
            // adjust cursor
            g_CTRL_wCurrentScreenAddress += g_CTRL_uBytesPerLine;
            break;
        
        // if *WR_FASTEST it doesn't matter in wich mode we are...
        //case CTRL_DIR_RD_NONE_WR_FASTEST:
        //    break;
        
        // this one is INVALID in write mode
        //case CTRL_DIR_RD_RIGHT_WR_INVALID:
        //    break;
    }

    // if the last command was a write we do not need to send it again
    if ( g_CTRL_uLastCommand != CMD_WRITE )
        CMG_LLIO_WriteCommand( CMD_MWRITE );

    // write data byte
    CMG_LLIO_WriteData( byData );

    // set that last command was a write
    g_CTRL_uLastCommand = CMD_WRITE;
}



/****************************************************************************
| CMG_CTRL_GetByte
|-----------------------------------
| get a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+***************************************************************************/
_PUBLIC
cmg_u8 CMG_CTRL_GetByte( void )
{
    cmg_u8  byData = 0;

    // PRE-check for a direction change
    switch ( g_CTRL_iDirections )
    {
        // -> (right)
        case CTRL_DIR_RD_RIGHT_WR_INVALID:
            // if current direction is NOT right
            if ( g_CTRL_iCurrentDirection != DIR_RIGHT )
            {
                // set right direction
                CMG_LLIO_WriteCommand( CMD_CSRDIR_RIGHT_0 );
                g_CTRL_iCurrentDirection = DIR_RIGHT;
                g_CTRL_uLastCommand = CMD_OTHER;
            }
            // adjust cursor
            g_CTRL_wCurrentScreenAddress++;
            break;
        
        // on none direction we have to set back the cursor afterwards
        // because the controller doesn't support a none command :-(
        //case CTRL_DIR_RD_NONE_WR_FASTEST:
        //case CTRL_DIR_RD_NONE_WR_RIGHT:
        //case CTRL_DIR_RD_NONE_WR_DOWN:
    }

    // if the last command was a read we do not need to send it again
    if ( g_CTRL_uLastCommand != CMD_READ )
        CMG_LLIO_WriteCommand( CMD_MREAD );

    // read data byte
    byData = CMG_LLIO_ReadData();

    // set that last command was a read
    g_CTRL_uLastCommand = CMD_READ;

    // POST-check for a direction change
    switch ( g_CTRL_iDirections )
    {
        // o (none)
        case CTRL_DIR_RD_NONE_WR_FASTEST:
        case CTRL_DIR_RD_NONE_WR_RIGHT:
        case CTRL_DIR_RD_NONE_WR_DOWN:
            // on every none direction we must set back the cursor afterwards
            CMG_LLIO_WriteCommand( CMD_CSRW_2 );
            CMG_LLIO_WriteData( (cmg_u8)( g_CTRL_wCurrentScreenAddress ) );
            CMG_LLIO_WriteData( (cmg_u8)( g_CTRL_wCurrentScreenAddress >> 8 ) );
            
            g_CTRL_uLastCommand = CMD_OTHER;
            break;
    }

    // return data
    return byData;
}



/****************************************************************************
| CMG_CTRL_FastClearScreen
|-----------------------------------
| Fast fills complete screen with byCol color.
+----------------------------------------------------------------------------
| @Params:          
|   * byCol:        color
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_FastClearScreen( cmg_u8 byCol )
{
    cmg_u16     wLenBytes;
    
    // only 0x00 or 0xff is allowed
    if ( byCol )    byCol = 0xff;

    // set cursor
    CMG_LLIO_WriteCommand( CMD_CSRDIR_RIGHT_0 );
    CMG_LLIO_WriteCommand( CMD_CSRW_2 );
    CMG_LLIO_WriteData( 0x00 );
    CMG_LLIO_WriteData( 0x00 );

    // calc byte length
    wLenBytes = (cmg_u16)( g_CTRL_uBytesPerLine * g_CTRL_iDisplayHeight );

    // fill
    CMG_LLIO_WriteCommand( CMD_MWRITE );
    while(  wLenBytes-- != 0 )
        CMG_LLIO_WriteData( byCol );
        
    CMG_CTRL_SetAddressPtr( 0 );
}



//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
