//***************************************************************************
// CMG_CTRL_T6963.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         16.11.2006
//***************************************************************************
// History:
//   05.12.2006     *stable*
//   23.01.2007     added power management
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************


// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_CTRL_CONFIGURED
    #error CMG_CTRL not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_CTRL_T6963 )

    #if !defined( CMG_CTRL_WIDTH ) || !defined( CMG_CTRL_HEIGHT ) || !defined( CMG_CTRL_BITDEPTH ) 
        #error CMG_CTRL_* are not defined. Please customize the config file correctly using the *.config template.
    #endif 
    #if CMG_CTRL_BITDEPTH != 1
        #error CMG_CRTL_T6963 only supports CMG_CTRL_BITDEPTH==1. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_CTRL.h"
#include    "../1_LLIO/CMG_LLIO.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------

// address line defines
#define     CMG_LLIO_WriteCommand           CMG_LLIO_WriteA1
#define     CMG_LLIO_WriteData              CMG_LLIO_WriteA0
#define     CMG_LLIO_ReadStatus             CMG_LLIO_ReadA1
#define     CMG_LLIO_ReadData               CMG_LLIO_ReadA0


// T6963 commands   (* marks used)
#define     CMD_CURSOR_PTR                  0x21    // 
#define     CMD_OFFSET_REG                  0x22    // 
#define     CMD_ADDRESS_PTR                 0x24    // *

#define     CMD_TEXT_HOME_ADDR              0x40    // 
#define     CMD_TEXT_AREA                   0x41    // 
#define     CMD_GRAPHIC_HOME_ADDR           0x42    // *
#define     CMD_GRAPHIC_AREA                0x43    // *

#define     CMD_MODE_OR                     0x80    // *
#define     CMD_MODE_XOR                    0x81    // 
#define     CMD_MODE_AND                    0x83    // 
#define     CMD_MODE_TEXT_ATTRIB            0x84    // 

#define     CMD_DISPLAY_MODE_OFF            0x90    // *
#define     CMD_DISPLAY_MODE_GRAPHIC_ONLY   0x98    // *

#define     CMD_CURSOR_PATTERN_SELECT       0xa0    // 

#define     CMD_DATA_AUTO_WRITE             0xb0    // *
#define     CMD_DATA_AUTO_READ              0xb1    // 
#define     CMD_DATA_AUTO_RESET             0xb2    // *

#define     CMD_DATA_WRITE_INC              0xc0    // *
#define     CMD_DATA_READ_INC               0xc1    // *
#define     CMD_DATA_WRITE_DEC              0xc2    // *
#define     CMD_DATA_READ_DEC               0xc3    // *
#define     CMD_DATA_WRITE                  0xc4    // *
#define     CMD_DATA_READ                   0xc5    // *

#define     CMD_BIT_SETCLR                  0xf0    // *


// status bits
#define     WAIT_COMMAND_DATA_BITS          0x03
#define     WAIT_AUTOMODE_READ_BITS         0x04
#define     WAIT_AUTOMODE_WRITE_BITS        0x08
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
cmg_Coord   g_CTRL_iDisplayWidth;            // display x resolution
cmg_Coord   g_CTRL_iDisplayHeight;           // display y resolution
cmg_mu16    g_CTRL_uBytesPerLine;            // bytes per line

// controler specific
cmg_ms8     g_CTRL_iDirections;             // next selected cursor dir
cmg_ScrAddr g_CTRL_wCurrentScreenAddress;   // current address ptr
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// functions
//---------------------------------------------------------------------------


/*---------------------------------------------------------------------------
| _CTRL_StatusWait
|-----------------------------------
| wait until lcd driver is ready
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_StatusWait( void )
{
    while ( ( CMG_LLIO_ReadStatus() & WAIT_COMMAND_DATA_BITS ) != WAIT_COMMAND_DATA_BITS );
}



/*---------------------------------------------------------------------------
| _CTRL_SendCommand
|-----------------------------------
| send a command to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_SendCommand( cmg_u8 byCommand )
{
    _CTRL_StatusWait();
    CMG_LLIO_WriteCommand( byCommand );
}



/*---------------------------------------------------------------------------
| _CTRL_SendCommand_1
|-----------------------------------
| send a command with one data byte to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
|   * byData:       data byte
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_SendCommand_1( cmg_u8 byCommand, cmg_u8 byData )
{
    _CTRL_StatusWait();
    CMG_LLIO_WriteData( byData );
    _CTRL_StatusWait();
    CMG_LLIO_WriteCommand( byCommand );
}



/*---------------------------------------------------------------------------
| _CTRL_SendCommand_2
|-----------------------------------
| send a command with two data bytes to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
|   * byDataLo:     lo data byte
|   * byDataHi:     hi data byte
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_SendCommand_2( cmg_u8 byCommand, cmg_u8 byDataLo, cmg_u8 byDataHi )
{
    _CTRL_StatusWait();
    CMG_LLIO_WriteData( byDataLo );
    _CTRL_StatusWait();
    CMG_LLIO_WriteData( byDataHi );
    _CTRL_StatusWait();
    CMG_LLIO_WriteCommand( byCommand );
}



/*---------------------------------------------------------------------------
| _CTRL_SendCommand_2Word
|-----------------------------------
| send a command with a word (two data bytes) to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
|   * wData:        data word
| @Return Value:    none
+--------------------------------------------------------------------------*/
// void _CTRL_SendCommand_2Word( u8 byCommand, u16 wData )
#define     _CTRL_SendCommand_2Word(cmd,wData)       _CTRL_SendCommand_2( cmd, LO_BYTE( wData ), HI_BYTE( wData ) )



/*---------------------------------------------------------------------------
| _CTRL_SendCommand_Recv
|-----------------------------------
| send a command and receive one byte from the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byCommand:    command
| @Return Value:    data byte
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u8 _CTRL_SendCommand_Recv( cmg_u8 byCommand )
{
    _CTRL_StatusWait();
    CMG_LLIO_WriteCommand( byCommand );
    _CTRL_StatusWait();
    return CMG_LLIO_ReadData();
}



/*---------------------------------------------------------------------------
| _CTRL_WriteData_AutoMode
|-----------------------------------
| write one data byte in the auto mode to the driver
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       data byte
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _CTRL_WriteData_AutoMode( cmg_u8 byData )
{
    while ( ( CMG_LLIO_ReadStatus() & WAIT_AUTOMODE_WRITE_BITS ) != WAIT_AUTOMODE_WRITE_BITS );
    CMG_LLIO_WriteData( byData );
}



/*---------------------------------------------------------------------------
| _CTRL_ReadData_AutoMode
|-----------------------------------
| read one data byte in the auto mode from the driver
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+--------------------------------------------------------------------------*/
/* currently unused...
_PRIVATE
cmg_u8 _CTRL_ReadData_AutoMode( void )
{
    while ( ( CMG_LLIO_ReadStatus() & WAIT_AUTOMODE_READ_BITS ) != WAIT_AUTOMODE_READ_BITS );
    return CMG_LLIO_ReadData();
}
*/



//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_CTRL_Init
|-----------------------------------
| Init (init downwards: CMG_LLIO_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_Init( void )
{
    cmg_Result  iResult;

    // init CMG_LLIO_*
    iResult = CMG_LLIO_Init();
    CHECKRESULT();
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // set global variables
    g_CTRL_iDisplayWidth  = CMG_CTRL_WIDTH;
    g_CTRL_iDisplayHeight = CMG_CTRL_HEIGHT;
    g_CTRL_uBytesPerLine  = GET_BYTES_FROM_BITS( g_CTRL_iDisplayWidth );
    

    // . . . . . . . . . . . . . . . . . . . . . . .
    // init display driver
    // turn display off
    _CTRL_SendCommand( CMD_DISPLAY_MODE_OFF );
    
    // set graphic mode details
    _CTRL_SendCommand_2Word( CMD_GRAPHIC_HOME_ADDR, 0x0000 );
    _CTRL_SendCommand_2Word( CMD_GRAPHIC_AREA, g_CTRL_uBytesPerLine );
    _CTRL_SendCommand( CMD_MODE_OR );
    
    // clear screen
    CMG_CTRL_FastClearScreen( 0x00 );

    // reset cursor
    CMG_CTRL_SetAddressPtr( 0x0000 );

    // turn display on in graphics mode
    _CTRL_SendCommand( CMD_DISPLAY_MODE_GRAPHIC_ONLY );


    // . . . . . . . . . . . . . . . . . . . . . . .
    // init globals
    g_CTRL_iDirections = CTRL_DIR_RD_NONE_WR_FASTEST;
    g_CTRL_wCurrentScreenAddress = 0;


    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_CTRL_Exit();
    return iResult;
}



/****************************************************************************
| CMG_CTRL_Exit
|-----------------------------------
| Exit (exit downwards: CMG_LLIO_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_Exit( void )
{
    _CTRL_SendCommand( CMD_DISPLAY_MODE_OFF );

    // exit CMG_LLIO_*
    CMG_LLIO_Exit();
}



/****************************************************************************
| CMG_CTRL_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // turn display off
            _CTRL_SendCommand( CMD_DISPLAY_MODE_OFF );

            // on off call parent at last
            CMG_LLIO_PowerManagement( uState );
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // on on call parent first
            CMG_LLIO_PowerManagement( uState );

            // turn display on in graphics mode
            _CTRL_SendCommand( CMD_DISPLAY_MODE_GRAPHIC_ONLY );
            break;


        // just pass trough unknown
        default:
            // case CMG_PWR_BACKLIGHT_ON:
            // case CMG_PWR_BACKLIGHT_OFF:
            CMG_LLIO_PowerManagement( uState );
            break;
    }
}



/****************************************************************************
| CMG_CTRL_ControllerSpecific
|-----------------------------------
| Controller specific functions (contrast, etc.)
+----------------------------------------------------------------------------
| @Params:         
|   * uFunction:        selected feature
|   * pParam:           corresponding parameter
| @Return Value:    result, if controller supports this
+***************************************************************************/
_PUBLIC
cmg_Result CMG_CTRL_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam )
{
    // this driver supports no specific function
    return CMG_ERROR_NOTSUPPORTED;
}




/****************************************************************************
| CMG_CTRL_SetAddressPtr
|-----------------------------------
| set the address pointer
+----------------------------------------------------------------------------
| @Params:          
|   * wAddress:     address pointer to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetAddressPtr( cmg_ScrAddr wScreenAddress )
{
    _CTRL_SendCommand_2Word( CMD_ADDRESS_PTR, wScreenAddress );
    g_CTRL_wCurrentScreenAddress = wScreenAddress;
}



/****************************************************************************
| CMG_CTRL_SetDirections
|-----------------------------------
| set cursor pointer directions
+----------------------------------------------------------------------------
| @Params:          
|   * iDirections:      directions
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetDirections( cmg_ms8 iDirections )
{
    g_CTRL_iDirections = iDirections;
}



/****************************************************************************
| CMG_CTRL_SetByte
|-----------------------------------
| set a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       data byte
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_SetByte( cmg_u8 byData )
{
    switch ( g_CTRL_iDirections )
    {
        // ->
        case CTRL_DIR_RD_NONE_WR_FASTEST:
        case CTRL_DIR_RD_NONE_WR_RIGHT:
            _CTRL_SendCommand_1( CMD_DATA_WRITE_INC, byData );
            g_CTRL_wCurrentScreenAddress++;
            break;
            
        // v
        case CTRL_DIR_RD_NONE_WR_DOWN:
            _CTRL_SendCommand_1( CMD_DATA_WRITE, byData );
            g_CTRL_wCurrentScreenAddress += g_CTRL_uBytesPerLine;
            _CTRL_SendCommand_2Word( CMD_ADDRESS_PTR, g_CTRL_wCurrentScreenAddress );
            break;
        
        // this one is INVALID in write mode
        //case CTRL_DIR_RD_RIGHT_WR_INVALID:
        //    break;
    }
}



/****************************************************************************
| CMG_CTRL_GetByte
|-----------------------------------
| get a data byte and modify the address pointer accordingly
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    data byte
+***************************************************************************/
_PUBLIC
cmg_u8 CMG_CTRL_GetByte( void )
{
    cmg_u8  byData = 0;

    switch ( g_CTRL_iDirections )
    {
        // o
        case CTRL_DIR_RD_NONE_WR_FASTEST:
        case CTRL_DIR_RD_NONE_WR_RIGHT:
        case CTRL_DIR_RD_NONE_WR_DOWN:
            byData = _CTRL_SendCommand_Recv( CMD_DATA_READ );
            break;

        // ->
        case CTRL_DIR_RD_RIGHT_WR_INVALID:
            byData = _CTRL_SendCommand_Recv( CMD_DATA_READ_INC );
            g_CTRL_wCurrentScreenAddress++;
            break;
    }
    return byData;
}



/****************************************************************************
| CMG_CTRL_FastClearScreen
|-----------------------------------
| Fast fills complete screen with byCol color.
+----------------------------------------------------------------------------
| @Params:          
|   * byCol:        color
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CTRL_FastClearScreen( cmg_u8 byCol )
{
    cmg_u16     wLenBytes;
    
    // only 0x00 or 0xff is allowed
    if ( byCol )    byCol = 0xff;

    // set cursor and prepare increment
    _CTRL_SendCommand_2Word( CMD_ADDRESS_PTR, 0x0000 );
    _CTRL_SendCommand( CMD_DATA_AUTO_WRITE );
    
    // calc byte length
    wLenBytes = (cmg_u16)( g_CTRL_uBytesPerLine * g_CTRL_iDisplayHeight );

    // fill
    while(  wLenBytes-- != 0 )
        _CTRL_WriteData_AutoMode( byCol );

    _CTRL_SendCommand( CMD_DATA_AUTO_RESET );

    CMG_CTRL_SetAddressPtr( 0 );
}



//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
