//***************************************************************************
// CMG_DRV.h
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         16.11.2006
//***************************************************************************
// History:
//   25.01.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************

#ifndef CMG_DRV_H
#define CMG_DRV_H


typedef struct _cmg_DRV_PenState
{
    cmg_bool    g_DRV_bPenScaled;           // if scaled pen is used
    union {
        cmg_u8      g_DRV_byPenCurrentBit;      // current pen bit
        cmg_sfp1616 g_DRV_sfpPenCurrentBitPos;  // current fp pen bit
    };
} cmg_DRV_PenState;




//---------------------------------------------------------------------------
// Declarations      (for details see below)
//---------------------------------------------------------------------------


// ===================
// ==== Init/Exit ====
// ===================

    //-- Init/Exit ----------------------------------------------------------
        cmg_Result  CMG_DRV_Init( void );
        void        CMG_DRV_Exit( void );
        void        CMG_DRV_PowerManagement( cmg_mu8 uState );
        cmg_Result  CMG_DRV_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam );
    //-----------------------------------------------------------------------



// ===========================
// ==== BPP Calculations =====
// ===========================

    //-- byte/bit calculations ----------------------------------------------
        #define     CMG_DRV_BIT_FROM_X(x)           ( (cmg_u8) ( ( ~(x) ) & 0x07 ) )
        #define     CMG_DRV_BYTE_FROM_X(x)          ( (x) >> 3 )
        #define     CMG_DRV_FULL_BYTES_FROM_X(x)    ( GET_BYTES_FROM_BITS( x ) )
    //-----------------------------------------------------------------------



// ==========================
// ==== Mode Management =====
// ==========================

    //-- ROP ----------------------------------------------------------------
            #define     DRV_ROPMODE_COPY                0x00
            #define     DRV_ROPMODE_XOR                 0x01
            #define     DRV_ROPMODE_AND                 0x02
            #define     DRV_ROPMODE_OR                  0x03
        void        CMG_DRV_SetROPMode( cmg_mu8 uROPMode );
    //-----------------------------------------------------------------------
    
    //-- Colors / Pens ------------------------------------------------------
        void        CMG_DRV_SetSolidColor( cmg_Color SolidColor );
            #define     DRV_SCALE_NONE                  ( INT_TO_SFP( 1 ) )
        void        CMG_DRV_SetPen( cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes, cmg_sfp1616 sfpScaleFactor );
        void        CMG_DRV_SpoolPen( cmg_Coord iPixels );
        void        CMG_DRV_SpoolPenRelative( cmg_Coord iPixels );
        void        CMG_DRV_PenState_SaveRestore( cmg_DRV_PenState *pPenState_SaveTo, cmg_DRV_PenState *pPenState_RestoreFrom );
    //-----------------------------------------------------------------------



// ==================
// ==== Drawing =====
// ==================

    //-- Output -------------------------------------------------------------
        void        CMG_DRV_Pixel( cmg_Coord iX, cmg_Coord iY );
        void        CMG_DRV_HLine( cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength );
        void        CMG_DRV_VLine( cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength );
    //-----------------------------------------------------------------------

    //-- Input --------------------------------------------------------------
        void        CMG_DRV_GetHLine( cmg_u8 *pDestBuffer, cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength );
    //-----------------------------------------------------------------------


//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// details
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
