//***************************************************************************
// CMG_DRV_UNI_1BPP.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         16.11.2006
//***************************************************************************
// History:
//   25.01.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRV_CONFIGURED
    #error CMG_DRV not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRV_UNI_1BPP )
    #if CMG_CTRL_BITDEPTH != 1
        #error CMG_CTRL_BITDEPTH must be 1 to use CMG_DRV_UNI_1BPP. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRV.h"
#include    "../2_CTRL/CMG_CTRL.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
// convert x y coordinates into screen address
#define     ADDR_FROM_XY(x,y)       ( (cmg_ScrAddr) ( ( (y) * g_CTRL_uBytesPerLine ) + CMG_DRV_BYTE_FROM_X( x ) ) )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------

    //-- Modes --------------------------------------------------------------
        cmg_mu8     g_DRV_uROPMode;             // current rasterization operation mode
    //-----------------------------------------------------------------------

    //-- Color --------------------------------------------------------------
            #define     DRV_COLORMODE_SOLIDCOLOR        0x00
            #define     DRV_COLORMODE_PEN               0x01
        cmg_mu8     g_DRV_uColorMode;           // current color mode (solid - pen)

        // solid color
        cmg_Color   g_DRV_SolidColor;           // current solid color

        // pen
        cmg_u8     *g_DRV_pbyPenStartAddr;      // pen start ptr
        cmg_u8     *g_DRV_pbyPenStopAddr;       // pen stop ptr
        cmg_u8     *g_DRV_pbyPenCurrentAddr;    // current pen ptr
        cmg_u8      g_DRV_byPenCurrentBit;      // current pen bit

        // scaled pen
        cmg_bool    g_DRV_bPenScaled;           // if scaled pen is used
        cmg_sfp1616 g_DRV_sfpPenScaleFactor;    // pen scale factor
        cmg_sfp1616 g_DRV_sfpPenCurrentBitPos;  // current fp pen bit
    //-----------------------------------------------------------------------

//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


// ===================
// ==== Init/Exit ====
// ===================

/****************************************************************************
| CMG_DRV_Init
|-----------------------------------
| Init (init downwards: CMG_CTRL_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_DRV_Init( void )
{
    cmg_Result  iResult;

    // init CMG_CTRL_*
    iResult = CMG_CTRL_Init();
    CHECKRESULT();

    // init globals
    CMG_DRV_SetROPMode( DRV_ROPMODE_COPY );
    CMG_DRV_SetSolidColor( 0xff );

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_DRV_Exit();
    return iResult;
}



/****************************************************************************
| CMG_DRV_Exit
|-----------------------------------
| Exit (exit downwards: CMG_CTRL_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_Exit( void )
{
    // exit CMG_CTRL_*
    CMG_CTRL_Exit();
}



/****************************************************************************
| CMG_DRV_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|    uState:        switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // ... nothing to do ...

            // on off call parent at last
            CMG_CTRL_PowerManagement( uState );
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // on on call parent first
            CMG_CTRL_PowerManagement( uState );

            // ... nothing to do ...
            break;


        // just pass trough unknown
        default:
            // case CMG_PWR_BACKLIGHT_ON:
            // case CMG_PWR_BACKLIGHT_OFF:
            CMG_CTRL_PowerManagement( uState );
            break;
    }
}



/****************************************************************************
| CMG_DRV_ControllerSpecific
|-----------------------------------
| Controller specific functions (contrast, etc.)
+----------------------------------------------------------------------------
| @Params:         
|   * uFunction:        selected feature
|   * pParam:           corresponding parameter
| @Return Value:    result, if controller supports this
+***************************************************************************/
_PUBLIC
cmg_Result CMG_DRV_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam )
{
    // just handle through
    return CMG_CTRL_ControllerSpecific( uFunction, pParam );
}





// ==========================
// ==== Mode Management =====
// ==========================



/****************************************************************************
| CMG_DRV_SetROPMode
|-----------------------------------
| Set rop mode
+----------------------------------------------------------------------------
| @Params:         
|   * byROPMode:        rop mode to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_SetROPMode( cmg_mu8 uROPMode )
{
    // set rop mode
    g_DRV_uROPMode = uROPMode;
}



/****************************************************************************
| CMG_DRV_SetSolidColor
|-----------------------------------
| Set draw mode to pen and initializes pen
+----------------------------------------------------------------------------
| @Params:         
|   * SolidColor:       color to select
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_SetSolidColor( cmg_Color SolidColor )
{
    // set color mode
    g_DRV_uColorMode = DRV_COLORMODE_SOLIDCOLOR;
    // set solid color
    g_DRV_SolidColor = SolidColor;
}



/****************************************************************************
| CMG_DRV_SetPen
|-----------------------------------
| Set draw mode to scaled pen and initializes pen
+----------------------------------------------------------------------------
| @Params:         
|   * pbyPenStartAddr:  pen start address
|   * uPenLengthBytes:  pen length in bytes
|   * sfpScaleFactor:   pen scale or DRV_SCALE_NONE
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_SetPen( cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes, cmg_sfp1616 sfpScaleFactor )
{
    // set color mode
    g_DRV_uColorMode       = DRV_COLORMODE_PEN;

    // set pen
    g_DRV_pbyPenStartAddr   = pbyPenStartAddr;
    g_DRV_pbyPenStopAddr    = pbyPenStartAddr + uPenLengthBytes;
    g_DRV_pbyPenCurrentAddr = pbyPenStartAddr;

    if ( sfpScaleFactor == DRV_SCALE_NONE )
    {
        // set normal pen
        g_DRV_byPenCurrentBit   = 7;
        g_DRV_bPenScaled        = false;
    }
    else
    {
        // set scaled pen
        g_DRV_bPenScaled            = true;
        g_DRV_sfpPenScaleFactor     = sfpScaleFactor;
        g_DRV_sfpPenCurrentBitPos   = INT_TO_SFP( 8 ) - 1;
    }
}





/****************************************************************************
| CMG_DRV_PenSpool
|-----------------------------------
| spool pen position (absolute)
+----------------------------------------------------------------------------
| @Params:         
|   * iPixels:      number of pixels
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_SpoolPen( cmg_Coord iPixels )
{
    // . . . . . . . . . . . . . . . . . . . . . . . . .
    // reset bit position

    // reset addr
    g_DRV_pbyPenCurrentAddr = g_DRV_pbyPenStartAddr;
    
    // reset bit pos
    if ( !g_DRV_bPenScaled )
        g_DRV_byPenCurrentBit   = 7;
    else
        g_DRV_sfpPenCurrentBitPos   = INT_TO_SFP( 8 ) - 1;

    
    // spool to position
    CMG_DRV_SpoolPenRelative( iPixels );
}




/****************************************************************************
| CMG_DRV_SpoolPenRelative
|-----------------------------------
| spool pen position forward or reward
+----------------------------------------------------------------------------
| @Params:         
|   * iPixels:      number of pixels
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_SpoolPenRelative( cmg_Coord iPixels )
{
    cmg_mu16    uBytes;


    // . . . . . . . . . . . . . . . . . . . . . . . . .
    // spool forward
    if ( iPixels > 0 )
    {
        if ( !g_DRV_bPenScaled )
        {
            // normal pen

            // get bits and bytes to spool
            uBytes = CMG_DRV_BYTE_FROM_X( iPixels );

            // calc bit
            g_DRV_byPenCurrentBit -= ( iPixels * CMG_CTRL_BITDEPTH ) & 0x07;
        
            // on byte overflow (should be on <0)
            if ( g_DRV_byPenCurrentBit > 7 )
            {
                g_DRV_byPenCurrentBit += 8;
                g_DRV_pbyPenCurrentAddr++;
            }
        }
        else
        {
            // scaled pen

            // adjust next bit position (sub scale factor)
            g_DRV_sfpPenCurrentBitPos -= g_DRV_sfpPenScaleFactor * iPixels;

            uBytes = 0;

            // if the byte is used up
            if  ( g_DRV_sfpPenCurrentBitPos < 0 )
            {
                // invert
                g_DRV_sfpPenCurrentBitPos = -g_DRV_sfpPenCurrentBitPos;

                // calc number of bytes
                uBytes = ( cmg_mu16 ) ( ( SFP_TO_INT( g_DRV_sfpPenCurrentBitPos ) >> 3 ) + 1 );
                // calc new bit position
                g_DRV_sfpPenCurrentBitPos = INT_TO_SFP( 8 ) - ( g_DRV_sfpPenCurrentBitPos & ( INT_TO_SFP( 8 ) - 1 ) );
            }
        }

        // adjust byte pos
        g_DRV_pbyPenCurrentAddr += uBytes;

        // adjust pen address
        while ( ( g_DRV_pbyPenCurrentAddr < g_DRV_pbyPenStartAddr ) || ( g_DRV_pbyPenCurrentAddr >= g_DRV_pbyPenStopAddr ) )
            g_DRV_pbyPenCurrentAddr -= g_DRV_pbyPenStopAddr - g_DRV_pbyPenStartAddr;
    }

    // . . . . . . . . . . . . . . . . . . . . . . . . .
    // spool backwards
    if ( iPixels < 0 )
    {
        // adjust number of pixels
        iPixels = -iPixels;

        if ( !g_DRV_bPenScaled )
        {
            // normal pen

            // get bits and bytes to spool
            uBytes = CMG_DRV_BYTE_FROM_X( iPixels );

            // calc bit
            g_DRV_byPenCurrentBit += ( iPixels * CMG_CTRL_BITDEPTH ) & 0x07;
        
            // on byte overflow
            if ( g_DRV_byPenCurrentBit > 7 )
            {
                g_DRV_byPenCurrentBit -= 8;
                g_DRV_pbyPenCurrentAddr--;
            }
        }
        else
        {
            // scaled pen

            // adjust next bit position (sub scale factor)
            g_DRV_sfpPenCurrentBitPos += g_DRV_sfpPenScaleFactor * iPixels;

            uBytes = 0;

            // if the byte is used up
            if  ( g_DRV_sfpPenCurrentBitPos >= INT_TO_SFP( 8 ) )
            {
                // calc number of bytes
                uBytes = ( (cmg_mu16)( SFP_TO_INT( g_DRV_sfpPenCurrentBitPos ) ) ) >> 3;
                // calc new bit position
                g_DRV_sfpPenCurrentBitPos = g_DRV_sfpPenCurrentBitPos & ( INT_TO_SFP( 8 ) - 1 );
            }
        }

        // adjust byte pos
        g_DRV_pbyPenCurrentAddr -= uBytes;

        // adjust pen address
        while ( ( g_DRV_pbyPenCurrentAddr < g_DRV_pbyPenStartAddr ) || ( g_DRV_pbyPenCurrentAddr >= g_DRV_pbyPenStopAddr ) )
            g_DRV_pbyPenCurrentAddr += g_DRV_pbyPenStopAddr - g_DRV_pbyPenStartAddr;
    }
}





/****************************************************************************
| CMG_DRV_PenState_SaveRestore
|-----------------------------------
| spool pen position forward or reward
+----------------------------------------------------------------------------
| @Params:         
|   * pPenState_SaveTo:         save penstate to (can be NULL, then ignored)
|   * pPenState_RestoreFrom:    restore penstate from (can be NULL, then ignored)
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_PenState_SaveRestore( cmg_DRV_PenState *pPenState_SaveTo, cmg_DRV_PenState *pPenState_RestoreFrom )
{
    // save current state
    if ( pPenState_SaveTo )
    {
        pPenState_SaveTo->g_DRV_bPenScaled = g_DRV_bPenScaled;
        if ( !g_DRV_bPenScaled )
            // normal pen
            pPenState_SaveTo->g_DRV_byPenCurrentBit = g_DRV_byPenCurrentBit;
        else
            // scaled pen
            pPenState_SaveTo->g_DRV_sfpPenCurrentBitPos = g_DRV_sfpPenCurrentBitPos;
    }

    // save current state
    if ( pPenState_RestoreFrom )
    {
        g_DRV_bPenScaled = pPenState_RestoreFrom->g_DRV_bPenScaled;
        if ( !g_DRV_bPenScaled )
            // normal pen
            g_DRV_byPenCurrentBit = pPenState_RestoreFrom->g_DRV_byPenCurrentBit;
        else
            // scaled pen
            g_DRV_sfpPenCurrentBitPos = pPenState_RestoreFrom->g_DRV_sfpPenCurrentBitPos;
    }
}




// ========================
// ==== ROP Functions =====
// ========================


/*---------------------------------------------------------------------------
| _DRV_ROP_Mask
|-----------------------------------
| Rasterization Operation
| apply the bitmask and the global color 
| with the selected ropmode to the data byte
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       data byte
|   * byBitMask:    bit mask
| @Return Value:    modified data byte
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u8  _DRV_ROP_MaskSolidColor( cmg_u8 byData, cmg_u8 byBitMask )
{
    // manipulate pixels
    switch ( g_DRV_uROPMode )
    {
        case DRV_ROPMODE_COPY:
            if ( g_DRV_SolidColor )
                byData |= byBitMask;
            else
                byData &= ~byBitMask;
            break;

        case DRV_ROPMODE_XOR:
            byData ^= byBitMask;
            break;

        case DRV_ROPMODE_OR:
            if ( g_DRV_SolidColor )
                byData |= byBitMask;
            break;

        case DRV_ROPMODE_AND:
            if ( !g_DRV_SolidColor )
                byData &= ~byBitMask;
            break;
    }
    return byData;
}



/*---------------------------------------------------------------------------
| _DRV_ROP_MaskSource
|-----------------------------------
| Rasterization Operation
| apply the bitmask and the source
| with the selected ropmode to the data byte
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       input data byte
|   * byBitMask:    bit mask
|   * bySource:     source byte
| @Return Value:    modified data byte
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u8  _DRV_ROP_MaskSource( cmg_u8 byData, cmg_u8 byBitMask, cmg_u8 bySource )
{
    // manipulate pixels
    switch ( g_DRV_uROPMode )
    {
        case DRV_ROPMODE_COPY:
            byData = ( bySource & byBitMask ) | ( byData & ( ~byBitMask ) );
            break;

        case DRV_ROPMODE_XOR:
            byData = ( bySource & byBitMask ) ^ byData;
            break;

        case DRV_ROPMODE_OR:
            byData = ( bySource & byBitMask ) | byData;
            break;

        case DRV_ROPMODE_AND:
            byData = ( bySource | ( ~byBitMask ) ) & byData;
            break;
    }
    return byData;
}



/*---------------------------------------------------------------------------
| _DRV_ROP_MaskSource
|-----------------------------------
| Rasterization Operation
| apply the source on the complete byte
| with the selected ropmode to the data byte
+----------------------------------------------------------------------------
| @Params:          
|   * byData:       input data byte
|   * bySource:     source byte
| @Return Value:    modified data byte
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u8  _DRV_ROP_Source( cmg_u8 byData, cmg_u8 bySource )
{
    // manipulate pixels
    switch ( g_DRV_uROPMode )
    {
        case DRV_ROPMODE_COPY:
            byData = bySource;
            break;

        case DRV_ROPMODE_XOR:
            byData = bySource ^ byData;
            break;

        case DRV_ROPMODE_OR:
            byData = bySource | byData;
            break;

        case DRV_ROPMODE_AND:
            byData = bySource & byData;
            break;
    }
    return byData;
}





// ==================================
// ==== GetBits / Pen Functions =====
// ==================================


/*---------------------------------------------------------------------------
| _DRV_GetBits
|-----------------------------------
| Get next byLenght source bits from pen. Upper unused bits are NOT defined!
| If you need the exact data you must AND the result with the amount of bits.
+----------------------------------------------------------------------------
| @Params:          
|   * byLength:     number of bits to get (maximum == 8)
| @Return Value:    data bits (right aligned) 
+--------------------------------------------------------------------------*/
_PRIVATE
_INLINE_FORCE
cmg_u8 _DRV_GetBits_Normal( cmg_u8 byLength )
{
    cmg_u8  byData, byHelper;

    // helper var
    byHelper = g_DRV_byPenCurrentBit + 1;

    // can we get all needed bits from the current byte?
    if ( byHelper >= byLength )
    {
        // get and align bits
        byData = *g_DRV_pbyPenCurrentAddr >> ( byHelper - byLength );
        // update bit position
        g_DRV_byPenCurrentBit -= byLength;
        // if all bits of this byte are used
        if ( g_DRV_byPenCurrentBit > 7 )    // underflow
        {
            // next byte with reset on pen end
            g_DRV_byPenCurrentBit = 7;
            if ( ++g_DRV_pbyPenCurrentAddr == g_DRV_pbyPenStopAddr )
                g_DRV_pbyPenCurrentAddr = g_DRV_pbyPenStartAddr;
        }
    }
    else
    {
        // we need to get the bits from two bytes
        // modify helper
        byHelper = byLength - byHelper;
        // get first half
        byData = *g_DRV_pbyPenCurrentAddr << byHelper;
        // next byte with reset on pen end
        if ( ++g_DRV_pbyPenCurrentAddr == g_DRV_pbyPenStopAddr )
            g_DRV_pbyPenCurrentAddr = g_DRV_pbyPenStartAddr;
        // get second half
        byData |= *g_DRV_pbyPenCurrentAddr >> ( 8 - byHelper );
        // update bit position
        g_DRV_byPenCurrentBit = g_DRV_byPenCurrentBit + 8 - byLength;
    }

    // return bits
    return byData;
}




/*---------------------------------------------------------------------------
| _DRV_GetBits
|-----------------------------------
| Get next byLenght source bits from pen. Upper unused bits are NOT defined!
| If you need the exact data you must AND the result with the amount of bits.
+----------------------------------------------------------------------------
| @Params:          
|   * byLength:     number of bits to get (maximum == 8)
| @Return Value:    data bits (right aligned) 
+--------------------------------------------------------------------------*/
_PRIVATE
_INLINE_FORCE
cmg_u8 _DRV_GetBits_Scaled( cmg_u8 byLength )
{
    cmg_u8  byData  ;

    // reset result
    byData = 0;
    
    // for each bit to get
    while ( byLength-- )
    {
        // shift current data byte left, and fill bit0 with the next source bit (g_DRV_sfpPenCurrentBitPos)
        byData = ( byData << 1 ) | ( ( *g_DRV_pbyPenCurrentAddr >> SFP_TO_INT( g_DRV_sfpPenCurrentBitPos ) ) & 0x01 );

        // adjust next bit position (sub scale factor)
        g_DRV_sfpPenCurrentBitPos -= g_DRV_sfpPenScaleFactor;

        // if the source byte is used
        while ( g_DRV_sfpPenCurrentBitPos < 0 )
        {
            // adjust bit position (add an new byte)
            g_DRV_sfpPenCurrentBitPos += INT_TO_SFP( 8 );

            // get next source byte with reset on pen end
            if ( ++g_DRV_pbyPenCurrentAddr == g_DRV_pbyPenStopAddr )
                g_DRV_pbyPenCurrentAddr = g_DRV_pbyPenStartAddr;
        }
    }

    // return bits
    return byData;
}




/*---------------------------------------------------------------------------
| _DRV_GetBits
|-----------------------------------
| Get next byLenght source bits from pen. Upper unused bits are NOT defined!
| If you need the exact data you must AND the result with the amount of bits.
+----------------------------------------------------------------------------
| ! Returns normal pen bits or scaled pen bits according to the current mode.
+----------------------------------------------------------------------------
| @Params:          
|   * byLength:     number of bits to get (maximum == 8)
| @Return Value:    data bits (right aligned) 
+--------------------------------------------------------------------------*/
_PRIVATE
cmg_u8 _DRV_GetBits( cmg_u8 byLength )
{
    if ( !g_DRV_bPenScaled )
        return ( _DRV_GetBits_Normal( byLength ) );
    else
        return ( _DRV_GetBits_Scaled( byLength ) );
}






// ==================
// ==== Drawing =====
// ==================



/****************************************************************************
| CMG_DRV_Pixel
|-----------------------------------
| Set a pixel at x/y with the color
+----------------------------------------------------------------------------
| @Params:          
|   * iX:           x coord
|   * iY:           y coord
|   * byColor:      color
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_Pixel( cmg_Coord iX, cmg_Coord iY )
{
    cmg_u8      byBit, byBitMask;

    // set directions
    CMG_CTRL_SetDirections( CTRL_DIR_RD_NONE_WR_FASTEST );
  
    // set address
    CMG_CTRL_SetAddressPtr( ADDR_FROM_XY( iX, iY ) );

    // get bit / bitmask
    byBit = CMG_DRV_BIT_FROM_X( iX );
    byBitMask = BIT( byBit );

    // draw
    if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
        CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), byBitMask ) );
    else
        CMG_CTRL_SetByte( _DRV_ROP_MaskSource( CMG_CTRL_GetByte(), byBitMask, (cmg_u8) ( _DRV_GetBits( 1 ) << byBit ) ) );
}




/****************************************************************************
| CMG_DRV_HLine
|-----------------------------------
| draws a horizantal line rightwards (increasing x)
| (optimized)
+----------------------------------------------------------------------------
| @Params:   
|   * iX:           x start point
|   * iY:           y start point
|   * iLength:      number of pixels to draw rightwards
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_HLine( cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength )
{
    cmg_u8      byDestAddrBit, byBitMask, bySource;

    // set directions
    CMG_CTRL_SetDirections( CTRL_DIR_RD_NONE_WR_RIGHT );

    // set address
    CMG_CTRL_SetAddressPtr( ADDR_FROM_XY( iX, iY ) );

    // get bit
    byDestAddrBit = CMG_DRV_BIT_FROM_X( iX );

    // small line fitting in one byte
    if ( ( ((cmg_Coord)byDestAddrBit) - iLength ) >= 0 )
    {
        // detailed pixel mask calculation:
        // byBitMask = ( ( 1 << ( byBit + 1 ) ) - 1 ) - ( ( 1 << ( byBit + 1 - iLength ) ) - 1 );
        // optimized:
        byBitMask = ( BIT( byDestAddrBit ) - BIT( byDestAddrBit - iLength ) ) << 1;

        // draw
        if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
        {   // solid color
            CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), byBitMask ) );
        }
        else
        {   // pen
            bySource = _DRV_GetBits( (cmg_u8) iLength ) << ( byDestAddrBit + 1 - iLength );
            CMG_CTRL_SetByte( _DRV_ROP_MaskSource( CMG_CTRL_GetByte(), byBitMask, bySource ) );
        }
        // _END_ line
    }
    else
    {
        // first byte (align on 8 bit)
        if ( byDestAddrBit != 7 )
        {
            // draw
            byBitMask = ( 1 << ( byDestAddrBit + 1 ) ) - 1;
            if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
            {   // solid color
                CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), byBitMask ) );
            }
            else
            {   // pen
                bySource = _DRV_GetBits( (cmg_u8)( byDestAddrBit + 1 ) );
                CMG_CTRL_SetByte( _DRV_ROP_MaskSource( CMG_CTRL_GetByte(), byBitMask, bySource ) );
            }

            iLength -= byDestAddrBit + 1;
        }

        // calc number of mid bytes, store temp in iX
        iX = CMG_DRV_BYTE_FROM_X( iLength );
        // calc number of end bits, store temp in iLength
        iLength &= 0x07;

        // mid bytes (aligned on 8 bit)
        // rastermode?
        if ( g_DRV_uROPMode == DRV_ROPMODE_COPY )
        {
            // copy rastermode
            // draw
            if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
            {   // solid color
                bySource = g_DRV_SolidColor ? 0xff : 0x00;
                while ( iX-- )
                    CMG_CTRL_SetByte( bySource );
            }
            else
            {   // pen
                while ( iX-- )
                    CMG_CTRL_SetByte( _DRV_GetBits( 8 ) );
            }
        }
        else
        {
            // other rastermodes
            // draw
            if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
            {   // solid color
                while ( iX-- )
                    CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), 0xff ) );
            }
            else
            {   // pen
                while ( iX-- )
                    CMG_CTRL_SetByte( _DRV_ROP_Source( CMG_CTRL_GetByte(), _DRV_GetBits( 8 ) ) );
            }
        }
    
        // last byte 
        if ( iLength > 0 )
        {
            // draw
            byBitMask = -( 1 << ( 8 - iLength ) );
            if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
            {   // solid color
                CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), byBitMask ) );
            }
            else
            {   // pen
                bySource = _DRV_GetBits( (cmg_u8) iLength ) << ( 8 - iLength );
                CMG_CTRL_SetByte( _DRV_ROP_MaskSource( CMG_CTRL_GetByte(), byBitMask, bySource ) );
            }
        }
    }
}




/****************************************************************************
| CMG_DRV_VLine
|-----------------------------------
| draws a vertical line downwards (increasing y)
| (optimized)
+----------------------------------------------------------------------------
| @Params:   
|   * iX:           x start point
|   * iY:           y start point
|   * iLength:      number of pixels to draw downwards
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_VLine( cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength )
{
    cmg_u8      byBit, byBitMask;
    
    // set directions
    CMG_CTRL_SetDirections( CTRL_DIR_RD_NONE_WR_DOWN );

    // set address
    CMG_CTRL_SetAddressPtr( ADDR_FROM_XY( iX, iY ) );

    // get bit
    byBit = CMG_DRV_BIT_FROM_X( iX );
    byBitMask = BIT( byBit );

    // draw
    if ( g_DRV_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
    {
        while ( (iLength--) > 0 )
            CMG_CTRL_SetByte( _DRV_ROP_MaskSolidColor( CMG_CTRL_GetByte(), byBitMask ) );
    }
    else
    {
        while ( (iLength--) > 0 )
            CMG_CTRL_SetByte( _DRV_ROP_MaskSource( CMG_CTRL_GetByte(), byBitMask, (cmg_u8) ( _DRV_GetBits( 1 ) << byBit ) ) );
    }
}




/****************************************************************************
| CMG_DRV_GetHLine
|-----------------------------------
| Get screen bits into memory buffer
+----------------------------------------------------------------------------
| @Params:   
|   * pDestBuffer:  ptr to destination buffer that receives bits (make sure
|                   it large enough)
|   * iX:           x start point
|   * iY:           y start point
|   * iLength:      number of pixels get
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRV_GetHLine( cmg_u8 *pDestBuffer, cmg_Coord iX, cmg_Coord iY, cmg_Coord iLength )
{
    cmg_u8      byBit;

    // set directions
    CMG_CTRL_SetDirections( CTRL_DIR_RD_RIGHT_WR_INVALID );

    // set address
    CMG_CTRL_SetAddressPtr( ADDR_FROM_XY( iX, iY ) );

    // get bit
    byBit = CMG_DRV_BIT_FROM_X( iX );

    // calc number of bytes to process, store back in iLength
    iLength = CMG_DRV_FULL_BYTES_FROM_X( iLength );


    // if screen x is byte aligned we can just copy byte for byte
    if ( byBit == 7 )
    {
        // for all bytes
        while ( iLength-- )
            // store dest byte
            *pDestBuffer++ = CMG_CTRL_GetByte();
    }
    // if screen x is _NOT_ byte aligned we must compound the destination 
    // byte by shifting the two screen bytes
    else
    {
        cmg_u8      byDestByte, byScreenByte, byShiftLeftBits;

        // calculate shift bit counts (left and right (stored in byByt to safe one local var))
        byShiftLeftBits  = 7 - byBit;
        byBit++;

        // get first screen byte
        byScreenByte = CMG_CTRL_GetByte();

        // for all bytes
        while ( iLength-- )
        {
            // adjust first half
            byDestByte = byScreenByte << byShiftLeftBits;
            // get next screen byte
            byScreenByte = CMG_CTRL_GetByte();
            // adjust second half
            byDestByte |= byScreenByte >> byBit;
            // store dest byte
            *pDestBuffer++ = byDestByte;
        }
    }
}



//---------------------------------------------------------------------------


#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
