//***************************************************************************
// CMG_DRAW.h
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.12.2006
//***************************************************************************
// History:
//   ??.12.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************

#ifndef CMG_DRAW_H
#define CMG_DRAW_H



typedef struct _cmg_Pen     {
    cmg_u8     *pbyPenStartAddr;            // pen start addr
    cmg_mu16    uPenLengthBytes;            // number of pen bytes
    cmg_sfp1616 sfpPenScaleFactor;          // pen scale factor
    cmg_Coord   iPenSpoolPixels;            // pen spool pixels
} cmg_Pen;



typedef struct  _cmg_Brush  {
    cmg_u8     *pbyBrushStartAddr;          // brush start addr
    cmg_mu16    uBrushLineLengthBytes;      // number of brush bytes per line
    cmg_mu16    uBrushLines;                // number of brush line
    cmg_sfp1616 sfpBrushScaleFactorX;       // brush scale factor x
    cmg_sfp1616 sfpBrushScaleFactorY;       // brush scale factor y
    cmg_Coord   iBrushSpoolPixelsX;         // brush spool pixels x
    cmg_Coord   iBrushSpoolPixelsY;         // brush spool pixels y
} cmg_Brush;



typedef struct  _cmg_Image  {
    cmg_u8     *pbyImageStartAddr;          // image start addr
    cmg_Coord   iImageWidth;                // image x
    cmg_Coord   iImageHeight;               // image y
    cmg_mu16    uImageLineLengthBytes;      // number of image bytes per line
} cmg_Image;



//---------------------------------------------------------------------------
// Declarations      (for details see below)
//---------------------------------------------------------------------------

// ===================
// ==== Init/Exit ====
// ===================

    //-- Init/Exit ----------------------------------------------------------
        cmg_Result  CMG_DRAW_Init( void );
        void        CMG_DRAW_Exit( void );
        void        CMG_PowerManagement( cmg_mu8 uState );
        cmg_Result  CMG_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam );
    //-----------------------------------------------------------------------


// =======================
// ==== Mode Exports =====
// =======================

        void        CMG_GetScreenSize( cmg_Coord *piScreenSizeX, cmg_Coord *piScreenSizeY );


    //-- Display Orientation ------------------------------------------------
            #define     DISPLAYORIENTATION_NORMAL       0x00
            #define     DISPLAYORIENTATION_XYSWAP       0x01
            #define     DISPLAYORIENTATION_XMIRROR      0x02
            #define     DISPLAYORIENTATION_YMIRROR      0x04
            #define     DISPLAYORIENTATION_ROTATE90     ( DISPLAYORIENTATION_XYSWAP | DISPLAYORIENTATION_XMIRROR )
            #define     DISPLAYORIENTATION_ROTATE180    ( DISPLAYORIENTATION_XMIRROR | DISPLAYORIENTATION_YMIRROR )
            #define     DISPLAYORIENTATION_ROTATE270    ( DISPLAYORIENTATION_XYSWAP | DISPLAYORIENTATION_YMIRROR )
        void        CMG_SetDisplayOrientation( cmg_mu8 byDisplayOrientationMask );
        cmg_mu8     CMG_GetDisplayOrientation( void );
    //-----------------------------------------------------------------------

    //-- Clipping -----------------------------------------------------------
        void        CMG_SetClippingWindow( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 );
        void        CMG_GetClippingWindow( cmg_Coord *piX1, cmg_Coord *piY1, cmg_Coord *piX2, cmg_Coord *piY2 );
    //-----------------------------------------------------------------------

    //-- ROP ----------------------------------------------------------------
            #define     ROPMODE_COPY                    0x00
            #define     ROPMODE_XOR                     0x01
            #define     ROPMODE_AND                     0x02
            #define     ROPMODE_OR                      0x03
        void        CMG_SetROPMode( cmg_mu8 uROPMode );
    //-----------------------------------------------------------------------

    //-- Drawstyle / Color / Pen / Brush ------------------------------------
            #define     DRAWSTYLE_BORDER                0x01
            #define     DRAWSTYLE_FILL                  0x02
            #define     DRAWSTYLE_BOTH                  0x03
        void        CMG_SetDrawStyle( cmg_mu8 uDrawStyle );
        void        CMG_SetColor( cmg_Color Color );
        void        CMG_SetFillColor( cmg_Color FillColor );
        void        CMG_SetPen( cmg_Pen *pPen );
        void        CMG_SetBrush( cmg_Brush *pBrush );
        void        CMG_CreatePen( cmg_Pen *pPen, cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes );
        void        CMG_CreatePenEx( cmg_Pen *pPen, cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes, cmg_Coord iPenSpoolPixels, float fPenScaleFactor );
        void        CMG_CreateBrush( cmg_Brush *pBrush, cmg_u8 *pbyBrushStartAddr, cmg_mu16 uBrushLineLengthBytes, cmg_mu16 uBrushLines );
        void        CMG_CreateBrushEx( cmg_Brush *pBrush, cmg_u8 *pbyBrushStartAddr, cmg_mu16 uBrushLineLengthBytes, cmg_mu16 uBrushLines, cmg_Coord iBrushSpoolPixelsX, cmg_Coord iBrushSpoolPixelsY, float fBrushScaleFactorX, float fBrushScaleFactorY );
    //-----------------------------------------------------------------------

        
// ==========================
// ==== Drawing Exports =====
// ==========================

    //-- Pixel / Line / Wideless --------------------------------------------
        void        CMG_Pixel( cmg_Coord iX, cmg_Coord iY );

            #define     LINESUPPRESS_NONE               0x00
            #define     LINESUPPRESS_FIRST              0x01
            #define     LINESUPPRESS_LAST               0x02
        #define     CMG_Line(iX1,iY1,iX2,iY2)           CMG_LineEx( iX1, iY1, iX2, iY2, LINESUPPRESS_NONE )
        void        CMG_LineEx( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_u8 uSuppressPixelMask );
        void        CMG_LineList( cmg_Coord (*paiCoords)[2], cmg_mu16 uNumberOfPoints, cmg_bool bCloseList );
        void        CMG_LineWidth( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iWidth );
        //?? Arc
    //-----------------------------------------------------------------------

    //-- Solids -------------------------------------------------------------
        void        CMG_Rectangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 );
        void        CMG_Triangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iX3, cmg_Coord iY3 );
        #define     CMG_Circle(iXCenter,iYCenter,iRadius)           CMG_Ellipse( iXCenter, iYCenter, iRadius, iRadius )
        void        CMG_Ellipse( cmg_Coord iXCenter, cmg_Coord iYCenter, cmg_Coord iRadiusX, cmg_Coord iRadiusY );
        void        CMG_RoundedRectangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iRadius );
    //-----------------------------------------------------------------------

    //-- Blitting -----------------------------------------------------------
        void        CMG_CreateImage( cmg_Image *pImage, cmg_u8 *pbyImageStartAddr, cmg_Coord iImageWidth, cmg_Coord iImageHeight, cmg_mu16 uImageLineLengthBytes );
        void        CMG_Image( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Image *pImage );
        void        CMG_ImageTruncated( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iSizeX, cmg_Coord iSizeY, cmg_Image *pImage, cmg_Coord iSrcStartX, cmg_Coord iSrcStartY );
        void        CMG_ImageScaled( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iDestSizeX, cmg_Coord iDestSizeY, cmg_Image *pImage );
        
        cmg_mu16    CMG_GetScreen( cmg_u8 *pbyDestStartAddr, cmg_mu16 uDestAvailableBytes, cmg_mu16 *puDestBytesPerLine, cmg_Coord iSrcX, cmg_Coord iSrcY, cmg_Coord iSizeX, cmg_Coord iSizeY );
        void        CMG_Scroll( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iSrcX, cmg_Coord iSrcY, cmg_Coord iSizeX, cmg_Coord iSizeY );
    //-----------------------------------------------------------------------


//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// details
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
