//***************************************************************************
// CMG_DRAW_Ellipse.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.03.2007
//***************************************************************************
// History:
//   07.03.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_ELLIPSE )
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "CMG_DRAW_Ellipse.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------






/*
  ------------------
  Ellipse algorithm explainations:
  ------------------

_Ellipse_Inside:
    cmg_Coord       iCount, iX_Right, iX_Left, iY_Top, iY_Bottom;
    cmg_CoordLong   iDeltaX, iDeltaY, iRot, iRotX, iRotY;


    iDeltaX = ( iRadiusX * iRadiusX ) << 1;
    iDeltaY = ( iRadiusY * iRadiusY ) << 1;


    iX_Left   = iMidX;
    iX_Right  = iMidX;
    iY_Top    = iMidY - iRadiusY;
    iY_Bottom = iMidY + iRadiusY;

    iRot  = iRadiusX * iRadiusX * iRadiusY;
    iRotX = 0;
    iRotY = iRot << 1;
    iCount  = iRadiusY;


    while ( iCount > 0 )
    {
        _DRAW_ClippedPixel( iX_Right, iY_Top );
        _DRAW_ClippedPixel( iX_Left, iY_Bottom );

        if ( iRot > 0 )
        {
            iX_Left--;
            iX_Right++;
            iRotX += iDeltaY;
            iRot -= iRotX;
        }
        if ( iRot <= 0 )
        {
            iY_Top++;   
            iY_Bottom--;
            iRotY -= iDeltaX;
            iRot += iRotY;
            iCount--;
        }
  
        _DRAW_ClippedPixel( iX_Right, iY_Bottom );
        _DRAW_ClippedPixel( iX_Left, iY_Top );
    }




_Ellipse_Outside:
    cmg_Coord       iCount, iX_Right, iX_Left, iY_Top, iY_Bottom;
    cmg_CoordLong   iDeltaX, iDeltaY, iRot, iRotX, iRotY;


    iDeltaX = ( iRadiusX * iRadiusX ) << 1;
    iDeltaY = ( iRadiusY * iRadiusY ) << 1;

    iX_Left   = iMidX - iRadiusX;
    iX_Right  = iMidX + iRadiusX;
    iY_Top    = iMidY;
    iY_Bottom = iMidY;

    iRot  = iRadiusY * iRadiusY * iRadiusX;
    iRotX = iRot << 1;
    iRotY = 0;
    iCount  = iRadiusX;


    while ( iCount > 0 )
    {
        _DRAW_ClippedPixel( iX_Right, iY_Top );
        _DRAW_ClippedPixel( iX_Left, iY_Bottom );

        if ( iRot > 0 )
        {
            iY_Bottom--;
            iY_Top++;
            iRotY += iDeltaX;
            iRot -= iRotY;
        }
        if ( iRot <= 0 )
        {
            iX_Right--;   
            iX_Left++;
            iRotX -= iDeltaY;
            iRot += iRotX;
            iCount--;
        }
  
        _DRAW_ClippedPixel( iX_Right, iY_Bottom );
        _DRAW_ClippedPixel( iX_Left, iY_Top );
    }





Shortened Ellipse Loops:
------------------------

_Ellipse_Inside:
    _Ellipse_OneTimeInit( &EllData, iRadiusX, iRadiusY );
    _Ellipse_LoopInside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );

    while ( EllData.iCount > 0 )
    {
        _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Top );
        _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Bottom );

        _Ellipse_LoopInside( &EllData );

        _Ellipse_ClippedPixel( EllData.iX_Right, EllData.iY_Bottom );
        _Ellipse_ClippedPixel( EllData.iX_Left, EllData.iY_Top );
    }




_Ellipse_Outside:
    _Ellipse_OneTimeInit( &EllData, iRadiusX, iRadiusY );
    _Ellipse_LoopOutside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );

    while ( EllData.iCount > 0 )
    {
        _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Top );
        _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Bottom );

        _Ellipse_LoopOutside( &EllData );

        _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Bottom );
        _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Top );
    }


*/




//---------------------------------------------------------------------------
// privates
//---------------------------------------------------------------------------


/*---------------------------------------------------------------------------
| _Ellipse_OneTimeInit
|-----------------------------------
| set deltas, must be only done once since this vales won't be changed
+----------------------------------------------------------------------------
| @Params:         
|   * pEllData:     local ellipse data structure
|   * iRadiusX:     x radius
|   * iRadiusY:     y radius
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _Ellipse_OneTimeInit( cmg_DRAW_Ellipse_Data *pEllData, cmg_Coord iRadiusX, cmg_Coord iRadiusY )
{
    // set deltas, must be only done once since this vales won't be changed
    pEllData->iDeltaX = ( iRadiusX * iRadiusX ) << 1;
    pEllData->iDeltaY = ( iRadiusY * iRadiusY ) << 1;
}



/*---------------------------------------------------------------------------
| _Ellipse_LoopInside_Init
|-----------------------------------
| init start positions and internal variables
| must be done before EACH loop
| LoopInside means to draw the ellipse from the top to mid and from the 
| bottom to the mid simulatiously
+----------------------------------------------------------------------------
| @Params:         
|   * pEllData:     local ellipse data structure
|   * iMidX:        midpoint x
|   * iMidY:        midpoint y
|   * iRadiusX:     x radius
|   * iRadiusY:     y radius
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _Ellipse_LoopInside_Init( cmg_DRAW_Ellipse_Data *pEllData, cmg_Coord iMidX, cmg_Coord iMidY, cmg_Coord iRadiusX, cmg_Coord iRadiusY )
{
    // set start positions
    pEllData->iX_Left   = iMidX;
    pEllData->iX_Right  = iMidX;
    pEllData->iY_Top    = iMidY - iRadiusY;
    pEllData->iY_Bottom = iMidY + iRadiusY;

    // set internal variables
    // pEllData->iRot      = iRadiusX * iRadiusX * iRadiusY;    // !!! is the same as:
    pEllData->iRot      = ( pEllData->iDeltaX >> 1 ) * iRadiusY;
    pEllData->iRotX     = 0;
    pEllData->iRotY     = pEllData->iRot << 1;
    pEllData->iCount    = iRadiusY;
}



/*---------------------------------------------------------------------------
| _Ellipse_LoopOutside_Init
|-----------------------------------
| init start positions and internal variables
| must be done before EACH loop
| LoopOutside means to draw the ellipse from the mid to top and from the 
| mid to the bottom simulatiously (other direction than LoopInside)
+----------------------------------------------------------------------------
| @Params:         
|   * pEllData:     local ellipse data structure
|   * iMidX:        midpoint x
|   * iMidY:        midpoint y
|   * iRadiusX:     x radius
|   * iRadiusY:     y radius
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _Ellipse_LoopOutside_Init( cmg_DRAW_Ellipse_Data *pEllData, cmg_Coord iMidX, cmg_Coord iMidY, cmg_Coord iRadiusX, cmg_Coord iRadiusY )
{
    // set start positions
    pEllData->iX_Left   = iMidX - iRadiusX;
    pEllData->iX_Right  = iMidX + iRadiusX;
    pEllData->iY_Top    = iMidY;
    pEllData->iY_Bottom = iMidY;

    // set internal variables
    // pEllData->iRot      = iRadiusY * iRadiusY * iRadiusX;    // !!! is the same as:
    pEllData->iRot      = ( pEllData->iDeltaY >> 1 ) * iRadiusX;
    pEllData->iRotX     = pEllData->iRot << 1;
    pEllData->iRotY     = 0;
    pEllData->iCount    = iRadiusX;
}



/*---------------------------------------------------------------------------
| _Ellipse_LoopInside
|-----------------------------------
| inner ellipse loop to update to the next pixel
| see example above to see usage
+----------------------------------------------------------------------------
| @Params:         
|   * pEllData:     local ellipse data structure
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _Ellipse_LoopInside( cmg_DRAW_Ellipse_Data *pEllData )
{
    // inner ellipse loop to update to the next pixel
    if ( pEllData->iRot > 0 )
    {
        pEllData->iX_Left--;
        pEllData->iX_Right++;
        pEllData->iRotX += pEllData->iDeltaY;
        pEllData->iRot  -= pEllData->iRotX;
    }
    if ( pEllData->iRot <= 0 )
    {
        pEllData->iY_Top++;   
        pEllData->iY_Bottom--;
        pEllData->iRotY -= pEllData->iDeltaX;
        pEllData->iRot  += pEllData->iRotY;
        pEllData->iCount--;
    }
}



/*---------------------------------------------------------------------------
| _Ellipse_LoopOutside
|-----------------------------------
| inner ellipse loop to update to the next pixel
| see example above to see usage
+----------------------------------------------------------------------------
| @Params:         
|   * pEllData:     local ellipse data structure
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _Ellipse_LoopOutside( cmg_DRAW_Ellipse_Data *pEllData )
{
    // inner ellipse loop to update to the next pixel
    if ( pEllData->iRot > 0 )
    {
        pEllData->iY_Bottom++;
        pEllData->iY_Top--;
        pEllData->iRotY += pEllData->iDeltaX;
        pEllData->iRot  -= pEllData->iRotY;
    }                   
    if ( pEllData->iRot <= 0 )
    {
        pEllData->iX_Right--;   
        pEllData->iX_Left++;
        pEllData->iRotX -= pEllData->iDeltaY;
        pEllData->iRot  += pEllData->iRotX;
        pEllData->iCount--;
    }
}




//---------------------------------------------------------------------------







//---------------------------------------------------------------------------
// exports
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_Ellipse
|-----------------------------------
| ellipse algorithm (bresenham)
+----------------------------------------------------------------------------
| @Params:          
|   * iMidX:        x midpoint
|   * iMidY:        y midpoint
|   * iRadiusX:     x radius
|   * iRadiusY:     y radius
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_Ellipse( cmg_Coord iMidX, cmg_Coord iMidY, cmg_Coord iRadiusX, cmg_Coord iRadiusY )
{
    cmg_DRAW_Ellipse_Data       EllData;


    // display orientation
    _DRAW_AdjustXY( &iMidX, &iMidY );



    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // param check
    // invalid radii?
    if ( ( iRadiusX < 0 ) || ( iRadiusY < 0 ) )
        return;
    
    // simple line or pixel to draw
    if ( iRadiusX == 0 ) 
    {
        // no x radius
        g_DRAW_bProhibitCoordAdjustment = true;
        
        if ( iRadiusY == 0 )
            CMG_Pixel( iMidX, iMidY );
        else
            CMG_Line( iMidX, (cmg_Coord)( iMidY - iRadiusY ), iMidX, (cmg_Coord)( iMidY + iRadiusY ) );

        g_DRAW_bProhibitCoordAdjustment = false;
        return;
    }
    else if ( iRadiusY == 0 ) 
    {
        // no y radius
        g_DRAW_bProhibitCoordAdjustment = true;
        CMG_Line( (cmg_Coord)( iMidX - iRadiusX ), iMidY, (cmg_Coord)( iMidX + iRadiusX ), iMidY );
        g_DRAW_bProhibitCoordAdjustment = false;
        return;
    }



    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // one time data structure init
    _Ellipse_OneTimeInit( &EllData, iRadiusX, iRadiusY );




    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // fill inside
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_FILL )
    {
        cmg_Coord       iSpoolStartX;
        // last loop values
        cmg_Coord       iXLastLeft, iXLastRight, iYLast;


        // set brush/solid
        _DRAW_ApplyBrush( 0 );


        // init spool start value
        iSpoolStartX = iMidX - iRadiusX;
        // when to draw a border we have to shift the brush one inside
        if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
            iSpoolStartX++;



        // . . . . . . . . . . . . . . . . . . 
        // fill upper half of ellipse
        // init ellipse loop
        _Ellipse_LoopInside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );
        // init last loop values
        iXLastLeft  = EllData.iX_Left;
        iXLastRight = EllData.iX_Right;
        iYLast      = EllData.iY_Top;

        // if we have to draw a borter too we only fill the inside without border but use a different technique
        // requiring one line less
        if ( g_DRAW_uDrawStyle == DRAWSTYLE_BOTH )
            EllData.iCount--;

        // for all pixels
        while ( EllData.iCount > 0 )
        {
            // update positions
            _Ellipse_LoopInside( &EllData );

            // if line has changed
            if ( iYLast != EllData.iY_Top ) 
            {
                // if we also draw the border only fill the inside, else the complete ellipse
                if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
                    _DRAW_ClippedHLine( (cmg_Coord)( EllData.iX_Left + 1 ), (cmg_Coord)( EllData.iX_Right - 1 ), EllData.iY_Top, iSpoolStartX );
                else
                    _DRAW_ClippedHLine( iXLastLeft, iXLastRight, iYLast, iSpoolStartX );

                // store old line
                iYLast      = EllData.iY_Top;
            }

            // store old x coords
            iXLastLeft  = EllData.iX_Left;
            iXLastRight = EllData.iX_Right;
        }


        // . . . . . . . . . . . . . . . . . . 
        // fill lower half of ellipse
        // init ellipse loop
        _Ellipse_LoopOutside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );
        // init last loop values
        iXLastLeft  = EllData.iX_Left;
        iXLastRight = EllData.iX_Right;
        iYLast      = EllData.iY_Top;
        
        // if we _DON'T_ have to draw a borter too we do have to draw an extra line due the useage of a different technique
        if ( g_DRAW_uDrawStyle != DRAWSTYLE_BOTH )
            _DRAW_ClippedHLine( EllData.iX_Left, EllData.iX_Right, EllData.iY_Bottom, iSpoolStartX );


        // for all pixels
        while ( EllData.iCount > 0 )
        {
            // update positions
            _Ellipse_LoopOutside( &EllData );

            // if line has changed
            if ( iYLast != EllData.iY_Bottom )
            {
                // if we also draw the border only fill the inside, else the complete ellipse
                if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
                    _DRAW_ClippedHLine( (cmg_Coord)( iXLastLeft + 1 ), (cmg_Coord)( iXLastRight - 1 ), iYLast, iSpoolStartX );
                else
                    _DRAW_ClippedHLine( EllData.iX_Left, EllData.iX_Right, EllData.iY_Bottom, iSpoolStartX );

                // store old line
                iYLast      = EllData.iY_Bottom;
            }

            // store old x coords
            iXLastLeft  = EllData.iX_Left;
            iXLastRight = EllData.iX_Right;
        }
    }



    
    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // draw border
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
    {
        cmg_mu8                     uLoop;


        // set pen/solid
        _DRAW_ApplyPen( 0 );



        // . . . . . . . . . . . . . . . . . . . . . . . . .
        // only solid color 
        // (can parallely draw pixels)
        if ( g_DRAW_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
        {
            // init ellipse loop
            _Ellipse_LoopInside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );

            // for all pixels
            while ( EllData.iCount > 0 )
            {
                // draw the first two quadrants simultaneously
                _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Top );
                _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Bottom );

                // update loop
                _Ellipse_LoopInside( &EllData );
  
                // draw the last two quadrants simultaneously
                _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Bottom );
                _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Top );
           }
        }



        // . . . . . . . . . . . . . . . . . . . . . . . . .
        // with pen 
        // (must draw pixels in order to prevent pen)
        else
        {
            // we need two passes to draw all quadrants in order
            for ( uLoop = 0; uLoop < 2; uLoop++ )
            {
                // draw: (uLoop==0): Q1  /  (uLoop==1): Q3
                // init ellipse loop
                _Ellipse_LoopInside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );
                // for all pixels
                while ( EllData.iCount > 0 )
                {
                    if ( uLoop == 0 )   // Q1
                        _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Top );
                    else                // Q3
                        _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Bottom );

                    _Ellipse_LoopInside( &EllData );
                }

                // draw: (uLoop==0): Q2  /  (uLoop==1): Q4
                // init ellipse loop
                _Ellipse_LoopOutside_Init( &EllData, iMidX, iMidY, iRadiusX, iRadiusY );
                // for all pixels
                while ( EllData.iCount > 0 )
                {
                    if ( uLoop == 0 )   // Q2
                        _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Bottom );
                    else                // Q4
                        _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Top );
                    _Ellipse_LoopOutside( &EllData );
                }
            }
        }
    }
}

                    




/****************************************************************************
| CMG_Circle
|-----------------------------------
| circle algorithm draws an ellipse with equal radii
+----------------------------------------------------------------------------
| @Params:          
|   * iXMid:        x midpoint
|   * iYMid:        y midpoint
|   * iRadius:      radius
| @Return Value:    none
+***************************************************************************/
/*
  Defined in header file:
    #define     CMG_Circle(iXCenter,iYCenter,iRadius)          CMG_Ellipse( iXCenter, iYCenter, iRadius, iRadius )

_PUBLIC
void CMG_Circle( cmg_Coord iXMid, cmg_Coord iYMid, cmg_Coord iRadius )
{
    CMG_Ellipse( iXCenter, iYCenter, iRadius, iRadius );
}
*/




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
