//***************************************************************************
// CMG_DRAW_GetScreen.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         11.02.2007
//***************************************************************************
// History:
//   11.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_GETSCREEN )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_GetScreen
|-----------------------------------
| Get bitmap from screen
+----------------------------------------------------------------------------
| @Params:         
|   * pbyDestStartAddr:     destination start address (can be NULL, then the
|                           needed memory bytes will be returned in 
|                           *puMemSizeBytes)
|   * uDestAvailableBytes:  available memory bytes in buffer
|   * puDestBytesPerLine:   linesize in bytes (out only, can be NULL)
|   * iSrcX:                x start 
|   * iSrcY:                y start
|   * iSizeX:               x size to draw
|   * iSizeY:               y size to draw
| @Return Value:    actually needed memory bytes in buffer. If an error 
|                   occures (insufficent mem or clipping error) the return 
|                   value will be 0)
+***************************************************************************/
_PUBLIC
cmg_mu16 CMG_GetScreen( cmg_u8 *pbyDestStartAddr, cmg_mu16 uDestAvailableBytes, cmg_mu16 *puDestBytesPerLine, cmg_Coord iSrcX, cmg_Coord iSrcY, cmg_Coord iSizeX, cmg_Coord iSizeY )
{
    cmg_mu16    uNeededMem, uBytesPerLineSize;

    // display orientation
    _DRAW_AdjustXY( &iSrcX, &iSrcY );
    
    // adjust x size
    if ( iSizeX < 0 )
    {
        iSrcX -= iSizeX;
        iSizeX = -iSizeX;
    }
    // adjust y size
    if ( iSizeY < 0 )
    {
        iSrcY -= iSizeY;
        iSizeY = -iSizeY;
    }

    // clipping (regardless of current clipwindow, clip on whole screen)
    if ( ( iSrcX < 0 ) || ( iSrcY < 0 ) || 
        ( ( iSrcX + iSizeX ) >= g_DRAW_iScreenSizeX ) || 
        ( ( iSrcY + iSizeY ) >= g_DRAW_iScreenSizeY ) ) 
    {
        // return 0 for error
        return 0;
    }


    // calc linesize and needed bytes
    uBytesPerLineSize = CMG_DRV_FULL_BYTES_FROM_X( iSizeX );
    uNeededMem = uBytesPerLineSize * iSizeY;

    // if caller wants to know the linesize in bytes
    if ( puDestBytesPerLine )
        *puDestBytesPerLine = uBytesPerLineSize;

    // no destination addr given? just return needed size
    if ( !pbyDestStartAddr )
        return uNeededMem;
    
    // check for enough memsize; return 0 for error
    if ( uDestAvailableBytes < uNeededMem )
        return 0;


    // for every line
    while ( iSizeY-- )
    {
        // set pen
        CMG_DRV_GetHLine( pbyDestStartAddr, iSrcX, iSrcY++, iSizeX );

        // update destination start (next line)
        pbyDestStartAddr += uBytesPerLineSize;
    }

    // return needed mem
    return uNeededMem;
}




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
