//***************************************************************************
// CMG_DRAW_Image.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         04.03.2007
//***************************************************************************
// History:
//   05.03.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // this draw file will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------




typedef struct _cmg_DRAW_Image_StateBackup {
    cmg_Brush       *pBrush;
    cmg_sfp1616     sfpBrush_CurrentLine;
    cmg_mu8         uDrawStyle;
} cmg_DRAW_Image_StateBackup;





/*---------------------------------------------------------------------------
| _Image_StateBackup_Save
|-----------------------------------
| make backup of all changed values due image drawing
+----------------------------------------------------------------------------
| @Params:         
|   * pStateBackup: buffer
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _Image_StateBackup_Save( cmg_DRAW_Image_StateBackup *pStateBackup )
{
    // save current brush and its state
    pStateBackup->pBrush = g_DRAW_pBrush;
    pStateBackup->sfpBrush_CurrentLine = g_DRAW_sfpBrushCurrentLine;
    // save current drawstyle
    pStateBackup->uDrawStyle = g_DRAW_uDrawStyle;
}





/*---------------------------------------------------------------------------
| _Image_StateBackup_Restore
|-----------------------------------
| restore backup of all changed values due image drawing
+----------------------------------------------------------------------------
| @Params:         
|   * pStateBackup: buffer
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PRIVATE
void _Image_StateBackup_Restore( cmg_DRAW_Image_StateBackup *pStateBackup )
{
    // restore old brush and its state
    CMG_SetBrush( pStateBackup->pBrush );
    g_DRAW_sfpBrushCurrentLine = pStateBackup->sfpBrush_CurrentLine;
    // restore draw style
    CMG_SetDrawStyle( pStateBackup->uDrawStyle );
}




      

/****************************************************************************
| CMG_CreateImage
|-----------------------------------
| bit blitting
+----------------------------------------------------------------------------
| @Params:         
|   * iDestX:               destination x point
|   * iDestY:               destination y point
|   * pImage:               image
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CreateImage( cmg_Image *pImage, cmg_u8 *pbyImageStartAddr, cmg_Coord iImageWidth, cmg_Coord iImageHeight, cmg_mu16 uImageLineLengthBytes )
{
    pImage->pbyImageStartAddr       = pbyImageStartAddr;
    pImage->iImageWidth             = iImageWidth;
    pImage->iImageHeight            = iImageHeight;
    pImage->uImageLineLengthBytes   = uImageLineLengthBytes;
}





/****************************************************************************
| CMG_Image
|-----------------------------------
| bit blitting
+----------------------------------------------------------------------------
| @Params:         
|   * iDestX:               destination x point
|   * iDestY:               destination y point
|   * pImage:               image
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_Image( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Image *pImage )
{
    cmg_DRAW_Image_StateBackup  StateBackup;
    cmg_Brush                   ImageBrush;


   
    // . . . . . . . . . . . . . . . .
    // save state
    _Image_StateBackup_Save( &StateBackup );



    // . . . . . . . . . . . . . . . .
    // brush and style
    // create image brush
    CMG_CreateBrush( &ImageBrush, pImage->pbyImageStartAddr, pImage->uImageLineLengthBytes, pImage->iImageHeight );

    // set brush
    CMG_SetBrush( &ImageBrush );

    // set draw style
    CMG_SetDrawStyle( DRAWSTYLE_FILL );



    // . . . . . . . . . . . . . . . .
    // draw 
    CMG_Rectangle( iDestX, iDestY, (cmg_Coord)( iDestX + pImage->iImageWidth - 1 ), (cmg_Coord)( iDestY + pImage->iImageHeight - 1 ) );



    // . . . . . . . . . . . . . . . .
    // restore state
    _Image_StateBackup_Restore( &StateBackup );
}





/****************************************************************************
| CMG_ImageTruncated
|-----------------------------------
| bit blitting
+----------------------------------------------------------------------------
| @Params:         
|   * iDestX:               destination x point
|   * iDestY:               destination y point
|   * iSizeX:               x truncated size to draw
|   * iSizeY:               y size to draw
|   * pImage:               image
|   * iSrcStartX:           x start (spool)
|   * iSrcStartY:           y start (spool)
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_ImageTruncated( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iSizeX, cmg_Coord iSizeY, cmg_Image *pImage, cmg_Coord iSrcStartX, cmg_Coord iSrcStartY )
{
    cmg_DRAW_Image_StateBackup  StateBackup;
    cmg_Brush                   ImageBrush;


   
    // . . . . . . . . . . . . . . . .
    // save state
    _Image_StateBackup_Save( &StateBackup );



    // . . . . . . . . . . . . . . . .
    // brush and style
    // create image brush
    CMG_CreateBrush( &ImageBrush, pImage->pbyImageStartAddr, pImage->uImageLineLengthBytes, pImage->iImageHeight );
    ImageBrush.iBrushSpoolPixelsX = iSrcStartX;
    ImageBrush.iBrushSpoolPixelsY = iSrcStartY;

    // set brush
    CMG_SetBrush( &ImageBrush );

    // set draw style
    CMG_SetDrawStyle( DRAWSTYLE_FILL );



    // . . . . . . . . . . . . . . . .
    // draw 
    CMG_Rectangle( iDestX, iDestY, (cmg_Coord)( iDestX + iSizeX - 1 ), (cmg_Coord)( iDestY + iSizeY - 1 ) );



    // . . . . . . . . . . . . . . . .
    // restore state
    _Image_StateBackup_Restore( &StateBackup );
}





/****************************************************************************
| CMG_BltScaled
|-----------------------------------
| scaled bit blitting
+----------------------------------------------------------------------------
| @Params:         
|   * iDestX:               destination x point
|   * iDestY:               destination y point
|   * iDestSizeX:           x size to draw
|   * iDestSizeY:           y size to draw
|   * pImage:               image
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_ImageScaled( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iDestSizeX, cmg_Coord iDestSizeY, cmg_Image *pImage )
{
    cmg_DRAW_Image_StateBackup  StateBackup;
    cmg_Brush                   ImageBrush;


   
    // . . . . . . . . . . . . . . . .
    // save state
    _Image_StateBackup_Save( &StateBackup );



    // . . . . . . . . . . . . . . . .
    // brush and style
    // create image brush
    CMG_CreateBrush( &ImageBrush, pImage->pbyImageStartAddr, pImage->uImageLineLengthBytes, pImage->iImageHeight );
    ImageBrush.sfpBrushScaleFactorX = INT_TO_SFP( pImage->iImageWidth  ) / iDestSizeX;
    ImageBrush.sfpBrushScaleFactorY = INT_TO_SFP( pImage->iImageHeight ) / iDestSizeY;

    // set brush
    CMG_SetBrush( &ImageBrush );

    // set draw style
    CMG_SetDrawStyle( DRAWSTYLE_FILL );



    // . . . . . . . . . . . . . . . .
    // draw 
    CMG_Rectangle( iDestX, iDestY, (cmg_Coord)( iDestX + iDestSizeX - 1 ), (cmg_Coord)( iDestY + iDestSizeY - 1 ) );



    // . . . . . . . . . . . . . . . .
    // restore state
    _Image_StateBackup_Restore( &StateBackup );
}



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
