//***************************************************************************
// CMG_DRAW_Line.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         11.02.2007
//***************************************************************************
// History:
//   11.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // this draw file will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "CMG_DRAW_Line.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// predef
//---------------------------------------------------------------------------
_PRIVATE    
_INLINE_FORCE
void     _DRAW_BresenhamLine( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_u8 bySuppressPixelMask, cmg_Coord iClipSpool );
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_Line
|-----------------------------------
| draw a line
| implenented as macro in .h file
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          x1 coord
|   * iY1:          y1 coord
|   * iX2:          x2 coord
|   * iY2:          y2 coord
| @Return Value:    none
+***************************************************************************/
/*_PUBLIC
void CMG_Line( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 )
{
    implenented as macro in .h file:
    #define     CMG_Line(iX1,iY1,iX2,iY2 )          CMG_LineEx( iX1, iY1, iX2, iY2, DRV_LINESUPPRESS_NONE )
}
*/




/****************************************************************************
| CMG_Line
|-----------------------------------
| draw a line
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          x1 coord
|   * iY1:          y1 coord
|   * iX2:          x2 coord
|   * iY2:          y2 coord
|   * bySuppressPixelMask:  hide first and/or last pixel
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LineEx( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_u8 uSuppressPixelMask )
{
    cmg_Coord   idX, idY;
    cmg_Coord   iClipSpool;

    // display orientation
    _DRAW_AdjustXY( &iX1, &iY1 );
    _DRAW_AdjustXY( &iX2, &iY2 );


    // if to suppress first pixel we must spool one more
    iClipSpool = 0;
    if ( uSuppressPixelMask & LINESUPPRESS_FIRST )
        iClipSpool++;

    
    // get deltas
    idX = iX2 - iX1;
    idY = iY2 - iY1;
    

    // HLine ?
    if ( idY == 0 )
    {
        // clip y
        if ( ( iY1 < g_DRAW_iClippingWindow_TY ) || ( iY1 > g_DRAW_iClippingWindow_BY ) )
            return;

        // sort x values and suppress pixels
        if ( idX >= 0 )
        {
            // handle suppress pixels
            if ( uSuppressPixelMask & LINESUPPRESS_FIRST )
                iX1++;
            if ( uSuppressPixelMask & LINESUPPRESS_LAST )
                iX2--;
        }
        else
        {
            // x1 > x2 (wrong direction)
            // swap x1/x2 (y2 is temp)
            iY2 = iX2;
            iX2 = iX1;
            iX1 = iY2;
            
            // handle suppress pixels (must be inverted)
            if ( uSuppressPixelMask & LINESUPPRESS_FIRST )
                iX2--;
            if ( uSuppressPixelMask & LINESUPPRESS_LAST )
                iX1++;
        }

        // clip
        if ( g_DRAW_iClippingWindow_LX > iX1 )
        {
            // update spool (since we must clip)
            iClipSpool += g_DRAW_iClippingWindow_LX - iX1;
            // set new x
            iX1 = g_DRAW_iClippingWindow_LX;
        }
        iX2 = MIN( iX2, g_DRAW_iClippingWindow_RX );
        
        // recalc dx
        idX = iX2 - iX1;

        // nothing to draw anymore?
        if ( idX < 0 )
            return;

        // apply pen 
        _DRAW_ApplyPen( iClipSpool );

        // only one pixel?
        if ( idX == 0 )
        {
            CMG_DRV_Pixel( iX1, iY1 );
            return;
        }

        // draw HLine (dx + 1 pixels)
        CMG_DRV_HLine( iX1, iY1, (cmg_Coord)( idX + 1 ) );

        return;
    }
    

    // VLine ?
    if ( idX == 0 )
    {
        // clip x
        if ( ( iX1 < g_DRAW_iClippingWindow_LX ) || ( iX1 > g_DRAW_iClippingWindow_RX ) )
            return;

        // sort y values and suppress pixels
        if ( idY >= 0 )
        {
            // handle suppress pixels
            if ( uSuppressPixelMask & LINESUPPRESS_FIRST )
                iY1++;
            if ( uSuppressPixelMask & LINESUPPRESS_LAST )
                iY2--;
        }
        else
        {
            // y1 > y2 (wrong direction)
            // swap y1/y2 (x2 is temp)
            iX2 = iY2;
            iY2 = iY1;
            iY1 = iX2;
            
            // handle suppress pixels (must be inverted)
            if ( uSuppressPixelMask & LINESUPPRESS_FIRST )
                iY2--;
            if ( uSuppressPixelMask & LINESUPPRESS_LAST )
                iY1++;
        }

        // clip
        if ( g_DRAW_iClippingWindow_TY > iY1 )
        {
            // update spool (since we must clip)
            iClipSpool += g_DRAW_iClippingWindow_TY - iY1;
            // set new x
            iY1 = g_DRAW_iClippingWindow_TY;
        }
        iY2 = MIN( iY2, g_DRAW_iClippingWindow_BY );
        
        // recalc dx
        idY = iY2 - iY1;

        // nothing to draw anymore?
        if ( idY < 0 )
            return;

        // apply pen 
        _DRAW_ApplyPen( iClipSpool );
        
        // only one pixel?
        if ( idY == 0 )
        {
            CMG_DRV_Pixel( iX1, iY1 );
            return;
        }

        // draw HLine (dx + 1 pixels)
        CMG_DRV_VLine( iX1, iY1, (cmg_Coord)( idY + 1 ) );

        return;
    }


    // draw normal line
    _DRAW_BresenhamLine( iX1, iY1, iX2, iY2, uSuppressPixelMask, iClipSpool );
}










/*---------------------------------------------------------------------------
| _Line_Loop_Init
|-----------------------------------
| init start positions and internal variables
| must be done before EACH loop
+----------------------------------------------------------------------------
| @Params:         
|   * pLineData:    local line data structure
|   * iX1           x1 point (start)
|   * iY1           y1 point (start)
|   * iX2           x2 point (stop)
|   * iY2           y2 point (stop)
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED
void _Line_Loop_Init( cmg_DRAW_Line_Data *pLineData, cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 )
{
    // . . . . . . . . . . . . . . . . . . . . . .
    // set internal variables

    // swap x/y coords if dy > dx
    {
        // caluclate preliminary deltas (speedup only)
        cmg_Coord   iDeltaX = iX2 - iX1;
        cmg_Coord   iDeltaY = iY2 - iY1;

        // calculate: "bSwapXY = abs( iDeltaY ) > abs( iDeltaX );"  and swap it if its true
        if ( ( pLineData->bSwapXY = ( ( iDeltaY >= 0 ) ? iDeltaY : -iDeltaY ) > ( ( iDeltaX >= 0 ) ? iDeltaX : -iDeltaX ) ) != 0 )
        {
            // swap x and y (use iDeltaX as temp)
            SWAP( iX1, iY1, iDeltaX );
            SWAP( iX2, iY2, iDeltaX );
        }

        // explanation:
        // if (bSwapXY == false):   Long->X, Short->Y
        //    (bSwapXY == true ):   Long->Y, Short->X
    }



    // caluclate deltas and increment
    // long dir
    if ( iX1 <= iX2 )
    {
        pLineData->iDeltaLong = iX2 - iX1;
        pLineData->iIncLong   = 1;
    }
    else 
    {
        pLineData->iDeltaLong = iX1 - iX2;
        pLineData->iIncLong   = -1;
    }

    // short dir
    if ( iY1 <= iY2)
    {
        pLineData->iDeltaShort = iY2 - iY1;
        pLineData->iIncShort   = 1;
    }
    else 
    {
        pLineData->iDeltaShort = iY1 - iY2;
        pLineData->iIncShort   = -1;
    }

    // calc start drift value
    pLineData->iDrift = pLineData->iDeltaLong >> 1;

    // calc number of pixels
    pLineData->iCount = pLineData->iDeltaLong + 1;

    // set positions    
    pLineData->iLong  = iX1;
    pLineData->iShort = iY1;
    

    
    // . . . . . . . . . . . . . . . . . . . . . .
    // set start positions
    if ( !pLineData->bSwapXY )
    {
        pLineData->iX = iX1;
        pLineData->iY = iY1;
    }
    else 
    {
        pLineData->iX = iY1;
        pLineData->iY = iX1;
    }
}






/*---------------------------------------------------------------------------
| _Line_Loop
|-----------------------------------
| inner line loop to update to the next pixel
| see example above to see usage
+----------------------------------------------------------------------------
| @Params:         
|   * pLineData:    local line data structure
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED
void _Line_Loop( cmg_DRAW_Line_Data *pLineData )
{
    // inner line loop to update to the next pixel

    // long dir adjust (always)
    pLineData->iLong  += pLineData->iIncLong;

    // short dir adjust (only step if needed)
    pLineData->iDrift -= pLineData->iDeltaShort;
    if ( pLineData->iDrift < 0 )
    {
        pLineData->iShort += pLineData->iIncShort;
        pLineData->iDrift += pLineData->iDeltaLong;
    }

    // update positions depending on swap state
    if ( !pLineData->bSwapXY )
    {
        // x is long dir
        pLineData->iX = pLineData->iLong;
        pLineData->iY = pLineData->iShort;
    }
    else 
    {
        // y is long dir
        pLineData->iX = pLineData->iShort;
        pLineData->iY = pLineData->iLong;
    }

    // dec count
    pLineData->iCount--;
}







/*---------------------------------------------------------------------------
| _DRAW_BresenhamLine
|-----------------------------------
| optimized line algorithm (ideas from Bresenham's algorithm)
+----------------------------------------------------------------------------
| @Params:         
|   * iX1           x1 point (start)
|   * iY1           y1 point (start)
|   * iX2           x2 point (stop)
|   * iY2           y2 point (stop)
|   * bySuppressPixelMask   mask with wich pixels should be suppressed
| @Return Value:    none
+--------------------------------------------------------------------------*/
/* ### UNIVERSAL LINE ALGORITHM ### */
_PRIVATE
_INLINE_FORCE
void _DRAW_BresenhamLine( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_u8 bySuppressPixelMask, cmg_Coord iClipSpool )
{
    cmg_DRAW_Line_Data  LineData;                         // local line data
    cmg_bool            bNeedClipping, bInVisibleWindow;  // clipping helpers



    // . . . . . . . . . . . . . . . . . . . .
    // init line loop
    _Line_Loop_Init( &LineData, iX1, iY1, iX2, iY2 );



    // . . . . . . . . . . . . . . . . . . . .
    // init clipping
    // first assume best case (no clipping and start pos in visible window)
    bNeedClipping = false;
    bInVisibleWindow = true;
    // do we need clipping ?
    if ( ( iX1 < g_DRAW_iClippingWindow_LX ) || ( iX1 > g_DRAW_iClippingWindow_RX ) || ( iY1 < g_DRAW_iClippingWindow_TY ) || ( iY1 > g_DRAW_iClippingWindow_BY ) )
    {
        bNeedClipping = true;
        bInVisibleWindow = false;
    }
    if ( ( iX2 < g_DRAW_iClippingWindow_LX ) || ( iX2 > g_DRAW_iClippingWindow_RX ) || ( iY2 < g_DRAW_iClippingWindow_TY ) || ( iY2 > g_DRAW_iClippingWindow_BY ) )
    {
        bNeedClipping = true;
    }

    
    
    // . . . . . . . . . . . . . . . . . . . .
    // pixel suppressing
    // suppress last pixel ?
    if ( bySuppressPixelMask & LINESUPPRESS_LAST )
        if ( !LineData.iCount-- )
            return;

    // suppress first pixel ?
    if ( bySuppressPixelMask & LINESUPPRESS_FIRST )
    {
        _DRAW_ApplyPen( 1 );
        goto _SuppressFirstPixel;
    }

    

    // . . . . . . . . . . . . . . . . . . . .
    // apply pen 
    _DRAW_ApplyPen( iClipSpool );



    // . . . . . . . . . . . . . . . . . . . .
    // main pixel loop
    while ( 1 ) 
    {
        // draw pixel
        if ( !bNeedClipping )
        {
            CMG_DRV_Pixel( LineData.iX, LineData.iY );
        }
        else
        {
            // if we don't found out before: check if we are in the visible window
            if ( !bInVisibleWindow )
            {
                // if we are still outside
                if ( ( LineData.iX < g_DRAW_iClippingWindow_LX ) || ( LineData.iX > g_DRAW_iClippingWindow_RX ) || ( LineData.iY < g_DRAW_iClippingWindow_TY ) || ( LineData.iY > g_DRAW_iClippingWindow_BY ) )
                {
                    iClipSpool++;
                }
                // if we get inside now
                else
                {
                    // enable drawing
                    bInVisibleWindow = true;
                    // spool pen correctly
                    _DRAW_ApplyPen( iClipSpool );
                }
            }

            // if we are in the visible window
            if ( bInVisibleWindow )
            {
                // if we get outside the visible window quit
                if ( ( LineData.iX < g_DRAW_iClippingWindow_LX ) || ( LineData.iX > g_DRAW_iClippingWindow_RX ) || ( LineData.iY < g_DRAW_iClippingWindow_TY ) || ( LineData.iY > g_DRAW_iClippingWindow_BY ) )
                    return;

                // we are inside our clipping window
                CMG_DRV_Pixel( LineData.iX, LineData.iY );
            }
        }


_SuppressFirstPixel:
        // line end condition
        if ( LineData.iCount <= 1 )
            break;

        // calc next line pixel
        _Line_Loop( &LineData );
    }
}





#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
