//***************************************************************************
// CMG_DRAW_Line.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         20.02.2007
//***************************************************************************
// History:
//   20.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_LINELIST )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
// other:
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_LineList
|-----------------------------------
| draw a continous line from a list
+----------------------------------------------------------------------------
| @Params:          
|   * paiCoords:        pointer to an array with points. Each point is an 
|                       array of cmg_Coord[2] - [0] = x and [1] = y. This
|                       means you can access it like paiCoords[pointnr][0/1]
|   * uNumberOfPoints:  number of points in array. 1 = one pixel, 
|                       2 = one line, 3 = two lines (0-1, 1-2)
|   * bCloseList:       if to close the list. This means to draw the line 
|                       from the last point to the first: 
|                       -> line( point[ uNumberOfPoints - 1 ], point[ 0 ] )
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LineList( cmg_Coord (*paiCoords)[2], cmg_mu16 uNumberOfPoints, cmg_bool bCloseList )
{
    cmg_mu16    uNr;

    // nothing to draw
    if ( uNumberOfPoints <= 0 )
        return;

    // only one point
    if ( uNumberOfPoints == 1 )
    {
        CMG_Pixel( paiCoords[0][0], paiCoords[0][1] );
        return;
    }

    // draw all mid parts
    for ( uNr = 1; uNr < ( uNumberOfPoints - 1 ); uNr++ )
        CMG_LineEx( paiCoords[ uNr - 1 ][0], paiCoords[ uNr - 1 ][1], paiCoords[ uNr ][0], paiCoords[ uNr ][1], LINESUPPRESS_LAST );

    // close list (connect last point with first point)
    if ( bCloseList )
    {
        // draw last part in list 
        CMG_LineEx( paiCoords[ uNr - 1 ][0], paiCoords[ uNr - 1 ][1], paiCoords[ uNr ][0], paiCoords[ uNr ][1], LINESUPPRESS_LAST );
        // connect last and fist one
        CMG_LineEx( paiCoords[ uNr ][0], paiCoords[ uNr ][1], paiCoords[0][0], paiCoords[0][1], LINESUPPRESS_LAST );
    }
    else
    {
        // draw last part and don't suppress last pixel!
        CMG_LineEx( paiCoords[ uNr - 1 ][0], paiCoords[ uNr - 1 ][1], paiCoords[ uNr ][0], paiCoords[ uNr ][1], LINESUPPRESS_NONE );
    }
}



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
