//***************************************************************************
// CMG_DRAW_LineWidth.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         21.02.2007
//***************************************************************************
// History:
//   21.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_LINEWIDTH )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
// other:
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_Line
|-----------------------------------
| draw a line with a width
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          x1 coord
|   * iY1:          y1 coord
|   * iX2:          x2 coord
|   * iY2:          y2 coord
|   * iWidth:       width of the line in pixels
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_LineWidth( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iWidth )
{
    cmg_Coord   iAbsDeltaX, iAbsDeltaY, iDeltaStart;

    // param check
    if ( iWidth < 1 )
        return;

    // calc deltas
    iAbsDeltaX = iX2 - iX1;
    if ( iAbsDeltaX < 0 )   iAbsDeltaX = -iAbsDeltaX;
    iAbsDeltaY = iY2 - iY1;
    if ( iAbsDeltaY < 0 )   iAbsDeltaY = -iAbsDeltaY;

    // calc delta
    iDeltaStart = ( iWidth - 1 ) >> 1;

    // get width direction
    if ( iAbsDeltaX >= iAbsDeltaY )
    {
        // add width in y direction

        // adjust y
        iY1 -= iDeltaStart;
        iY2 -= iDeltaStart;

        // draw lines
        while ( iWidth-- )
            CMG_Line( iX1, iY1++, iX2, iY2++ );
    }
    else
    {
        // add width in x direction

        // adjust x
        iX1 -= iDeltaStart;
        iX2 -= iDeltaStart;

        // draw lines
        while ( iWidth-- )
            CMG_Line( iX1++, iY1, iX2++, iY2 );
    }
}



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
