//***************************************************************************
// CMG_DRAW_Main.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.12.2006
//***************************************************************************
// History:
//   ??.??.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // main draw file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../2_CTRL/CMG_CTRL.h"      // only for g_CTRL_iDisplay* vars
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------

// display
cmg_Coord   g_DRAW_iScreenSizeX;            // logical screen x size
cmg_Coord   g_DRAW_iScreenSizeY;            // logical screen y size

cmg_mu8     g_DRAW_uDisplayOrientationMask; // mirrow/swap flags

// clipping
cmg_Coord   g_DRAW_iClippingWindow_LX;      // clipping rect left x
cmg_Coord   g_DRAW_iClippingWindow_RX;      // clipping rect right x
cmg_Coord   g_DRAW_iClippingWindow_TY;      // clipping rect top y
cmg_Coord   g_DRAW_iClippingWindow_BY;      // clipping rect bottom y


cmg_bool    g_DRAW_bProhibitCoordAdjustment;// internal use to prohibit coord adjm.

// byROPMode
cmg_mu8     g_DRAW_uROPMode;                // rop mode

// drawstyle
cmg_mu8     g_DRAW_uDrawStyle;              // drawstyle (border, solid, both)


// color modes
    #define     DRV_COLORMODE_SOLIDCOLOR        0x00
    #define     DRV_COLORMODE_PEN_BRUSH         0x01
cmg_mu8     g_DRAW_uColorMode;              // current color mode (solid - pen)
cmg_mu8     g_DRAW_uFillColorMode;          // current fillcolor mode (solid - brush)


// foreground color
cmg_Color   g_DRAW_Color;                   // foreground color

// background color
cmg_Color   g_DRAW_FillColor;               // background color

// pen
cmg_Pen    *g_DRAW_pPen;                    // current selected pen

// brush
cmg_Brush  *g_DRAW_pBrush;                  // current selected brush
cmg_sfp1616 g_DRAW_sfpBrushCurrentLine;     // current brush fp line

//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


// ===================
// ==== Init/Exit ====
// ===================

/****************************************************************************
| CMG_DRAW_Init
|-----------------------------------
| Init (init downwards: CMG_DRV_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_DRAW_Init( void )
{
    cmg_Result  iResult;

    // init CMG_DRV_*
    iResult = CMG_DRV_Init();
    CHECKRESULT();

    // init globals (also sets g_DRAW_iScreenSize*)
    CMG_SetDisplayOrientation( DISPLAYORIENTATION_NORMAL );
    CMG_SetClippingWindow( 0, 0, (cmg_Coord)(g_DRAW_iScreenSizeX - 1), (cmg_Coord)(g_DRAW_iScreenSizeY - 1) );
    CMG_SetROPMode( ROPMODE_COPY );
    CMG_SetDrawStyle( DRAWSTYLE_BORDER );
    CMG_SetColor( 0xff );
    CMG_SetFillColor( 0xff );
    g_DRAW_bProhibitCoordAdjustment = false;

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_DRAW_Exit();
    return iResult;
}



/****************************************************************************
| CMG_DRAW_Exit
|-----------------------------------
| Exit (exit downwards: CMG_DRV_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_DRAW_Exit( void )
{
    // exit CMG_DRV_*
    CMG_DRV_Exit();
}



/****************************************************************************
| CMG_PowerManagement
|-----------------------------------
| Power Management
+----------------------------------------------------------------------------
| @Params:         
|   * uState:       switch to powerstate
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_PowerManagement( cmg_mu8 uState )
{
    switch ( uState )
    {
        // display off
        case CMG_PWR_DISPLAY_OFF:
            // ... nothing to do ...

            // on off call parent at last
            CMG_DRV_PowerManagement( uState );
            break;


        // display on
        case CMG_PWR_DISPLAY_ON:
            // on on call parent first
            CMG_DRV_PowerManagement( uState );

            // ... nothing to do ...
            break;


        // just pass trough unknown
        default:
            // case CMG_PWR_BACKLIGHT_ON:
            // case CMG_PWR_BACKLIGHT_OFF:
            CMG_DRV_PowerManagement( uState );
            break;
    }
}



/****************************************************************************
| CMG_ControllerSpecific
|-----------------------------------
| Controller specific functions (contrast, etc.)
+----------------------------------------------------------------------------
| @Params:         
|   * uFunction:        selected feature
|   * pParam:           corresponding parameter
| @Return Value:    result, if controller supports this
+***************************************************************************/
_PUBLIC
cmg_Result CMG_ControllerSpecific( cmg_mu8 uFunction, cmg_void *pParam )
{
    // just handle through
    return CMG_DRV_ControllerSpecific( uFunction, pParam );
}






// ===========================
// ==== Internal Exports =====
// ===========================


// adjust x y accordingly to the display orientation
_PROTECTED
void _DRAW_AdjustXY( cmg_Coord *piX, cmg_Coord *piY )
{
    // if coord adjustment is prohibited return
    if ( g_DRAW_bProhibitCoordAdjustment )
        return;

    // swap xy
    if ( g_DRAW_uDisplayOrientationMask & DISPLAYORIENTATION_XYSWAP )
        SWAP_TYPE( *piX, *piY, cmg_Coord );

    // mirror x
    if ( g_DRAW_uDisplayOrientationMask & DISPLAYORIENTATION_XMIRROR )
        *piX = g_CTRL_iDisplayWidth - 1 - *piX;
    
    // mirror y
    if ( g_DRAW_uDisplayOrientationMask & DISPLAYORIENTATION_YMIRROR )
        *piY = g_CTRL_iDisplayHeight - 1 - *piY;
}






// =======================
// ==== Main Exports =====
// =======================




/****************************************************************************
| CMG_GetScreenSize
|-----------------------------------
| Gets the screen size
+----------------------------------------------------------------------------
| @Params:          
|   * piScreenSizeX:    x screen size
|   * piScreenSizeY:    y screen size
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_GetScreenSize( cmg_Coord *piScreenSizeX, cmg_Coord *piScreenSizeY )
{
    // store x
    if ( piScreenSizeX )
        *piScreenSizeX = g_DRAW_iScreenSizeX;

    // store y
    if ( piScreenSizeY )
        *piScreenSizeY = g_DRAW_iScreenSizeY;
}




/****************************************************************************
| CMG_SetDisplayOrientation
|-----------------------------------
| Set the diaply orientation
+----------------------------------------------------------------------------
| @Params:          
|   * uDisplayOrientationMask:    the diaply orientation to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetDisplayOrientation( cmg_mu8 uDisplayOrientationMask )
{
    // simply store parameter
    g_DRAW_uDisplayOrientationMask = uDisplayOrientationMask;
    
    // set screen size
    if ( uDisplayOrientationMask & DISPLAYORIENTATION_XYSWAP )
    {
        // swap xy
        g_DRAW_iScreenSizeX = g_CTRL_iDisplayHeight;
        g_DRAW_iScreenSizeY = g_CTRL_iDisplayWidth;
    }
    else
    {
        // normal xy
        g_DRAW_iScreenSizeX = g_CTRL_iDisplayWidth;
        g_DRAW_iScreenSizeY = g_CTRL_iDisplayHeight;
    }
}



/****************************************************************************
| CMG_GetDisplayOrientation
|-----------------------------------
| Get the diaply orientation
+----------------------------------------------------------------------------
| @Params:          
| @Return Value:    Display Orientation Mask
+***************************************************************************/
_PUBLIC
cmg_mu8 CMG_GetDisplayOrientation( void )
{
    return g_DRAW_uDisplayOrientationMask;
}




/****************************************************************************
| CMG_SetClippingWindow
|-----------------------------------
| Set a clipping window you can't draw outside. The both specified points
| span a rectangle. All pixels inside can be modified. 
| lower <= pixel <= upper
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:      Point 1 X (eg. Top Left)
|   * iY1:      Point 1 Y (eg. Top Left)
|   * iX2:      Point 2 X (eg. Bottom Right)
|   * iY2:      Point 2 Y (eg. Bottom Right)
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetClippingWindow( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 )
{
    // display orientation
    _DRAW_AdjustXY( &iX1, &iY1 );
    _DRAW_AdjustXY( &iX2, &iY2 );
    
    // sort values
    g_DRAW_iClippingWindow_LX = MIN( iX1, iX2 );
    g_DRAW_iClippingWindow_RX = MAX( iX1, iX2 );
    g_DRAW_iClippingWindow_TY = MIN( iY1, iY2 );
    g_DRAW_iClippingWindow_BY = MAX( iY1, iY2 );

    // clip values
    g_DRAW_iClippingWindow_LX = BOUND_MIN_MAX( g_DRAW_iClippingWindow_LX, 0, g_CTRL_iDisplayWidth - 1 );
    g_DRAW_iClippingWindow_RX = BOUND_MIN_MAX( g_DRAW_iClippingWindow_RX, 0, g_CTRL_iDisplayWidth - 1 );
    g_DRAW_iClippingWindow_TY = BOUND_MIN_MAX( g_DRAW_iClippingWindow_TY, 0, g_CTRL_iDisplayHeight - 1 );
    g_DRAW_iClippingWindow_BY = BOUND_MIN_MAX( g_DRAW_iClippingWindow_BY, 0, g_CTRL_iDisplayHeight - 1 );
}




/****************************************************************************
| CMG_GetClippingWindow
|-----------------------------------
| Return current clipping window.
| If pointer is NULL no value will be returned for this.
+----------------------------------------------------------------------------
| @Params:          
|   * piX1:     Point 1 X (eg. Top Left)
|   * piY1:     Point 1 Y (eg. Top Left)
|   * piX2:     Point 2 X (eg. Bottom Right)
|   * piY2:     Point 2 Y (eg. Bottom Right)
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_GetClippingWindow( cmg_Coord *piX1, cmg_Coord *piY1, cmg_Coord *piX2, cmg_Coord *piY2 )
{
    // store x1
    if ( piX1 )
        *piX1 = g_DRAW_iClippingWindow_LX;

    // store y1
    if ( piY1 )
        *piY1 = g_DRAW_iClippingWindow_TY;

    // store x2
    if ( piX2 )
        *piX2 = g_DRAW_iClippingWindow_RX;

    // store y2
    if ( piY2 )
        *piY2 = g_DRAW_iClippingWindow_BY;
}




/****************************************************************************
| CMG_SetROPMode
|-----------------------------------
| Sets the ROP mode
+----------------------------------------------------------------------------
| @Params:          
|   * uROPMode:     ROP mode to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetROPMode( cmg_mu8 uROPMode )
{
    g_DRAW_uROPMode = uROPMode;
    CMG_DRV_SetROPMode( uROPMode );
}




//---------------------------------------------------------------------------


#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
