//***************************************************************************
// CMG_DRAW_MainDrawHelpers.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         05.12.2006
//***************************************************************************
// History:
//   ??.??.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // main draw file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../2_CTRL/CMG_CTRL.h"      // only for g_CTRL_iDisplay* vars
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exports
//---------------------------------------------------------------------------


/*---------------------------------------------------------------------------
| _CMG_ApplyPen
|-----------------------------------
| Helper function to apply the current pen (solid color or real pen)
+----------------------------------------------------------------------------
| @Params:          
|   * iSpoolX:      initial x spool count
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED
void _DRAW_ApplyPen( cmg_Coord iSpoolX )
{
    if ( g_DRAW_uColorMode == DRV_COLORMODE_SOLIDCOLOR )
    {
        // DRV_COLORMODE_SOLIDCOLOR
        CMG_DRV_SetSolidColor( g_DRAW_Color );
    }
    else
    {
        // DRV_COLORMODE_PEN_BRUSH
        CMG_DRV_SetPen( g_DRAW_pPen->pbyPenStartAddr, g_DRAW_pPen->uPenLengthBytes, g_DRAW_pPen->sfpPenScaleFactor );
        // spool pen
        CMG_DRV_SpoolPen( (cmg_Coord)( g_DRAW_pPen->iPenSpoolPixels + iSpoolX ) );
    }
}




/*---------------------------------------------------------------------------
| _CMG_ApplyBrush
|-----------------------------------
| Helper function to apply the current brush (solid color or real brush)
+----------------------------------------------------------------------------
| @Params:          
|   * iSpoolY:      initial y spool count
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED
void _DRAW_ApplyBrush( cmg_Coord iSpoolY )
{
    if ( g_DRAW_uFillColorMode == DRV_COLORMODE_SOLIDCOLOR )
    {
        // DRV_COLORMODE_SOLIDCOLOR
        CMG_DRV_SetSolidColor( g_DRAW_FillColor );
    }
    else
    {
        // DRV_COLORMODE_PEN_BRUSH
        cmg_sfp1616     sfpBrushLines;

        g_DRAW_sfpBrushCurrentLine = ( g_DRAW_pBrush->iBrushSpoolPixelsY + iSpoolY ) * g_DRAW_pBrush->sfpBrushScaleFactorY;

        // adjust g_DRAW_sfpBrushCurrentLine
        sfpBrushLines = INT_TO_SFP( g_DRAW_pBrush->uBrushLines );
        while ( g_DRAW_sfpBrushCurrentLine < 0 )
            g_DRAW_sfpBrushCurrentLine += sfpBrushLines;
        while ( g_DRAW_sfpBrushCurrentLine >= sfpBrushLines )
            g_DRAW_sfpBrushCurrentLine -= sfpBrushLines;
    }
}




/*---------------------------------------------------------------------------
| _CMG_ApplyBrushNextLine
|-----------------------------------
| Helper function to apply the next line of the current brush (solid color or real brush)
+----------------------------------------------------------------------------
| @Params:          
|   * iSpoolX:      initial x spool count
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED
void _DRAW_ApplyBrushNextLine( cmg_Coord iLineSpoolX )
{
    // nothing to do for DRV_COLORMODE_SOLIDCOLOR anymore

    if ( g_DRAW_uFillColorMode == DRV_COLORMODE_PEN_BRUSH )
    {   
        // DRV_COLORMODE_PEN_BRUSH
        cmg_sfp1616     sfpBrushLines;

        // get current brush line and set it
        CMG_DRV_SetPen( g_DRAW_pBrush->pbyBrushStartAddr + ( ( (cmg_mu16) SFP_TO_INT( g_DRAW_sfpBrushCurrentLine ) ) * g_DRAW_pBrush->uBrushLineLengthBytes ), g_DRAW_pBrush->uBrushLineLengthBytes, g_DRAW_pBrush->sfpBrushScaleFactorX );

        // spool pen
        CMG_DRV_SpoolPen( (cmg_Coord)(g_DRAW_pBrush->iBrushSpoolPixelsX + iLineSpoolX) );

        // calc next line
        g_DRAW_sfpBrushCurrentLine += g_DRAW_pBrush->sfpBrushScaleFactorY;

        // adjust g_DRAW_sfpBrushCurrentLine
        sfpBrushLines = INT_TO_SFP( g_DRAW_pBrush->uBrushLines );
        while ( g_DRAW_sfpBrushCurrentLine < 0 )
            g_DRAW_sfpBrushCurrentLine += sfpBrushLines;
        while ( g_DRAW_sfpBrushCurrentLine >= sfpBrushLines )
            g_DRAW_sfpBrushCurrentLine -= sfpBrushLines;
    }
}





/*---------------------------------------------------------------------------
| _DRAW_ClippedPixel
|-----------------------------------
| Helper function to draw the next border pixel with clipping support
+----------------------------------------------------------------------------
| @Params:         
|   * iX:           x point
|   * iY:           y point
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _DRAW_ClippedPixel( cmg_Coord iX, cmg_Coord iY )
{
    // clipping
    if ( ( iX < g_DRAW_iClippingWindow_LX ) || ( iX > g_DRAW_iClippingWindow_RX ) ||
         ( iY < g_DRAW_iClippingWindow_TY ) || ( iY > g_DRAW_iClippingWindow_BY ) )
    {
        // if to use a pen and it is clipped, spool one forward
        if ( g_DRAW_uColorMode == DRV_COLORMODE_PEN_BRUSH )
            CMG_DRV_SpoolPenRelative( 1 );
        return;
    }

    // set pixel
    CMG_DRV_Pixel( iX, iY );
}




/*---------------------------------------------------------------------------
| _DRAW_ClippedHLine
|-----------------------------------
| Helper function to draw the next fill brush line with clipping support
+----------------------------------------------------------------------------
| @Params:         
|   * iXLeft:           left line start
|   * iXRight:          right line stop
|   * iY:               y line
|   * iSpoolLeftmostX:  leftmost point to calc spooling
| @Return Value:    none
+--------------------------------------------------------------------------*/
_PROTECTED 
void _DRAW_ClippedHLine( cmg_Coord iXLeft, cmg_Coord iXRight, cmg_Coord iY, cmg_Coord iSpoolLeftmostX )
{
    // update spool position according to x start position
    iSpoolLeftmostX = iXLeft - iSpoolLeftmostX;

    // clipping
    if ( ( iY < g_DRAW_iClippingWindow_TY ) || ( iY > g_DRAW_iClippingWindow_BY ) )
         goto _lAbort;

    // clipping
    if ( g_DRAW_iClippingWindow_LX > iXLeft )
    {
        // update spool (since we must clip)
        iSpoolLeftmostX += g_DRAW_iClippingWindow_LX - iXLeft;
        // set new x
        iXLeft = g_DRAW_iClippingWindow_LX;
    }
    iXRight = MIN( iXRight, g_DRAW_iClippingWindow_RX );

    // nothing to draw anymore?
    if ( ( iXRight - iXLeft ) < 0 )
        goto _lAbort;


    // set next brush line
    _DRAW_ApplyBrushNextLine( iSpoolLeftmostX );

    // draw the line
    CMG_DRV_HLine( iXLeft, iY, (cmg_Coord)( iXRight - iXLeft + 1 ) );
    return;


_lAbort:
    // whe we abort we do have to switch to the next line to prevent the brush alignment
    _DRAW_ApplyBrushNextLine( 0 );
}


//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
