//***************************************************************************
// CMG_DRAW_MainDrawstyleColorPenBrush.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         15.02.2007
//***************************************************************************
// History:
//   15.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // main draw file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_SetDrawStyle
|-----------------------------------
| Set the draw style
+----------------------------------------------------------------------------
| @Params:          
|   * uDrawStyle:   draw style to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetDrawStyle( cmg_mu8 uDrawStyle )
{
    g_DRAW_uDrawStyle = uDrawStyle;
}



/****************************************************************************
| CMG_SetColor
|-----------------------------------
| Set the color
+----------------------------------------------------------------------------
| @Params:          
|   * Color:        color to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetColor( cmg_Color Color )
{
    g_DRAW_Color = Color;
    g_DRAW_uColorMode = DRV_COLORMODE_SOLIDCOLOR;
}



/****************************************************************************
| CMG_SetFillColor
|-----------------------------------
| Set the fill color
+----------------------------------------------------------------------------
| @Params:          
|   * FillColor:    fill color to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetFillColor( cmg_Color FillColor )
{
    g_DRAW_FillColor = FillColor;
    g_DRAW_uFillColorMode = DRV_COLORMODE_SOLIDCOLOR;
}



/****************************************************************************
| CMG_SetPen
|-----------------------------------
| Set the pen
+----------------------------------------------------------------------------
| @Params:          
|   * pPen:         pen to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetPen( cmg_Pen *pPen )
{
    g_DRAW_pPen = pPen;
    g_DRAW_uColorMode = DRV_COLORMODE_PEN_BRUSH;

}



/****************************************************************************
| CMG_SetBrush
|-----------------------------------
| Set the brush
+----------------------------------------------------------------------------
| @Params:          
|   * pBrush:       brush to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_SetBrush( cmg_Brush *pBrush )
{
    g_DRAW_pBrush = pBrush;
    g_DRAW_uFillColorMode = DRV_COLORMODE_PEN_BRUSH;
}




/****************************************************************************
| CMG_CreatePen
|-----------------------------------
| Create a pen object
+----------------------------------------------------------------------------
| @Params:          
|   * pPen:             pointer to an allocated pen object
|   * pbyPenStartAddr:  start address of pen
|   * uPenLengthBytes:  length of pen in bytes
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CreatePen( cmg_Pen *pPen, cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes )
{
    // param check
    if ( !pPen )     return;
        
    pPen->pbyPenStartAddr   = pbyPenStartAddr;
    pPen->uPenLengthBytes   = uPenLengthBytes;
    pPen->iPenSpoolPixels   = 0;
    pPen->sfpPenScaleFactor = DRV_SCALE_NONE;
}



/****************************************************************************
| CMG_CreatePenEx
|-----------------------------------
| Create an extended pen object
+----------------------------------------------------------------------------
| @Params:          
|   * pPen:             pointer to an allocated pen object
|   * pbyPenStartAddr:  start address of pen
|   * uPenLengthBytes:  length of pen in bytes
|   * iPenSpoolPixels:  initial pixel spool
|   * fPenScaleFactor:  initial scale factor
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CreatePenEx( cmg_Pen *pPen, cmg_u8 *pbyPenStartAddr, cmg_mu16 uPenLengthBytes, cmg_Coord iPenSpoolPixels, float fPenScaleFactor )
{
    // param check
    if ( !pPen )     return;
        
    pPen->pbyPenStartAddr   = pbyPenStartAddr;
    pPen->uPenLengthBytes   = uPenLengthBytes;
    pPen->iPenSpoolPixels   = iPenSpoolPixels;
    pPen->sfpPenScaleFactor = (cmg_sfp1616) ( fPenScaleFactor * INT_TO_SFP( 1 ) );
}




/****************************************************************************
| CMG_CreateBrush
|-----------------------------------
| Create a brush object
+----------------------------------------------------------------------------
| @Params:          
|   * pBrush:                   pointer to an allocated brush object
|   * pbyBrushStartAddr:        start address of brush
|   * uBrushLineLengthBytes:    length of a line in bytes (x size / 8)
|   * uBrushLines:              line count (y size)
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CreateBrush( cmg_Brush *pBrush, cmg_u8 *pbyBrushStartAddr, cmg_mu16 uBrushLineLengthBytes, cmg_mu16 uBrushLines )
{
    // param check
    if ( !pBrush )     return;

    pBrush->pbyBrushStartAddr       = pbyBrushStartAddr;
    pBrush->uBrushLineLengthBytes   = uBrushLineLengthBytes;
    pBrush->uBrushLines             = uBrushLines;
    pBrush->iBrushSpoolPixelsX      = 0;
    pBrush->iBrushSpoolPixelsY      = 0;
    pBrush->sfpBrushScaleFactorX    = DRV_SCALE_NONE;
    pBrush->sfpBrushScaleFactorY    = DRV_SCALE_NONE;
}




/****************************************************************************
| CMG_CreateBrushEx
|-----------------------------------
| Create an extended brush object
+----------------------------------------------------------------------------
| @Params:          
|   * pBrush:                   pointer to an allocated brush object
|   * pbyBrushStartAddr:        start address of brush
|   * uBrushLineLengthBytes:    length of a line in bytes (x size / 8)
|   * uBrushLines:              line count (y size)
|   * iBrushSpoolPixelsX:       initial x pixel spool
|   * iBrushSpoolPixelsY:       initial y pixel spool
|   * fBrushScaleFactorX:       initial x scale factor
|   * fBrushScaleFactorY:       initial y scale factor
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_CreateBrushEx( cmg_Brush *pBrush, cmg_u8 *pbyBrushStartAddr, cmg_mu16 uBrushLineLengthBytes, cmg_mu16 uBrushLines, cmg_Coord iBrushSpoolPixelsX, cmg_Coord iBrushSpoolPixelsY, float fBrushScaleFactorX, float fBrushScaleFactorY )
{
    // param check
    if ( !pBrush )     return;

    pBrush->pbyBrushStartAddr       = pbyBrushStartAddr;
    pBrush->uBrushLineLengthBytes   = uBrushLineLengthBytes;
    pBrush->uBrushLines             = uBrushLines;
    pBrush->iBrushSpoolPixelsX      = iBrushSpoolPixelsX;
    pBrush->iBrushSpoolPixelsY      = iBrushSpoolPixelsY;
    pBrush->sfpBrushScaleFactorX    = (cmg_sfp1616) ( fBrushScaleFactorX * INT_TO_SFP( 1 ) );
    pBrush->sfpBrushScaleFactorY    = (cmg_sfp1616) ( fBrushScaleFactorY * INT_TO_SFP( 1 ) );
}




//---------------------------------------------------------------------------


#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
