//***************************************************************************
// CMG_DRAW_Rectangle.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         16.02.2007
//***************************************************************************
// History:
//   19.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#else
    // this draw file will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------




/****************************************************************************
| CMG_Rectangle
|-----------------------------------
| A rectangle
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          top left x
|   * iY1:          top left y
|   * iX2:          bottom right x
|   * iY2:          bottom right y
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_Rectangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2 )
{
    // display orientation
    _DRAW_AdjustXY( &iX1, &iY1 );
    _DRAW_AdjustXY( &iX2, &iY2 );


    // adjust coords
    {
        cmg_Coord   iTemp;
        if ( iX1 > iX2 )
            SWAP( iX1, iX2, iTemp );
        if ( iY1 > iY2 )
            SWAP( iY1, iY2, iTemp );
    }


    // draw border?
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
    {
        // turn off adjustment because we have done this already
        g_DRAW_bProhibitCoordAdjustment = true;
        CMG_LineEx( iX1, iY1, iX2, iY1, LINESUPPRESS_LAST );
        CMG_LineEx( iX2, iY1, iX2, iY2, LINESUPPRESS_LAST );
        CMG_LineEx( iX2, iY2, iX1, iY2, LINESUPPRESS_LAST );
        CMG_LineEx( iX1, iY2, iX1, iY1, LINESUPPRESS_LAST );
        g_DRAW_bProhibitCoordAdjustment = false;
    }


    // fill rect?
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_FILL )
    {
        cmg_Coord   iSizeX, iSizeY, iSpoolX, iSpoolY;
        

        // init spool
        iSpoolX = iSpoolY = 0;


        // border was drawn?
        if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
        {
            // since we drawed a border we must adjust the inner fill area
            iX1++;
            iX2--;
            iY1++;
            iY2--;
        }


        // clipping
        if ( g_DRAW_iClippingWindow_LX > iX1 )
        {
            // update spool (since we must clip)
            iSpoolX = g_DRAW_iClippingWindow_LX - iX1;
            // set new x
            iX1 = g_DRAW_iClippingWindow_LX;
        }
        if ( g_DRAW_iClippingWindow_TY > iY1 )
        {
            // update spool (since we must clip)
            iSpoolY = g_DRAW_iClippingWindow_TY - iY1;
            // set new x
            iY1 = g_DRAW_iClippingWindow_TY;
        }
        // clip bottom right - no spool update here
        iX2 = MIN( iX2, g_DRAW_iClippingWindow_RX );
        iY2 = MIN( iY2, g_DRAW_iClippingWindow_BY );

    
        // nothing to draw anymore?
        if ( ( ( iX2 - iX1 ) < 0 ) || ( ( iY2 - iY1 ) < 0 ) )
            return;

        // calc size
        iSizeX = iX2 - iX1 + 1;
        iSizeY = iY2 - iY1 + 1;

        // init brush
        _DRAW_ApplyBrush( iSpoolY );

        // for each line
        while ( iSizeY-- )
        {
            // set brush line
            _DRAW_ApplyBrushNextLine( iSpoolX );
            // draw line
            CMG_DRV_HLine( iX1, iY1, iSizeX );
            // next line
            iY1++;
        }
    }
}



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
