//***************************************************************************
// CMG_DRAW_RoundedRectangle.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         09.03.2007
//***************************************************************************
// History:
//   10.03.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_ROUNDEDRECTANGLE )
    #if !defined( CMG_DRAW_ELLIPSE )
        #error CMG_DRAW_ELLIPSE must be defined to use CMG_DRAW_ROUNDEDRECTANGLE. Please customize the config file correctly using the *.config template.
    #endif 
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "CMG_DRAW_Ellipse.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// exports
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_RoundedRectangle
|-----------------------------------
| A rectangle with rounded corners
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          top left x
|   * iY1:          top left y
|   * iX2:          bottom right x
|   * iY2:          bottom right y
|   * iRadius:      corner radius
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_RoundedRectangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iRadius )
{
    cmg_DRAW_Ellipse_Data       EllData;


    // display orientation
    _DRAW_AdjustXY( &iX1, &iY1 );
    _DRAW_AdjustXY( &iX2, &iY2 );


    // adjust coords
    {
        cmg_Coord   iTemp;
        if ( iX1 > iX2 )
            SWAP( iX1, iX2, iTemp );
        if ( iY1 > iY2 )
            SWAP( iY1, iY2, iTemp );
    }



    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // param check
    // invalid radii?
    if ( iRadius < 0 )
        return;
    
    // simple pixel to draw
    if ( iRadius == 0 ) 
    {
        CMG_Rectangle( iX1, iY1, iX2, iY2 );
        return;
    }

    iRadius = MIN( iRadius, ( iX2 - iX1 ) >> 1 );
    iRadius = MIN( iRadius, ( iY2 - iY1 ) >> 1 );




    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // one time data structure init
    _Ellipse_OneTimeInit( &EllData, iRadius, iRadius );



    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // fill inside
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_FILL )
    {
        cmg_Coord       iSpoolLeftmostX;
        cmg_Coord       iXInc;
        // last loop values
        cmg_Coord       iXLastLeft, iXLastRight, iYLast;

        // set brush/solid
        _DRAW_ApplyBrush( 0 );


        // init spool start value
        iSpoolLeftmostX = iX1;
        // when to draw a border we have to shift the brush one inside
        if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
            iSpoolLeftmostX++;


        // calc vars
        iXInc = iX2 - iX1 - ( iRadius << 1 );



        // . . . . . . . . . . . . . . . . . . 
        // fill upper rounded part
        // init ellipse loop
        _Ellipse_LoopInside_Init( &EllData, (cmg_Coord)( iX1 + iRadius ), (cmg_Coord)( iY1 + iRadius ), iRadius, iRadius );
        EllData.iX_Right += iXInc;
        // init last loop values
        iXLastLeft  = EllData.iX_Left;
        iXLastRight = EllData.iX_Right;
        iYLast      = EllData.iY_Top;

        // if we have to draw a borter too we only fill the inside without border but use a different technique
        // requiring one line less
        if ( g_DRAW_uDrawStyle == DRAWSTYLE_BOTH )
            EllData.iCount--;

        // for all pixels
        while ( EllData.iCount > 0 )
        {
            // update positions
            _Ellipse_LoopInside( &EllData );

            // if line has changed
            if ( iYLast != EllData.iY_Top ) 
            {
                // if we also draw the border only fill the inside, else the complete ellipse
                if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
                    _DRAW_ClippedHLine( (cmg_Coord)( EllData.iX_Left + 1 ), (cmg_Coord)( EllData.iX_Right - 1 ), EllData.iY_Top, iSpoolLeftmostX );
                else
                    _DRAW_ClippedHLine( iXLastLeft, iXLastRight, iYLast, iSpoolLeftmostX );

                // store old line
                iYLast      = EllData.iY_Top;
            }

            // store old x coords
            iXLastLeft  = EllData.iX_Left;
            iXLastRight = EllData.iX_Right;
        }


        
        // . . . . . . . . . . . . . . . . . . 
        // fill straight part
        if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
            iYLast++;

        // use iYLast for temp y counter
        iXLastLeft = iY2 - iY1 - ( iRadius << 1 );
        while ( iXLastLeft-- )
        {
            if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
                _DRAW_ClippedHLine( (cmg_Coord)( iX1 + 1 ), (cmg_Coord)( iX2 - 1 ), iYLast++, iSpoolLeftmostX );
            else
                _DRAW_ClippedHLine( iX1, iX2, iYLast++, iSpoolLeftmostX );
        }



        // . . . . . . . . . . . . . . . . . . 
        // fill lower rounded part
        // init ellipse loop
        _Ellipse_LoopOutside_Init( &EllData, (cmg_Coord)( iX1 + iRadius ), (cmg_Coord)( iY2 - iRadius ), iRadius, iRadius );
        EllData.iX_Right += iXInc;
        // init last loop values
        iXLastLeft  = EllData.iX_Left;
        iXLastRight = EllData.iX_Right;
        iYLast      = EllData.iY_Top;
        
        // if we _DON'T_ have to draw a borter too we do have to draw an extra line due the useage of a different technique
        if ( g_DRAW_uDrawStyle != DRAWSTYLE_BOTH )
            _DRAW_ClippedHLine( EllData.iX_Left, iXLastRight, EllData.iY_Bottom, iSpoolLeftmostX );


        // for all pixels
        while ( EllData.iCount > 0 )
        {
            // update positions
            _Ellipse_LoopOutside( &EllData );

            // if line has changed
            if ( iYLast != EllData.iY_Bottom )
            {
                // if we also draw the border only fill the inside, else the complete ellipse
                if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
                    _DRAW_ClippedHLine( (cmg_Coord)( iXLastLeft + 1 ), (cmg_Coord)( iXLastRight - 1 ), iYLast, iSpoolLeftmostX );
                else
                    _DRAW_ClippedHLine( EllData.iX_Left, EllData.iX_Right, EllData.iY_Bottom, iSpoolLeftmostX );

                // store old line
                iYLast      = EllData.iY_Bottom;
            }

            // store old x coords
            iXLastLeft  = EllData.iX_Left;
            iXLastRight = EllData.iX_Right;
        }
    }



    
    // . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    // draw border
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
    {
        // set pen/solid
        _DRAW_ApplyPen( 0 );

        // top tile
        CMG_DRV_HLine( (cmg_Coord)( iX1 + iRadius ), iY1, (cmg_Coord)( iX2 - iX1 - ( iRadius << 1 ) ) );

        // top right rounded edge
        _Ellipse_LoopInside_Init( &EllData, (cmg_Coord)( iX2 - iRadius ), (cmg_Coord)( iY1 + iRadius ), iRadius, iRadius ); 
        while ( EllData.iCount > 0 )
        {
            _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Top );
            _Ellipse_LoopInside( &EllData );
        }
        
        // right tile
        CMG_DRV_VLine( iX2, (cmg_Coord)( iY1 + iRadius ), (cmg_Coord)( iY2 - iY1 - ( iRadius << 1 ) ) );

        // bottom right rounded edge
        _Ellipse_LoopOutside_Init( &EllData, (cmg_Coord)( iX2 - iRadius ), (cmg_Coord)( iY2 - iRadius ), iRadius, iRadius ); 
        while ( EllData.iCount > 0 )
        {
            _DRAW_ClippedPixel( EllData.iX_Right, EllData.iY_Bottom );
            _Ellipse_LoopOutside( &EllData );
        }

        // bottom tile
        CMG_DRV_HLine( (cmg_Coord)( iX1 + iRadius + 1 ), iY2, (cmg_Coord)( iX2 - iX1 - ( iRadius << 1 ) ) );

        // bottom left rounded edge
        _Ellipse_LoopInside_Init( &EllData, (cmg_Coord)( iX1 + iRadius ), (cmg_Coord)( iY2 - iRadius ), iRadius, iRadius ); 
        while ( EllData.iCount > 0 )
        {
            _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Bottom );
            _Ellipse_LoopInside( &EllData );
        }

        // left tile
        CMG_DRV_VLine( iX1, (cmg_Coord)( iY1 + iRadius + 1 ), (cmg_Coord)( iY2 - iY1 - ( iRadius << 1 ) ) );

        // top left rounded edge
        _Ellipse_LoopOutside_Init( &EllData, (cmg_Coord)( iX1 + iRadius ), (cmg_Coord)( iY1 + iRadius ), iRadius, iRadius ); 
        while ( EllData.iCount > 0 )
        {
            _DRAW_ClippedPixel( EllData.iX_Left, EllData.iY_Top );
            _Ellipse_LoopOutside( &EllData );
        }
    }
}

                    






#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
