//***************************************************************************
// CMG_DRAW_Scroll.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         12.02.2007
//***************************************************************************
// History:
//   13.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_SCROLL )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------





/****************************************************************************
| CMG_Scroll
|-----------------------------------
| moves a display range to another (used for scrolling windows)
+----------------------------------------------------------------------------
| @Params:         
|   * iDestX:               destination x point
|   * iDestY:               destination y point
|   * iSrcX:                source x point
|   * iSrcY:                source y point
|   * iSizeX:               x size
|   * iSizeY:               y size
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_Scroll( cmg_Coord iDestX, cmg_Coord iDestY, cmg_Coord iSrcX, cmg_Coord iSrcY, cmg_Coord iSizeX, cmg_Coord iSizeY )
{
    // buffer to hold one line maximum
    cmg_u8      abyLineBuffer[ CMG_DRV_FULL_BYTES_FROM_X( CMG_CTRL_WIDTH ) ];
    cmg_Coord   iStepY;

    // display orientation
    _DRAW_AdjustXY( &iDestX, &iDestY );
    
    // param check
    if ( ( iSizeX <= 0 ) || ( iSizeY <= 0 ) )
        return;

    // set rop copy
    CMG_DRV_SetROPMode( ROPMODE_COPY );

    // get moving y direction
    if ( iDestY > iSrcY )
    {
        // move src downwards: start downmost and go upwards
        iSrcY  += iSizeY - 1;
        iDestY += iSizeY - 1;
        iStepY = -1;
    }
    else 
    {
        // move src upwards: start uppermost (set already) and go downwards
        iStepY = 1;
    }

    // move each line
    while ( iSizeY-- )
    {
        // get src line
        CMG_DRV_GetHLine( abyLineBuffer, iSrcX, iSrcY, iSizeX );

        // draw dest line
        CMG_DRV_SetPen( abyLineBuffer, CMG_DRV_FULL_BYTES_FROM_X( CMG_CTRL_WIDTH ), DRV_SCALE_NONE );
        CMG_DRV_HLine( iDestX, iDestY, iSizeX );

        // adjust current line
        iSrcY  += iStepY;
        iDestY += iStepY;
    }

    // restore rop mode
    CMG_DRV_SetROPMode( g_DRAW_uROPMode  );
}




#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
