//***************************************************************************
// CMG_DRAW_Triangle.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         21.02.2007
//***************************************************************************
// History:
//   21.02.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_DRAW_CONFIGURED
    #error CMG_DRAW not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_DRAW_TRIANGLE )
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_DRAW.h"
#include    "CMG_DRAW_Main.h"
#include    "CMG_DRAW_Line.h"
#include    "../3_DRV/CMG_DRV.h"
// other:
//---------------------------------------------------------------------------







// sort p1 and p2 according their y value and if thei are equal according their x value
#define     POINT_YX_SORT(iX1,iY1,iX2,iY2,iTemp)                                 \
                if ( ( iY1 > iY2 ) || ( ( iY1 == iY2 ) && ( iX1 > iX2 ) ) )     \
                {                                                               \
                    SWAP( iX1, iX2, iTemp );                                    \
                    SWAP( iY1, iY2, iTemp );                                    \
                }





/****************************************************************************
| CMG_Triangle
|-----------------------------------
| draw a triangle
+----------------------------------------------------------------------------
| @Params:          
|   * iX1:          x1 coord
|   * iY1:          y1 coord
|   * iX2:          x2 coord
|   * iY2:          y2 coord
|   * iX3:          x3 coord
|   * iY3:          y3 coord
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_Triangle( cmg_Coord iX1, cmg_Coord iY1, cmg_Coord iX2, cmg_Coord iY2, cmg_Coord iX3, cmg_Coord iY3 )
{
    // display orientation
    _DRAW_AdjustXY( &iX1, &iY1 );
    _DRAW_AdjustXY( &iX2, &iY2 );
    _DRAW_AdjustXY( &iX3, &iY3 );

    // . . . . . . . . . . . . . . . . . . . . . . 
    // border

    // if to draw the border its very easy
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
    {
        // switch off coord adjustments
        g_DRAW_bProhibitCoordAdjustment = true;
        
        // just connect the three points
        CMG_LineEx( iX1, iY1, iX2, iY2, LINESUPPRESS_LAST );
        CMG_LineEx( iX2, iY2, iX3, iY3, LINESUPPRESS_LAST );
        CMG_LineEx( iX3, iY3, iX1, iY1, LINESUPPRESS_LAST );

        // switch on coord adjustments
        g_DRAW_bProhibitCoordAdjustment = false;
    }


    // . . . . . . . . . . . . . . . . . . . . . . 
    // fill

    // if to draw the filled polygon
    if ( g_DRAW_uDrawStyle & DRAWSTYLE_FILL )
    {
        cmg_DRAW_Line_Data      LineData1, LineData2;
        cmg_Coord               iSpoolStartX;
        cmg_Coord               iLineY;
        cmg_Coord               iXLast1 = 0, iXLast2 = 0;


        // set brush/solid
        _DRAW_ApplyBrush( 0 );


        // . . . . . . . . . . . . . . . . . . . . . . 
        // sort points
        // sort points according their y value (and if equal their x value) (iLineY as temp)
        POINT_YX_SORT( iX1, iY1, iX2, iY2, iLineY );
        POINT_YX_SORT( iX2, iY2, iX3, iY3, iLineY );
        POINT_YX_SORT( iX1, iY1, iX2, iY2, iLineY );



        // . . . . . . . . . . . . . . . . . . . . . . 
        // setup start values
        // calc spool
        iSpoolStartX = MIN( iX1, MIN( iX2, iX3 ) );
        if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
            iSpoolStartX++;

        // init lines: 1-2 and 1-3
        _Line_Loop_Init( &LineData1, iX1, iY1, iX2, iY2 ); 
        _Line_Loop_Init( &LineData2, iX1, iY1, iX3, iY3 ); 


        // start line
        iLineY = iY1;



        // . . . . . . . . . . . . . . . . . . . . . . 
        // draw
        // for each line
        while ( 1 )
        {
            // step line1 to next y line point
            while ( LineData1.iY <= iLineY )
            {
                // save last x point
                iXLast1 = LineData1.iX;

                // if still points are on the line, get next point, else init line 2-3
                if ( LineData1.iCount > 0 )
                    _Line_Loop( &LineData1 );
                else
                    _Line_Loop_Init( &LineData1, iX2, iY2, iX3, iY3 ); 
            }

            // step line2 to next y line point
            while ( LineData2.iY <= iLineY )
            {
                // save last x point
                iXLast2 = LineData2.iX;
                _Line_Loop( &LineData2 );
            }

            // next line
            iLineY++;

            // did we draw the border?
            if ( g_DRAW_uDrawStyle & DRAWSTYLE_BORDER )
            {
                // only fill the inside of the triangle (prevent border)
                if ( LineData1.iX <= LineData2.iX )
                    _DRAW_ClippedHLine( (cmg_Coord)( LineData1.iX + 1 ), (cmg_Coord)( LineData2.iX - 1 ), iLineY, iSpoolStartX );
                else
                    _DRAW_ClippedHLine( (cmg_Coord)( LineData2.iX + 1 ), (cmg_Coord)( LineData1.iX - 1 ), iLineY, iSpoolStartX );

                // triangle end check
                if ( iLineY >= iY3 )
                    break;
            }
            else
            {
                // completely fill the triangle
                
                // sort x values and draw complete line
                if ( iXLast1 <= iXLast2 )
                    _DRAW_ClippedHLine( iXLast1, iXLast2, (cmg_Coord)( iLineY - 1 ), iSpoolStartX );
                else
                    _DRAW_ClippedHLine( iXLast2, iXLast1, (cmg_Coord)( iLineY - 1 ), iSpoolStartX );

                // triangle end check
                // we need an extra step since we are drawing (iLineY-1)
                if ( iLineY > iY3 )
                    break;
            }
        }
    }
}



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
