//***************************************************************************
// CMG_TEXT_Font.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         27.08.2007
//***************************************************************************
// History:
//   02.09.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_TEXT_CONFIGURED
    #error CMG_TEXT not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_TEXT_ENABLED )
    // main text file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_TEXT.h"
#include    "CMG_TEXT_Internal.h"
#include    "../4_DRAW/CMG_DRAW.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// RawFont schema
//---------------------------------------------------------------------------
/*
typedef struct _cmg_RawFont {
    cmg_RawFont_Header  FontHeader;                 // font header
    cmg_RawFont_Char    Char[];                     // chars
} cmg_RawFont;

typedef struct _cmg_RawFont_Header {
    cmg_u8              byFontType;                 // font compression type
    cmg_u8              byCharStart;                // first character index
    cmg_u8              byCharStop;                 // last character index
    cmg_u8              byLineSizeBytes;            // nr of bytes per char line
    cmg_u8              byHeight;                   // nr of lines per char
} cmg_RawFont_Header;

typedef struct _cmg_RawFont_Char {
    cmg_u8              byCharWidth;                // char size in bytes
    cmg_u8              abyRawCharData[];           // char data
} cmg_RawFont_Char;
*/
//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


/****************************************************************************
| CMG_TEXT_Font_Create
|-----------------------------------
| Create the specified font object
+----------------------------------------------------------------------------
| @Params:          
|   * pFont:            pointer to an allocated font object
|   * pFontRawData:     pointer to raw stored font data
|   * iCharSpacing:     additional char spacing
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_TEXT_Font_Create( cmg_Font *pFont, cmg_FontRawData *pFontRawData, cmg_ms8 iCharSpacing )
{
    // save state
    _TEXT_FontRawAccess_SaveState();
    
    // set new font ptr
    _TEXT_FontRawAccess_SetBaseAddress( pFontRawData );

    // read font type
    pFont->byFontType = _TEXT_FontRawAccess_GetByte();
    if ( pFont->byFontType != CMG_FONTTYPE_UNCOMPRESSED )
        return;

    // read font header
    pFont->byCharStart = _TEXT_FontRawAccess_GetByte();
    pFont->byCharStop = _TEXT_FontRawAccess_GetByte();
    pFont->byLineSizeBytes = _TEXT_FontRawAccess_GetByte();
    pFont->byHeight = _TEXT_FontRawAccess_GetByte();
    
    // calculate other needed values
    pFont->uCharDataBytes = pFont->byLineSizeBytes * pFont->byHeight;
    pFont->uCharOffsetBytes = 1 + pFont->uCharDataBytes;
    pFont->pFontRawCharDataStart = pFontRawData + 5; 

    // set spacing
    pFont->iCharSpacing = iCharSpacing;

    // restore state
    _TEXT_FontRawAccess_RestoreState();
}




/****************************************************************************
| CMG_TEXT_Font_Set
|-----------------------------------
| Set the specified font
+----------------------------------------------------------------------------
| @Params:          
|   * pFont:        font to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
cmg_Result CMG_TEXT_Font_Set( cmg_Font *pFont )
{
    cmg_Result  iResult;

    // first free any previously set font
    _TEXT_Font_Free();

    // set global var and char data start base ptr
    g_TEXT_pFont = pFont;
    _TEXT_FontRawAccess_SetBaseAddress( g_TEXT_pFont->pFontRawCharDataStart );

    // alloc image space for one char
    iResult = _TEXT_Mem_Alloc( &g_TEXT_pbyImageCharData, (cmg_mu16) ( g_TEXT_pFont->byLineSizeBytes * g_TEXT_pFont->byHeight ) );
    CHECKRESULT();

    // create image for char drawing
    CMG_CreateImage( &g_TEXT_ImageChar, g_TEXT_pbyImageCharData, 1, g_TEXT_pFont->byHeight, g_TEXT_pFont->byLineSizeBytes );

    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    _TEXT_Font_Free();
    return iResult;
}





/****************************************************************************
| CMG_TEXT_Font_Set
|-----------------------------------
| Frees the memory allocated with the current font
+----------------------------------------------------------------------------
| @Params:          
| @Return Value:    none
+***************************************************************************/
_PROTECTED
void _TEXT_Font_Free( void )
{
    g_TEXT_pFont = NULL;
    _TEXT_Mem_Free( &g_TEXT_pbyImageCharData );
}


//---------------------------------------------------------------------------

#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
