//***************************************************************************
// CMG_TEXT_Main.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         27.08.2007
//***************************************************************************
// History:
//   02.09.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_TEXT_CONFIGURED
    #error CMG_TEXT not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_TEXT_ENABLED )
    // main text file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_TEXT.h"
#include    "CMG_TEXT_Internal.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------
cmg_Font    *g_TEXT_pFont;

cmg_Color   g_TEXT_TextColor;
cmg_Color   g_TEXT_BackColor;

cmg_Image   g_TEXT_ImageChar;
cmg_u8      *g_TEXT_pbyImageCharData;
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------


// ===================
// ==== Init/Exit ====
// ===================

/****************************************************************************
| CMG_TEXT_Init
|-----------------------------------
| Init (init downwards: CMG_DRAW_Init)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    result
+***************************************************************************/
_PUBLIC
cmg_Result CMG_TEXT_DrawAndTextInit( void )
{
    cmg_Result  iResult;

    // init CMG_DRV_*
    iResult = CMG_DRAW_Init();
    CHECKRESULT();

    // init 
    g_TEXT_pFont = NULL;
    g_TEXT_pbyImageCharData = NULL;
    g_TEXT_TextColor = 1;
    g_TEXT_BackColor = 0;


    // return success
    return CMG_OK;

    //.............................
    // on error free everything
_Error:
    CMG_TEXT_Exit();
    return iResult;
}



/****************************************************************************
| CMG_TEXT_Exit
|-----------------------------------
| Exit (exit downwards: CMG_DRAW_Exit)
+----------------------------------------------------------------------------
| @Params:          none
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_TEXT_Exit( void )
{
    // free font, if any
    _TEXT_Font_Free();

    // exit CMG_DRAW_*
    CMG_DRAW_Exit();
}




/****************************************************************************
| CMG_TEXT_SetTextColor
|-----------------------------------
| Set the text color
+----------------------------------------------------------------------------
| @Params:          
|   * TextColor:    text color to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_TEXT_SetTextColor( cmg_Color TextColor )
{
    g_TEXT_TextColor = TextColor;
}




/****************************************************************************
| CMG_TEXT_SetBackColor
|-----------------------------------
| Set the text back color
+----------------------------------------------------------------------------
| @Params:          
|   * TextColor:    text back color to set
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_TEXT_SetBackColor( cmg_Color BackColor )
{
    g_TEXT_BackColor = BackColor;
}




/****************************************************************************
| CMG_TEXT_IsValidChar
|-----------------------------------
| Checks if the char is a valid renderable char in the current font
+----------------------------------------------------------------------------
| @Params:          
|   * TextColor:    cChar to check
| @Return Value:    if the char is a valid renderable char in the current font
+***************************************************************************/
_PUBLIC
cmg_bool CMG_TEXT_IsValidChar( cmg_char cChar )
{
	// check if char is inside the range
    if ( ( cChar >= g_TEXT_pFont->byCharStart ) && ( cChar <= g_TEXT_pFont->byCharStop ) )
        return true;
    
    return false;
}


//---------------------------------------------------------------------------

#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
