//***************************************************************************
// CMG_TEXT_Measure.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         27.08.2007
//***************************************************************************
// History:
//   02.09.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_TEXT_CONFIGURED
    #error CMG_TEXT not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_TEXT_ENABLED )
    // main text file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_TEXT.h"
#include    "CMG_TEXT_Internal.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------



/****************************************************************************
| CMG_TEXT_Measure_GetTextHeight
|-----------------------------------
| Get the height of the current font
+----------------------------------------------------------------------------
| @Params:          
| @Return Value:    height of the current font
+***************************************************************************/
_PUBLIC
cmg_Coord CMG_TEXT_Measure_GetTextHeight( void )
{
    return (cmg_Coord) g_TEXT_pFont->byHeight;
}





/****************************************************************************
| _TEXT_Measure_GetCharWidth
|-----------------------------------
| Get the width of a char with the current font
+----------------------------------------------------------------------------
| @Params:
|   * cChar:        char to render
| @Return Value:    width of the char with the current font
+***************************************************************************/
_PRIVATE
cmg_Coord _TEXT_Measure_GetCharWidth( cmg_char cChar )
{
    cmg_Coord   iWidth = 0;

    // if char is valid
    if ( CMG_TEXT_IsValidChar( cChar ) )
    {
        // set offset to char
        _TEXT_FontRawAccess_SetOffset( (cmg_mu16) ( ( cChar - g_TEXT_pFont->byCharStart ) * g_TEXT_pFont->uCharOffsetBytes ) );
        // read char width in first byte
        iWidth = (cmg_Coord) _TEXT_FontRawAccess_GetByte();
    }

    return iWidth;
}




/****************************************************************************
| CMG_TEXT_Measure_GetTextWidth
|-----------------------------------
| Get the width of the text with the current font
+----------------------------------------------------------------------------
| @Params:
|   * strText:      text to render
| @Return Value:    width of the text with the current font
+***************************************************************************/
_PUBLIC
cmg_Coord CMG_TEXT_Measure_GetTextWidth( cmg_char *strText )
{
    cmg_Coord   iWidth = 0;
    cmg_char    c;

    // add each char width
    while ( ( c = *strText++ ) != NULL )
        iWidth += _TEXT_Measure_GetCharWidth( c ) + g_TEXT_pFont->iCharSpacing;

    return iWidth;
}

//---------------------------------------------------------------------------

#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
