//***************************************************************************
// CMG_TEXT_Malloc.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         02.09.2007
//***************************************************************************
// History:
//   02.09.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_TEXT_CONFIGURED
    #error CMG_TEXT not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_TEXT_ENABLED )
    // main text file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_TEXT.h"
#include    "CMG_TEXT_Internal.h"
// other:
//#include    "malloc.h"
#include    "stdlib.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------

_PROTECTED
cmg_Result _TEXT_Mem_Alloc( cmg_u8 **ppbyData, cmg_mu16 uSize )
{
    cmg_Result  iResult;
    cmg_u8      *pbyData;

    // param check
    CHECK( !ppbyData, CMG_ERROR_PARAM );

    // alloc data
    pbyData = (cmg_u8*) malloc( uSize );
    CHECK( !pbyData, CMG_ERROR_NOMEM );

    // set data ptr
    *ppbyData = pbyData;

    // return success
    return CMG_OK;

    //.............................
    // on error 
_Error:
    return iResult;
}



_PROTECTED
void _TEXT_Mem_Free( cmg_u8 **ppbyData )
{
    // just free data
    if ( ppbyData )
    {
        if ( *ppbyData )
            free( *ppbyData );
        
        *ppbyData = NULL;
    }
}



//---------------------------------------------------------------------------

#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
