//***************************************************************************
// CMG_TEXT_Render.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         27.08.2007
//***************************************************************************
// History:
//   02.09.2007     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************



// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// conditional compilation and configuration checks
//---------------------------------------------------------------------------
#ifndef CMG_TEXT_CONFIGURED
    #error CMG_TEXT not configured. Please customize the config file correctly (use the *.config template).
#elif defined( CMG_TEXT_ENABLED )
    // main text file - will always compile
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include    "CMG_TEXT.h"
#include    "CMG_TEXT_Internal.h"
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// globals
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------




//---------------------------------------------------------------------------
// exportet functions
//---------------------------------------------------------------------------



/****************************************************************************
| _TEXT_RenderChar
|-----------------------------------
| Internal function to Renders a single char
+----------------------------------------------------------------------------
| @Params:       
|   * iX:           x coord for char start
|   * iY:           y coord for char start
|   * cChar:        char to render
| @Return Value:    none
+***************************************************************************/
_PRIVATE
cmg_Coord _TEXT_RenderChar( cmg_Coord iX, cmg_Coord iY, cmg_char cChar )
{
    cmg_Coord   iWidth = 0;
    cmg_u8      *pbyData;
    cmg_mu16    uLength;

    // if char is valid
    if ( CMG_TEXT_IsValidChar( cChar ) )
    {
        // set offset to char data start
        _TEXT_FontRawAccess_SetOffset( (cmg_mu16) ( ( cChar - g_TEXT_pFont->byCharStart ) * g_TEXT_pFont->uCharOffsetBytes ) );
        // and read char width
        iWidth = (cmg_Coord) _TEXT_FontRawAccess_GetByte();

        // prepare char image
        g_TEXT_ImageChar.iImageWidth = iWidth;
        pbyData = g_TEXT_ImageChar.pbyImageStartAddr;
        uLength = g_TEXT_pFont->uCharDataBytes;

        // read raw char data into image
        while ( uLength-- )
            *pbyData++ = _TEXT_FontRawAccess_GetByte();

        // draw image with char
        CMG_Image( iX, iY, &g_TEXT_ImageChar );
    }
    // return the char width
    return iWidth;
}




/****************************************************************************
| CMG_TEXT_Render
|-----------------------------------
| Renders some text
+----------------------------------------------------------------------------
| @Params:       
|   * iX:           x coord for text start
|   * iY:           y coord for text start
|   * strText:      text to render
| @Return Value:    none
+***************************************************************************/
_PUBLIC
void CMG_TEXT_Render( cmg_Coord iX, cmg_Coord iY, cmg_char *strText )
{
    cmg_char    c;
    // render each char
    while ( ( c = *strText++ ) != NULL )
        iX += _TEXT_RenderChar( iX, iY, c ) + g_TEXT_pFont->iCharSpacing;
}


//---------------------------------------------------------------------------

#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
