//***************************************************************************
// CMG_Commons.h
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         11.10.2006
//***************************************************************************
// History:
//   25.11.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************

#ifndef CMG_COMMONS_H
#define CMG_COMMONS_H


//---------------------------------------------------------------------------
// commomly used defines
//---------------------------------------------------------------------------

// set or clear a bit in val and save the result back to val
#define     SET_BIT(val,bit)            val |= ( 1 << (bit) )
#define     CLR_BIT(val,bit)            val &= ~( 1 << (bit) )

// set or clear a bit in val and return the result
#define     SETBIT(val,bit)             ( val | ( 1 << (bit) ) )
#define     CLRBIT(val,bit)             ( val & ( ~( 1 << (bit) ) ) )

// returns a value with the specified bit set
#define     BIT(bit)                    ( 1 << (bit) )

// get the upper or lower 8 bits form a word
#define     LO_BYTE(x)                  (cmg_u8)( (x) & ( (cmg_u8)0xff ) )
#define     HI_BYTE(x)                  (cmg_u8)( ( (x) >> 8 ) & ( (cmg_u8)0xff ) )

// build a word from two bytes
#define     BUILD_WORD(hi,lo)           ( ( ((cmg_u16)(hi)) << 8 ) | ((cmg_u8)(lo)) )

// necessary amount of bytes to store n bits
#define     GET_BYTES_FROM_BITS(bits)   ( ( ( (bits) - 1 ) >> 3 ) + 1 )

// varios sizes
#define     KB                          ( 1024 )
#define     MB                          ( 1024 * 1024 )

// memory usage
#define     SFREE(ptr)                  { if ( ptr ) { free( ptr ); ptr = NULL; } }

// checks and error handling
#define     CHECK(expr,error)           if ( expr ) { iResult = error; goto _Error; }
#define     CHECKRESULT()               if ( iResult != CMG_OK ) { goto _Error; }

// min / max / bound
#define     MIN(a,b)                    ( (a)<(b) ? (a) : (b) )
#define     MAX(a,b)                    ( (a)>(b) ? (a) : (b) )
#define     BOUND_MIN_MAX(var,lower,upper)  MIN( MAX( var, lower ), upper )

// swap
#define     SWAP(a,b,temp)              { (temp) = (a);   (a) = (b);   (b) = (temp); }
#define     SWAP_TYPE(a,b,type)         { type typeTemp;    SWAP( (a), (b), typeTemp ); }
//---------------------------------------------------------------------------



#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
