//***************************************************************************
// CMG_Types.h
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         11.10.2006
//***************************************************************************
// History:
//   ??.11.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************

#ifndef CMG_TYPES_H
#define CMG_TYPES_H


//---------------------------------------------------------------------------
// exact size types (must be exactly the size)
//---------------------------------------------------------------------------

// unsigned types
typedef     unsigned char           cmg_u8;
typedef     unsigned short          cmg_u16;
typedef     unsigned long           cmg_u32;

// signed types
typedef     signed char             cmg_s8;
typedef     signed short            cmg_s16;
typedef     signed long             cmg_s32;

//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// minimum size types (must be at least the size but can be larger to 
// improove performance)
//---------------------------------------------------------------------------

// unsigned types
typedef     cmg_u8                  cmg_mu8;
typedef     cmg_u16                 cmg_mu16;
typedef     cmg_u32                 cmg_mu32;

// signed types
typedef     cmg_s8                  cmg_ms8;
typedef     cmg_s16                 cmg_ms16;
typedef     cmg_s32                 cmg_ms32;

//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// other types
//---------------------------------------------------------------------------

typedef     void                    cmg_void;

typedef     cmg_u8                  cmg_bool;
#define     true                    1
#define     false                   0

typedef     char                    cmg_char;

typedef     cmg_ms16                cmg_Coord;      // all coordinates
typedef     cmg_ms32                cmg_CoordLong;  // double sized coordinate (i.e. for mul calculations)
typedef     cmg_u16                 cmg_ScrAddr;    // all controller screen addresses (must be u16)
typedef     cmg_u8                  cmg_Color;      // color (must be u8)


//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// fixed point types
//---------------------------------------------------------------------------
typedef     cmg_s32                 cmg_sfp1616;    // signed fixed point 16.16

#define     SFP_TO_INT(sfp)         ( (sfp) >> 16 ) // return int part of sfp
#define     SFP_ROUNDTO_INT(sfp)    ( ( (sfp) + 0x00007ff8l ) >> 16 ) // return rounded int part of sfp
#define     INT_TO_SFP(val)         ( ( (cmg_sfp1616)(val) ) << 16 )  // return sfp from int
#define     FLOAT_TO_SFP(val)       ( (cmg_sfp1616) ( ( (float)(val) ) * ( (float)(1l<<16) ) ) )  // return sfp from float
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// RESULT type
//---------------------------------------------------------------------------

typedef     cmg_mu8                 cmg_Result;

#define     CMG_OK                  0x00        // all ok
#define     CMG_ERROR_PARAM         0x01        // parameter error
#define     CMG_ERROR_NOMEM         0x02        // out of memory
#define     CMG_ERROR_OS            0x03        // operating system error (win32)
#define     CMG_ERROR_GTK           0x03        // GTK+ system error (GTK+)
#define     CMG_ERROR_OS_OR_GTK     0x03        // underlaying system error
#define     CMG_ERROR_NOTSUPPORTED  0x04        // ctrl specific fnc not supported

//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// defines
//---------------------------------------------------------------------------

// function modifiers
#define     _PUBLIC             
#define     _PROTECTED
#define     _PRIVATE                static

#define     NULL                    0


#ifdef WIN32
    // Mircosoft C Compiler
    #define     _INLINE_FORCE           _forceinline
    #define     _INLINE                 _inline
#else
    // GCC Compiler
    #define     _INLINE_FORCE           __attribute__((always_inline))
    #define     _INLINE                 inline
#endif


//---------------------------------------------------------------------------


#endif

/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
