//***************************************************************************
// CMG_PlatformDependent.c
//***************************************************************************
// CMG - Christian Merkle Graphics
// Copyright (C) 2oo6 - 2oo8 Christian Merkle 
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// To contact Christian Merkle, see his website at:
// http://www.cmerkle.de
//***************************************************************************
// Author:          Christian Merkle
// Created:         28.01.2006
//***************************************************************************
// History:
//   28.01.2006     *stable*
//***************************************************************************
// ToDo:
//   [ ]    xxx
//***************************************************************************


// main CMG include (will include user configuration)
#include    "../CMG_Main.h"



//---------------------------------------------------------------------------
// inludes
//---------------------------------------------------------------------------
// CMG:
#include "CMG_PlatformDependent.h"
// other:
//---------------------------------------------------------------------------




#if defined( CMG_PLATFORM_AVR )

    #include <util/delay.h>

    void CMG_Sleep_ms( cmg_u16 wDelay_ms )
    {
        for ( cmg_u16 wTime_ms = 0; wTime_ms < wDelay_ms; wTime_ms++ )  
            _delay_ms( 1.f );
    }


#elif defined( CMG_PLATFORM_ARM )

    void CMG_Sleep_ms( cmg_u16 wDelay_ms )
    {
        volatile long i = 0;
        for ( long x = 0; x < (wDelay_ms * 5450); x++ )  
            i++;
    }


#elif defined( CMG_PLATFORM_X86_WINDOWS )

    #include <windows.h>

    void CMG_Sleep_ms( cmg_u16 wDelay_ms )
    {
        Sleep( wDelay_ms );
    }


#elif defined( CMG_PLATFORM_X86_GTK )

    #include <glib.h>

    void CMG_Sleep_ms( cmg_u16 wDelay_ms )
    {
        g_usleep( ( (glong) wDelay_ms ) * 1000 );
    }


#endif








/////////////////////////////////////////////////////////////////////////////
// (c) 2oo6 - 2oo8 Christian Merkle
