
#define     CMG_PLATFORM_AVR
//#define     CMG_PLATFORM_ARM
//#define     CMG_PLATFORM_X86_GTK
//#define     CMG_PLATFORM_X86_WINDOWS


#ifdef WIN32
    #define     CMG_PLATFORM_X86_WINDOWS
    #define     CMG_PLATFORM_X86_GTK

    #undef      CMG_PLATFORM_AVR
    #undef      CMG_PLATFORM_ARM
#endif


#if defined( CMG_PLATFORM_X86_WINDOWS ) || defined( CMG_PLATFORM_X86_GTK )
    #define     CMG_PLATFORM_X86

    #undef      CMG_PLATFORM_AVR
    #undef      CMG_PLATFORM_ARM
#endif


#if defined( CMG_PLATFORM_X86 )

//############################################################################
// (1) CMG_LLIO - CONFIG
//############################################################################
//
// CMG_LLIO_EMU_*
// =====================
// Low Level Input Output for the LCD Emulator. 
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_LLIO_CONFIGURED
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Emulated Display details:
// --------------------------
// Use values defined by CTRL driver below. Use only own values if you know
// what you do:
#define     CMG_LLIO_EMU_WIDTH                      CMG_CTRL_WIDTH
#define     CMG_LLIO_EMU_HEIGHT                     CMG_CTRL_HEIGHT
#define     CMG_LLIO_EMU_BITDEPTH                   CMG_CTRL_BITDEPTH
//
//
// Driver emulator tu use:
// ------------------------
// You should use the same driver as selected in DRV.
// Use only _ONE_ of this list:
#define     CMG_LLIO_EMU_EMULATOR_T6963
//
//
// Display presentation system to use:
// ------------------------------------
// * Use Microsoft Windows GDI: (Compatible with all Windows versions):
// Use only _ONE_ of this list:
#define     CMG_LLIO_EMU_DISPLAY_GDI
//#define     CMG_LLIO_EMU_DISPLAY_GTK
//
//
//############################################################################
//############################################################################



#elif defined( CMG_PLATFORM_AVR )

//############################################################################
// (1) CMG_LLIO - CONFIG
//############################################################################
//
// CMG_LLIO_AVR_8D4CTL_8080
// ========================
// Low Level Input Output for Atmel AVR in 8080 mode. 
// Data: 8 bit; Control: /CS  A0  /RD  /WR
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_LLIO_CONFIGURED
#define     CMG_LLIO_AVR_8D4CTL_8080
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Port/Pin Assignments:
// =====================
//
// Initially wait time until display is ready in milliseconds:
#define     LLIO_DISPLAY_INIT_WAITTIME_MS   200
//
//
//
// Data-Port:
// ----------
// All 8 data pins of the display must be connected to an 8 bit port.
#define     LLIO_DATA_PORT                  PORTA
#define     LLIO_DATA_PIN                   PINA
#define     LLIO_DATA_PORT_DDR              DDRA
//
//
//
// Control-Pins:
// -------------
// Each control signal can be freely defined.
//
// * /CS:
#define     LLIO_CTRL_iCS_PORT              PORTC
#define     LLIO_CTRL_iCS_PORT_DDR          DDRC
#define     LLIO_CTRL_iCS_BIT               4
//
// * A0:
#define     LLIO_CTRL_A0_PORT               PORTC
#define     LLIO_CTRL_A0_PORT_DDR           DDRC
#define     LLIO_CTRL_A0_BIT                5
//
// * /RD:
#define     LLIO_CTRL_iRD_PORT              PORTC
#define     LLIO_CTRL_iRD_PORT_DDR          DDRC
#define     LLIO_CTRL_iRD_BIT               6
//
// * /WR:
#define     LLIO_CTRL_iWR_PORT              PORTC
#define     LLIO_CTRL_iWR_PORT_DDR          DDRC
#define     LLIO_CTRL_iWR_BIT               7
//
//
//
// Timing: Wait Cycles:
// -----------------------------
// Define here the amount of additional waitcycles (nops) when /WR or /RD
// getting low (active) until they get high again. Look for informations
// in the controller's datasheet:
#define     LLIO_WR_WAITCYCLES              2
#define     LLIO_RD_WAITCYCLES              10
//
//
//
// Display Power Supply Control:
// -----------------------------
// If the display is not always connected to power and can be switched 
// from an IO-pin define this:
#define     LLIO_DISPLAYPOWER_ENABLE
//
// If backlight is enabled configure the pins:
#define     LLIO_DISPLAYPOWER_PORT          PORTC
#define     LLIO_DISPLAYPOWER_PORT_DDR      DDRC
#define     LLIO_DISPLAYPOWER_BIT           2
//
// If the switching logic is negative on (inverted) define this:
#define     LLIO_DISPLAYPOWER_INVERTED
//
//
//
// Backlight Control:
// ------------------
// If a backlight is available and this backlight can be switched from an
// IO-pin define this:
#define     LLIO_BACKLIGHT_ENABLE
//
// If backlight is enabled configure the pins:
#define     LLIO_BACKLIGHT_PORT             PORTC
#define     LLIO_BACKLIGHT_PORT_DDR         DDRC
#define     LLIO_BACKLIGHT_BIT              3
//
// If the switching logic is negative on (inverted) define this:
//#define     LLIO_BACKLIGHT_INVERTED
//
// If the backlight should be initially on define this:
#define     LLIO_BACKLIGHT_INITIALLY_ON
//
//
//############################################################################
//############################################################################



#elif defined( CMG_PLATFORM_ARM )

//############################################################################
// (1) CMG_LLIO - CONFIG
//############################################################################
//
// CMG_LLIO_ARM_8D4CTL_8080
// ========================
// Low Level Input Output for ARM in 8080 mode. 
// Data: 8 bit; Control: /CS  A0  /RD  /WR
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_LLIO_CONFIGURED
#define     CMG_LLIO_ARM_8D4CTL_8080
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Port/Pin Assignments:
// =====================
//
// Initially wait time until display is ready in milliseconds:
#define     LLIO_DISPLAY_INIT_WAITTIME_MS   200
//
//
//
// Data-Port:
// ----------
// All 8 data pins of the display must be connected to an 8 bit port.
#define     LLIO_DATA_PORT_SET              IO0SET
#define     LLIO_DATA_PORT_CLR              IO0CLR
#define     LLIO_DATA_PORT_DIR              IO0DIR
#define     LLIO_DATA_PORT_PIN              IO0PIN
#define     LLIO_DATA_STARTBIT              8
//
//
//
// Control-Pins:
// -------------
// Each control signal can be freely defined.
//
// * /CS:
#define     LLIO_CTRL_iCS_PORT_SET          IO0SET
#define     LLIO_CTRL_iCS_PORT_CLR          IO0CLR
#define     LLIO_CTRL_iCS_PORT_DIR          IO0DIR
#define     LLIO_CTRL_iCS_BIT               16
//
// * A0:
#define     LLIO_CTRL_A0_PORT_SET           IO0SET
#define     LLIO_CTRL_A0_PORT_CLR           IO0CLR
#define     LLIO_CTRL_A0_PORT_DIR           IO0DIR
#define     LLIO_CTRL_A0_BIT                17
//
// * /RD:
#define     LLIO_CTRL_iRD_PORT_SET          IO0SET
#define     LLIO_CTRL_iRD_PORT_CLR          IO0CLR
#define     LLIO_CTRL_iRD_PORT_DIR          IO0DIR
#define     LLIO_CTRL_iRD_BIT               18
//
// * /WR:
#define     LLIO_CTRL_iWR_PORT_SET          IO0SET
#define     LLIO_CTRL_iWR_PORT_CLR          IO0CLR
#define     LLIO_CTRL_iWR_PORT_DIR          IO0DIR
#define     LLIO_CTRL_iWR_BIT               19
//
//
//
// Timing: Wait Cycles:
// -----------------------------
// Define here the amount of additional waitcycles (nops) when /WR or /RD
// getting low (active) until they get high again. Look for informations
// in the controller's datasheet:
#define     LLIO_WR_WAITCYCLES              15
#define     LLIO_RD_WAITCYCLES              31
//
//
//
// Display Power Supply Control:
// -----------------------------
// If the display is not always connected to power and can be switched 
// from an IO-pin define this:
#define     LLIO_DISPLAYPOWER_ENABLE
//
// If backlight is enabled configure the pins:
#define     LLIO_DISPLAYPOWER_PORT_SET      IO0SET
#define     LLIO_DISPLAYPOWER_PORT_CLR      IO0CLR
#define     LLIO_DISPLAYPOWER_PORT_DIR      IO0DIR
#define     LLIO_DISPLAYPOWER_BIT           23
//
// If the switching logic is negative on (inverted) define this:
#define     LLIO_DISPLAYPOWER_INVERTED
//
//
//
// Backlight Control:
// ------------------
// If a backlight is available and this backlight can be switched from an
// IO-pin define this:
#define     LLIO_BACKLIGHT_ENABLE
//
// If backlight is enabled configure the pins:
#define     LLIO_BACKLIGHT_PORT_SET         IO0SET
#define     LLIO_BACKLIGHT_PORT_CLR         IO0CLR
#define     LLIO_BACKLIGHT_PORT_DIR         IO0DIR
#define     LLIO_BACKLIGHT_BIT              22
//
// If the switching logic is negative on (inverted) define this:
//#define     LLIO_BACKLIGHT_INVERTED
//
// If the backlight should be initially on define this:
//#define     LLIO_BACKLIGHT_INITIALLY_ON
//
//
//############################################################################
//############################################################################


#else
#error No platform specified in config file.
#endif




//############################################################################
// (2) CMG_CTRL - CONFIG
//############################################################################
//
// CMG_CTRL_*
// =====================
// Display controller configuration. 
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_CTRL_CONFIGURED
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Select the used display controller.
// Use only _ONE_ of this list:
//
// * Toshiba T6963C (compatible with: CMG_DRV1_UNI_1BPP)
#define     CMG_CTRL_T6963
//
// * Sharp LH155 (compatible with: CMG_DRV1_UNI_1BPP)
//#define     CMG_CTRL_LH155
//
// * Epson S1D13700 (compatible with: CMG_DRV1_UNI_1BPP)
//#define     CMG_CTRL_S1D13700
// For the Epson S1D13700 you have to define the following values, too.
// You should find them in the controller's or display's datasheet.
// If graphics becomes unstable or there are streaks decrease the framerate.
// If screen flickers increase frame rate.
// The framerate should be around 60 Hz.
#define     CMG_CTRL_CONTROLLER_CLOCK_HZ          8000000
#define     CMG_CTRL_FRAMERATE                    60
//
//
//
// Controller config:
// =====================
// Configure the controller
#define     CMG_CTRL_WIDTH                          240
#define     CMG_CTRL_HEIGHT                         128
#define     CMG_CTRL_BITDEPTH                       1
//
//
//############################################################################
//############################################################################


//############################################################################
// (3) CMG_DRV - CONFIG
//############################################################################
//
// CMG_DRV_*
// =====================
// Display driver configuration. 
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_DRV_CONFIGURED
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Select one DRV interface.
// Check compatiblility with CTRL module (see CTRL module).
// Use only _ONE_ of this list:
//
// * univeral 1bpp
#define     CMG_DRV_UNI_1BPP
//
//
//############################################################################
//############################################################################



//############################################################################
// (4) CMG_DRAW - CONFIG
//############################################################################
//
// CMG_DRAW_*
// =====================
// DRAW configuration. 
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_DRAW_CONFIGURED
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to 
// your needs.
//
//
// Basic functions always included:
// ---------------------------------
// * Blt, Line, Pixel, Rectangle
//
//
// Define extra functions that you need: 
// --------------------------------------
#define     CMG_DRAW_ELLIPSE
#define     CMG_DRAW_GETSCREEN
#define     CMG_DRAW_LINELIST
#define     CMG_DRAW_LINEWIDTH
#define     CMG_DRAW_ROUNDEDRECTANGLE
#define     CMG_DRAW_SCROLL
#define     CMG_DRAW_TRIANGLE
//
//
//############################################################################
//############################################################################


//############################################################################
// (5) CMG_TEXT - CONFIG
//############################################################################
//
// CMG_TEXT_*
// =====================
// TEXT configuration.
//
//
// For internal correct compilation.
// *** DO NOT CHANGE! ***
#define     CMG_TEXT_CONFIGURED
//
//
//============================================================================
// User Area:
// ==========
// Read the sections carefully and change the values accordingly to
// your needs.
//
//
// Define to enable Text Rendering
// -------------------------------
#define   CMG_TEXT_ENABLED
//
//
//
// Select RawAccess mode for fonts
// -------------------------------
#define   CMG_TEXT_FONTRAWACCESS_RAM
// #define   CMG_TEXT_FONTRAWACCESS_AVRFLASH
//
//
//############################################################################
//############################################################################
